@@ -67,6 +67,9 @@ static Mutex s_callidMutex(false,"CallID");
 // this is to protect against two threads trying to (dis)connect a pair
 //  of call endpoints at the same time
 static Mutex s_mutex(true,"CallEndpoint");
+static const String s_audioType = "audio";
+static const String s_copyParams = "copyparams";
+
 
 CallEndpoint::CallEndpoint(const char* id)
     : m_peer(0), m_id(id), m_mutex(0)
@@ -223,17 +226,17 @@ String CallEndpoint::getPeerId() const
     return id;
 }
 
-DataEndpoint* CallEndpoint::getEndpoint(const char* type) const
+DataEndpoint* CallEndpoint::getEndpoint(const String& type) const
 {
-    if (null(type))
+    if (type.null())
 	return 0;
     const ObjList* pos = m_data.find(type);
     return pos ? static_cast<DataEndpoint*>(pos->get()) : 0;
 }
 
-DataEndpoint* CallEndpoint::setEndpoint(const char* type)
+DataEndpoint* CallEndpoint::setEndpoint(const String& type)
 {
-    if (null(type))
+    if (type.null())
 	return 0;
     DataEndpoint* dat = getEndpoint(type);
     if (!dat) {
@@ -259,9 +262,9 @@ void CallEndpoint::setEndpoint(DataEndpoint* endPoint)
 	endPoint->connect(m_peer->getEndpoint(endPoint->toString()));
 }
 
-void CallEndpoint::clearEndpoint(const char* type)
+void CallEndpoint::clearEndpoint(const String& type)
 {
-    if (null(type)) {
+    if (type.null()) {
 	ObjList* l = m_data.skipNull();
 	for (; l; l=l->skipNext()) {
 	    DataEndpoint* e = static_cast<DataEndpoint*>(l->get());
@@ -282,41 +285,46 @@ void CallEndpoint::clearEndpoint(const char* type)
     }
 }
 
-void CallEndpoint::setSource(DataSource* source, const char* type)
+void CallEndpoint::setSource(DataSource* source, const String& type)
 {
     DataEndpoint* dat = source ? setEndpoint(type) : getEndpoint(type);
     if (dat)
 	dat->setSource(source);
 }
 
-DataSource* CallEndpoint::getSource(const char* type) const
+DataSource* CallEndpoint::getSource(const String& type) const
 {
     DataEndpoint* dat = getEndpoint(type);
     return dat ? dat->getSource() : 0;
 }
 
-void CallEndpoint::setConsumer(DataConsumer* consumer, const char* type)
+void CallEndpoint::setConsumer(DataConsumer* consumer, const String& type)
 {
     DataEndpoint* dat = consumer ? setEndpoint(type) : getEndpoint(type);
     if (dat)
 	dat->setConsumer(consumer);
 }
 
-DataConsumer* CallEndpoint::getConsumer(const char* type) const
+DataConsumer* CallEndpoint::getConsumer(const String& type) const
 {
     DataEndpoint* dat = getEndpoint(type);
     return dat ? dat->getConsumer() : 0;
 }
 
-bool CallEndpoint::clearData(DataNode* node, const char* type)
+bool CallEndpoint::clearData(DataNode* node, const String& type)
 {
-    if (null(type) || !node)
+    if (type.null() || !node)
 	return false;
     Lock mylock(DataEndpoint::commonMutex());
     RefPointer<DataEndpoint> dat = getEndpoint(type);
     return dat && dat->clearData(node);
 }
 
+const String& CallEndpoint::audioType()
+{
+    return s_audioType;
+}
+
 
 static const String s_disconnected("chan.disconnected");
 
@@ -326,8 +334,8 @@ static Mutex s_paramMutex(true,"ChannelParams");
 Channel::Channel(Driver* driver, const char* id, bool outgoing)
     : CallEndpoint(id),
       m_parameters(""), m_driver(driver), m_outgoing(outgoing),
-      m_timeout(0), m_maxcall(0),
-      m_dtmfTime(0), m_dtmfSeq(0), m_answered(false)
+      m_timeout(0), m_maxcall(0), m_dtmfTime(0),
+      m_toutAns(0), m_dtmfSeq(0), m_answered(false)
 {
     init();
 }
@@ -335,8 +343,8 @@ Channel::Channel(Driver* driver, const char* id, bool outgoing)
 Channel::Channel(Driver& driver, const char* id, bool outgoing)
     : CallEndpoint(id),
       m_parameters(""), m_driver(&driver), m_outgoing(outgoing),
-      m_timeout(0), m_maxcall(0),
-      m_dtmfTime(0), m_dtmfSeq(0), m_answered(false)
+      m_timeout(0), m_maxcall(0), m_dtmfTime(0),
+      m_toutAns(0), m_dtmfSeq(0), m_answered(false)
 {
     init();
 }
@@ -418,6 +426,7 @@ void Channel::initChan()
     }
 #endif
     m_driver->m_total++;
+    m_driver->m_chanCount++;
     m_driver->channels().append(this);
     m_driver->changed();
 }
@@ -429,8 +438,11 @@ void Channel::dropChan()
     m_driver->lock();
     if (!m_driver)
 	Debug(DebugFail,"Driver lost in dropChan! [%p]",this);
-    if (m_driver->channels().remove(this,false))
+    if (m_driver->channels().remove(this,false)) {
+	if (m_driver->m_chanCount > 0)
+	    m_driver->m_chanCount--;
 	m_driver->changed();
+    }
     m_driver->unlock();
 }
 
@@ -517,8 +529,11 @@ void Channel::status(const char* newstat)
     Lock lock(mutex());
     m_status = newstat;
     if (!m_answered && (m_status == YSTRING("answered"))) {
-	m_maxcall = 0;
 	m_answered = true;
+	// stop pre-answer timeout, restart answered timeout
+	m_maxcall = 0;
+	if (m_toutAns)
+	    timeout(Time::now() + m_toutAns*(u_int64_t)1000);
     }
 }
 
@@ -527,19 +542,27 @@ const char* Channel::direction() const
     return m_outgoing ? "outgoing" : "incoming";
 }
 
-void Channel::setMaxcall(const Message* msg)
+void Channel::setMaxcall(const Message* msg, int defTout)
 {
-    int tout = msg ? msg->getIntValue(YSTRING("maxcall")) : 0;
-    if (tout > 0)
-	maxcall(Time::now() + tout*(u_int64_t)1000);
-    else
+    int tout = msg ? msg->getIntValue(YSTRING("timeout"),defTout) : defTout;
+    if (tout > 0) {
+	m_toutAns = tout;
+	timeout(Time::now() + tout*(u_int64_t)1000);
+    }
+    else if (tout == 0) {
+	m_toutAns = 0;
+	timeout(0);
+    }
+    if (m_answered)
 	maxcall(0);
-    if (msg) {
-	tout = msg->getIntValue(YSTRING("timeout"),-1);
-	if (tout > 0)
-	    timeout(Time::now() + tout*(u_int64_t)1000);
-	else if (tout == 0)
+    else if (msg) {
+	tout = msg->getIntValue(YSTRING("maxcall"),-1);
+	if (tout > 0) {
 	    timeout(0);
+	    maxcall(Time::now() + tout*(u_int64_t)1000);
+	}
+	else if (tout == 0)
+	    maxcall(0);
     }
 }
 
@@ -573,6 +596,7 @@ void Channel::complete(Message& msg, bool minimal) const
     if (m_lastPeerId)
 	msg.setParam("lastpeerid",m_lastPeerId);
     msg.setParam("answered",String::boolText(m_answered));
+    msg.setParam("direction",direction());
 }
 
 Message* Channel::message(const char* name, bool minimal, bool data)
@@ -589,7 +613,7 @@ Message* Channel::message(const char* name, const NamedList* original, const cha
     Message* msg = message(name,minimal,data);
     if (original) {
 	if (!params)
-	    params = original->getValue(YSTRING("copyparams"));
+	    params = original->getValue(s_copyParams);
 	if (!null(params))
 	    msg->copyParams(*original,params);
     }
@@ -634,8 +658,10 @@ bool Channel::msgRinging(Message& msg)
 bool Channel::msgAnswered(Message& msg)
 {
     m_maxcall = 0;
-    m_answered = true;
+    int tout = msg.getIntValue(YSTRING("timeout"),m_toutAns);
+    m_toutAns = (tout > 0) ? tout : 0;
     status("answered");
+    m_answered = true;
     if (m_billid.null())
 	m_billid = msg.getValue(YSTRING("billid"));
     return true;
@@ -739,14 +765,14 @@ void Channel::statusParams(String& str)
 	if (m_timeout) {
 	    str << ",timeout=";
 	    if (m_timeout > t)
-		str << (unsigned int)((m_timeout - t) / 1000);
+		str << (unsigned int)((m_timeout - t + 500) / 1000);
 	    else
 		str << "expired";
 	}
 	if (m_maxcall) {
 	    str << ",maxcall=";
 	    if (m_maxcall > t)
-		str << (unsigned int)((m_maxcall - t) / 1000);
+		str << (unsigned int)((m_maxcall - t + 500) / 1000);
 	    else
 		str << "expired";
 	}
@@ -782,9 +808,7 @@ bool Channel::callRouted(Message& msg)
 void Channel::callAccept(Message& msg)
 {
     status("accepted");
-    int tout = msg.getIntValue("timeout", m_driver ? m_driver->timeout() : 0);
-    if (tout > 0)
-	timeout(Time::now() + tout*(u_int64_t)1000);
+    setMaxcall(msg,m_driver ? m_driver->timeout() : 0);
     if (m_billid.null())
 	m_billid = msg.getValue(YSTRING("billid"));
     m_targetid = msg.getValue(YSTRING("targetid"));
@@ -821,6 +845,14 @@ void Channel::callConnect(Message& msg)
 void Channel::callRejected(const char* error, const char* reason, const Message* msg)
 {
     Debug(this,DebugMild,"Call rejected error='%s' reason='%s' [%p]",error,reason,this);
+    if (msg) {
+	const String* cp = msg->getParam(s_copyParams);
+	if (!TelEngine::null(cp)) {
+	    s_paramMutex.lock();
+	    parameters().copyParams(*msg,*cp);
+	    s_paramMutex.unlock();
+	}
+    }
     status("rejected");
 }
 
@@ -977,7 +1009,7 @@ bool Module::installRelay(int id, const char* name, unsigned priority)
 	return true;
     m_relays |= id;
 
-    MessageRelay* relay = new MessageRelay(name,this,id,priority);
+    MessageRelay* relay = new MessageRelay(name,this,id,priority,Module::name());
     m_relayList.append(relay)->setDelete(false);
     Engine::install(relay);
     return true;
@@ -1024,8 +1056,8 @@ bool Module::uninstallRelay(int id, bool delRelay)
 	    continue;
 	Engine::uninstall(r);
 	m_relays &= ~id;
-	if (delRelay)
-	    TelEngine::destruct(r);
+	l->remove(delRelay);
+	break;
     }
     return false;
 }
@@ -1222,7 +1254,7 @@ Driver::Driver(const char* name, const char* type)
       m_init(false), m_varchan(true),
       m_routing(0), m_routed(0), m_total(0),
       m_nextid(0), m_timeout(0),
-      m_maxroute(0), m_maxchans(0), m_dtmfDups(false)
+      m_maxroute(0), m_maxchans(0), m_chanCount(0), m_dtmfDups(false)
 {
     m_prefix << name << "/";
 }
@@ -1266,7 +1298,7 @@ void Driver::setup(const char* prefix, bool minimal)
 
 bool Driver::isBusy() const
 {
-    return (m_routing || m_chans.count());
+    return (m_routing || m_chanCount);
 }
 
 Channel* Driver::find(const String& id) const
@@ -1383,7 +1415,7 @@ bool Driver::received(Message &msg, int id)
 	    msg.userData(chan);
 	    if (chan->msgMasquerade(msg))
 		return true;
-	    chan->complete(msg);
+	    chan->complete(msg,msg.getBoolValue(YSTRING("complete_minimal"),false));
 	    return false;
 	case Locate:
 	    msg.userData(chan);
@@ -1418,10 +1450,8 @@ bool Driver::canAccept(bool routers)
 	return false;
     if (routers && !canRoute())
 	return false;
-    if (m_maxchans) {
-	Lock mylock(this);
-	return ((signed)m_chans.count() < m_maxchans);
-    }
+    if (m_maxchans)
+	return (m_chanCount < m_maxchans);
     return true;
 }
 
@@ -1461,7 +1491,7 @@ void Driver::genUpdate(Message& msg)
     msg.addParam("routed",String(m_routed));
     msg.addParam("routing",String(m_routing));
     msg.addParam("total",String(m_total));
-    msg.addParam("chans",String(m_chans.count()));
+    msg.addParam("chans",String(m_chanCount));
 }
 
 void Driver::statusModule(String& str)
@@ -1476,7 +1506,7 @@ void Driver::statusParams(String& str)
     str.append("routed=",",") << m_routed;
     str << ",routing=" << m_routing;
     str << ",total=" << m_total;
-    str << ",chans=" << m_chans.count();
+    str << ",chans=" << m_chanCount;
 }
 
 void Driver::statusDetail(String& str)
@@ -1587,6 +1617,12 @@ bool Router::route()
 		Debug(m_driver,DebugInfo,"Connection '%s' vanished while prerouting!",m_id.c_str());
 		return false;
 	    }
+	    const String* cp = m_msg->getParam(s_copyParams);
+	    if (!TelEngine::null(cp)) {
+		Channel::paramMutex().lock();
+		chan->parameters().copyParams(*m_msg,*cp);
+		Channel::paramMutex().unlock();
+	    }
 	    bool dropCall = ok && ((m_msg->retValue() == YSTRING("-")) || (m_msg->retValue() == YSTRING("error")));
 	    if (dropCall)
 		chan->callRejected(m_msg->getValue(YSTRING("error"),"unknown"),
@@ -1617,13 +1653,19 @@ bool Router::route()
     // chan will keep it referenced even if message user data is changed
     m_msg->userData(chan);
 
+    static const char s_noroute[] = "noroute";
+    static const char s_looping[] = "looping";
+    static const char s_noconn[] = "noconn";
+
     if (ok && m_msg->retValue().trimSpaces()) {
 	if ((m_msg->retValue() == YSTRING("-")) || (m_msg->retValue() == YSTRING("error")))
 	    chan->callRejected(m_msg->getValue(YSTRING("error"),"unknown"),
 		m_msg->getValue("reason"),m_msg);
-	else if (m_msg->getIntValue(YSTRING("antiloop"),1) <= 0)
-	    chan->callRejected(m_msg->getValue(YSTRING("error"),"looping"),
-		m_msg->getValue(YSTRING("reason"),"Call is looping"),m_msg);
+	else if (m_msg->getIntValue(YSTRING("antiloop"),1) <= 0) {
+	    const char* error = m_msg->getValue(YSTRING("error"),s_looping);
+	    chan->callRejected(error,m_msg->getValue(YSTRING("reason"),
+		((s_looping == error) ? "Call is looping" : (const char*)0)),m_msg);
+	}
 	else if (chan->callRouted(*m_msg)) {
 	    *m_msg = "call.execute";
 	    m_msg->setParam("callto",m_msg->retValue());
@@ -1633,9 +1675,13 @@ bool Router::route()
 	    if (ok)
 		chan->callAccept(*m_msg);
 	    else {
-		const char* error = m_msg->getValue(YSTRING("error"),"noconn");
-		const char* reason = m_msg->getValue(YSTRING("reason"),"Could not connect to target");
+		const char* error = m_msg->getValue(YSTRING("error"),s_noconn);
+		const char* reason = m_msg->getValue(YSTRING("reason"),
+		    ((s_noconn == error) ? "Could not connect to target" : (const char*)0));
 		Message m(s_disconnected);
+		const String* cp = m_msg->getParam(s_copyParams);
+		if (!TelEngine::null(cp))
+		    m.copyParams(*m_msg,*cp);
 		chan->complete(m);
 		m.setParam("error",error);
 		m.setParam("reason",reason);
@@ -1647,9 +1693,11 @@ bool Router::route()
 	    }
 	}
     }
-    else
-	chan->callRejected(m_msg->getValue(YSTRING("error"),"noroute"),
-	    m_msg->getValue(YSTRING("reason"),"No route to call target"),m_msg);
+    else {
+	const char* error = m_msg->getValue(YSTRING("error"),s_noroute);
+	chan->callRejected(error,m_msg->getValue(YSTRING("reason"),
+	    ((s_noroute == error) ? "No route to call target" : (const char*)0)),m_msg);
+    }
 
     // dereference again if the channel is dynamic
     if (m_driver->varchan())
