@@ -103,7 +103,9 @@ public:
 class EngineCommand : public MessageHandler
 {
 public:
-    EngineCommand() : MessageHandler("engine.command") { }
+    EngineCommand()
+	: MessageHandler("engine.command",100,"engine")
+	{ }
     virtual bool received(Message &msg);
     static void doCompletion(Message &msg, const String& partLine, const String& partWord);
 };
@@ -230,10 +232,11 @@ static Engine::PluginMode s_loadMode = Engine::LoadFail;
 static int s_maxworkers = 10;
 static bool s_debug = true;
 static bool s_capture = CAPTURE_EVENTS;
-static int s_maxevents = 10;
+static int s_maxevents = 25;
 static Mutex s_eventsMutex(false,"EventsList");
 static ObjList s_events;
 static String s_startMsg;
+static SharedVars s_vars;
 
 const TokenDict Engine::s_callAccept[] = {
     {"accept",      Engine::Accept},
@@ -290,7 +293,10 @@ private:
 class EngineSuperHandler : public MessageHandler
 {
 public:
-    EngineSuperHandler() : MessageHandler("engine.timer",0), m_seq(0) { }
+    EngineSuperHandler()
+	: MessageHandler("engine.timer",1,"engine"),
+	  m_seq(0)
+	{ }
     virtual bool received(Message &msg)
 	{ ::write(s_super_handle,&m_seq,1); m_seq++; return false; }
     char m_seq;
@@ -299,14 +305,18 @@ public:
 class EngineStatusHandler : public MessageHandler
 {
 public:
-    EngineStatusHandler() : MessageHandler("engine.status",0) { }
+    EngineStatusHandler()
+	: MessageHandler("engine.status",90,"engine")
+	{ }
     virtual bool received(Message &msg);
 };
 
 class EngineHelp : public MessageHandler
 {
 public:
-    EngineHelp() : MessageHandler("engine.help") { }
+    EngineHelp()
+	: MessageHandler("engine.help",100,"engine")
+	{ }
     virtual bool received(Message &msg);
 };
 
@@ -326,7 +336,9 @@ private:
 class EngineEventHandler : public MessageHandler
 {
 public:
-    EngineEventHandler() : MessageHandler("module.update",0) { }
+    EngineEventHandler()
+	: MessageHandler("module.update",1,"engine")
+	{ }
     virtual bool received(Message &msg);
 };
 
@@ -381,6 +393,13 @@ static void initUsrPath(String& path, const char* newPath = 0)
 	path = path.substr(0,path.length()-1);
 }
 
+// helper function to set up the config file name
+static void initCfgFile(const char* name)
+{
+    s_cfgfile = name;
+    if (s_cfgfile.endsWith(".exe") || s_cfgfile.endsWith(".EXE"))
+	s_cfgfile = s_cfgfile.substr(0,s_cfgfile.length()-4);
+}
 
 bool EngineStatusHandler::received(Message &msg)
 {
@@ -717,6 +736,8 @@ bool EngineHelp::received(Message &msg)
 	msg.retValue() << s_evtsOpt << s_evtsMsg;
     else if (line == YSTRING("logview"))
 	msg.retValue() << s_logvOpt << s_logvMsg;
+    else
+	return false;
     return true;
 }
 
@@ -1147,6 +1168,68 @@ bool SLib::unload(bool unloadNow)
 }
 
 
+void SharedVars::get(const String& name, String& rval)
+{
+    lock();
+    rval = m_vars.getValue(name,rval);
+    unlock();
+}
+
+void SharedVars::set(const String& name, const char* val)
+{
+    lock();
+    m_vars.setParam(name,val);
+    unlock();
+}
+
+bool SharedVars::create(const String& name, const char* val)
+{
+    Lock mylock(this);
+    if (m_vars.getParam(name))
+	return false;
+    m_vars.addParam(name,val);
+    return true;
+}
+
+void SharedVars::clear(const String& name)
+{
+    lock();
+    m_vars.clearParam(name);
+    unlock();
+}
+
+bool SharedVars::exists(const String& name)
+{
+    Lock mylock(this);
+    return m_vars.getParam(name) != 0;
+}
+
+unsigned int SharedVars::inc(const String& name, unsigned int wrap)
+{
+    Lock mylock(this);
+    unsigned int val = m_vars.getIntValue(name);
+    if (wrap)
+	val = val % (wrap + 1);
+    unsigned int nval = val + 1;
+    if (wrap)
+	nval = nval % (wrap + 1);
+    m_vars.setParam(name,String(nval));
+    return val;
+}
+
+unsigned int SharedVars::dec(const String& name, unsigned int wrap)
+{
+    Lock mylock(this);
+    unsigned int val = m_vars.getIntValue(name);
+    if (wrap)
+	val = val ? ((val - 1) % (wrap + 1)) : wrap;
+    else
+	val = val ? (val - 1) : 0;
+    m_vars.setParam(name,String(val));
+    return val;
+}
+
+
 Engine::Engine()
 {
     DDebug(DebugAll,"Engine::Engine() [%p]",this);
@@ -1189,6 +1272,13 @@ int Engine::engineInit()
     CapturedEvent::capturing(s_capture);
     if (s_capture && s_startMsg)
 	CapturedEvent::append(-1,s_startMsg);
+    String track = s_cfg.getValue("general","trackparam");
+    if (track.null() || track.toBoolean(false))
+	track = "handlers";
+    else if (!track.toBoolean(true))
+	track.clear();
+    if (track)
+	m_dispatcher.trackParam(track);
 #ifdef _WINDOWS
     int winTimerRes = s_cfg.getIntValue("general","wintimer");
     if ((winTimerRes > 0) && (winTimerRes < 100)) {
@@ -1254,6 +1344,8 @@ int Engine::engineInit()
 #endif
     s_params.addParam("maxworkers",String(s_maxworkers));
     s_params.addParam("maxevents",String(s_maxevents));
+    if (track)
+	s_params.addParam("trackparam",track);
 #ifdef _WINDOWS
     {
 	char buf[PATH_MAX];
@@ -1268,6 +1360,15 @@ int Engine::engineInit()
 	    s_params.addParam("workpath",buf);
     }
 #endif
+    NamedList* vars = s_cfg.getSection("variables");
+    if (vars) {
+	unsigned int n = vars->length();
+	for (unsigned int i = 0; i < n; i++) {
+	    NamedString* v = vars->getParam(i);
+	    if (v)
+		s_vars.set(v->name(),*v);
+	}
+    }
     DDebug(DebugAll,"Engine::run()");
     install(new EngineStatusHandler);
     install(new EngineEventHandler);
@@ -1786,6 +1887,11 @@ void Engine::clearEvents(const String& type)
 	s_events.remove(type);
 }
 
+SharedVars& Engine::sharedVars()
+{
+    return s_vars;
+}
+
 
 static void usage(bool client, FILE* f)
 {
@@ -1891,6 +1997,7 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 
     if (!cfgfile)
 	cfgfile = argv[0][0] ? argv[0] : "yate";
+    initCfgFile(cfgfile);
 
     int i;
     bool inopt = true;
@@ -1901,6 +2008,14 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 	    if (!::strncmp(pc,"-psn_",5))
 		continue;
 	    while (pc && *++pc) {
+		const char* param = 0;
+
+#define GET_PARAM \
+    if ('=' == pc[1]) param = pc + 2; \
+    else if (i+1 < argc) param = argv[++i]; \
+    else { noarg(client,argv[i]); return ENOENT; } \
+    pc = 0
+
 		switch (*pc) {
 		    case '-':
 			if (!*++pc) {
@@ -1965,84 +2080,44 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 			s_logtruncate = true;
 			break;
 		    case 'p':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			pidfile=argv[++i];
+			GET_PARAM;
+			pidfile = param;
 			break;
 		    case 'l':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			s_logfile=argv[++i];
+			GET_PARAM;
+			s_logfile = param;
 			break;
 		    case 'n':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			cfgfile=argv[++i];
+			GET_PARAM;
+			cfgfile = param;
 			break;
 		    case 'e':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			s_shrpath=argv[++i];
+			GET_PARAM;
+			s_shrpath = param;
 			break;
 		    case 'c':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			s_cfgpath=argv[++i];
+			GET_PARAM;
+			s_cfgpath = param;
 			break;
 		    case 'u':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			usrpath=argv[++i];
+			GET_PARAM;
+			usrpath = param;
 			break;
 		    case 'm':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			s_modpath=argv[++i];
+			GET_PARAM;
+			s_modpath = param;
 			break;
 		    case 'w':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			workdir = argv[++i];
+			GET_PARAM;
+			workdir = param;
 			break;
 		    case 'x':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			extraPath(argv[++i]);
+			GET_PARAM;
+			extraPath(param);
 			break;
 		    case 'N':
-			if (i+1 >= argc) {
-			    noarg(client,argv[i]);
-			    return ENOENT;
-			}
-			pc = 0;
-			s_node=argv[++i];
+			GET_PARAM;
+			s_node = param;
 			break;
 #ifdef RLIMIT_CORE
 		    case 'C':
@@ -2130,6 +2205,9 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 			badopt(client,*pc,argv[i]);
 			return EINVAL;
 		}
+
+#undef GET_PARAM
+
 	    }
 	}
 	else {
@@ -2144,10 +2222,7 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 
     s_mode = mode;
 
-    s_cfgfile = cfgfile;
-    if (s_cfgfile.endsWith(".exe") || s_cfgfile.endsWith(".EXE"))
-	s_cfgfile = s_cfgfile.substr(0,s_cfgfile.length()-4);
-
+    initCfgFile(cfgfile);
     initUsrPath(s_usrpath,usrpath);
 
     if (workdir && ::chdir(workdir)) {
