@@ -39,9 +39,10 @@ NamedList::NamedList(const char* name)
 NamedList::NamedList(const NamedList& original)
     : String(original)
 {
+    ObjList* dest = &m_params;
     for (const ObjList* l = original.m_params.skipNull(); l; l = l->skipNext()) {
 	const NamedString* p = static_cast<const NamedString*>(l->get());
-	m_params.append(new NamedString(p->name(),*p));
+	dest = dest->append(new NamedString(p->name(),*p));
     }
 }
 
@@ -109,7 +110,7 @@ NamedList& NamedList::setParam(const char* name, const char* value)
 
 NamedList& NamedList::clearParam(const String& name, char childSep)
 {
-    XDebug(DebugInfo,"NamedList::clearParam(\"%s\",'%1s')",
+    XDebug(DebugInfo,"NamedList::clearParam(\"%s\",'%.1s')",
 	name.c_str(),&childSep);
     String tmp;
     if (childSep)
@@ -139,7 +140,7 @@ NamedList& NamedList::clearParam(NamedString* param)
 
 NamedList& NamedList::copyParam(const NamedList& original, const String& name, char childSep)
 {
-    XDebug(DebugInfo,"NamedList::copyParam(%p,\"%s\",'%1s')",
+    XDebug(DebugInfo,"NamedList::copyParam(%p,\"%s\",'%.1s')",
 	&original,name.c_str(),&childSep);
     if (!childSep) {
 	// faster and simpler - used in most cases
@@ -149,10 +150,11 @@ NamedList& NamedList::copyParam(const NamedList& original, const String& name, c
     clearParam(name,childSep);
     String tmp;
     tmp << name << childSep;
+    ObjList* dest = &m_params;
     for (const ObjList* l = original.m_params.skipNull(); l; l = l->skipNext()) {
 	const NamedString* s = static_cast<const NamedString*>(l->get());
         if ((s->name() == name) || s->name().startsWith(tmp))
-	    addParam(s->name(),*s);
+	    dest = dest->append(new NamedString(s->name(),*s));
     }
     return *this;
 }
@@ -169,7 +171,7 @@ NamedList& NamedList::copyParams(const NamedList& original)
 
 NamedList& NamedList::copyParams(const NamedList& original, ObjList* list, char childSep)
 {
-    XDebug(DebugInfo,"NamedList::copyParams(%p,%p,'%1s') [%p]",
+    XDebug(DebugInfo,"NamedList::copyParams(%p,%p,'%.1s') [%p]",
 	&original,list,&childSep,this);
     for (; list; list = list->next()) {
 	GenObject* obj = list->get();
@@ -185,7 +187,7 @@ NamedList& NamedList::copyParams(const NamedList& original, ObjList* list, char
 
 NamedList& NamedList::copyParams(const NamedList& original, const String& list, char childSep)
 {
-    XDebug(DebugInfo,"NamedList::copyParams(%p,\"%s\",'%1s') [%p]",
+    XDebug(DebugInfo,"NamedList::copyParams(%p,\"%s\",'%.1s') [%p]",
 	&original,list.c_str(),&childSep,this);
     ObjList* l = list.split(',',false);
     if (l) {
@@ -201,18 +203,32 @@ NamedList& NamedList::copySubParams(const NamedList& original, const String& pre
 	&original,prefix.c_str(),String::boolText(skipPrefix),this);
     if (prefix) {
 	unsigned int offs = skipPrefix ? prefix.length() : 0;
+	ObjList* dest = &m_params;
 	for (const ObjList* l = original.m_params.skipNull(); l; l = l->skipNext()) {
 	    const NamedString* s = static_cast<const NamedString*>(l->get());
 	    if (s->name().startsWith(prefix)) {
 		const char* name = s->name().c_str() + offs;
 		if (*name)
-		    addParam(name,*s);
+		    dest = dest->append(new NamedString(name,*s));
 	    }
 	}
     }
     return *this;
 }
 
+bool NamedList::hasSubParams(const char* prefix) const
+{
+    XDebug(DebugInfo,"NamedList::hasSubParams(\"%s\") [%p]",prefix,this);
+    if (!TelEngine::null(prefix)) {
+	for (const ObjList* l = m_params.skipNull(); l; l = l->skipNext()) {
+	    const NamedString* s = static_cast<const NamedString*>(l->get());
+	    if (s->name().startsWith(prefix))
+		return true;
+	}
+    }
+    return false;
+}
+
 void NamedList::dump(String& str, const char* separator, char quote, bool force) const
 {
     if (force && str.null())
