@@ -219,22 +219,22 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	case IAXControl::Poke:
 	    break;
 	case IAXControl::Inval:
+	case IAXControl::FwDownl:
+	case IAXControl::TxCnt:
+	case IAXControl::TxAcc:
 	    // These are often used as keepalives
 	    return 0;
-	case IAXControl::FwDownl:
 	default:
-#ifdef DEBUG
 	    if (fullFrame) {
 	        if (fullFrame->destCallNo() == 0)
 		    Debug(this,DebugAll,"Unsupported incoming transaction Frame(%u,%u). Source call no: %u",
 			frame->type(),fullFrame->subclass(),fullFrame->sourceCallNo());
-#ifdef XDEBUG
 		else
 		    Debug(this,DebugAll,"Unmatched Frame(%u,%u) for (%u,%u)",
 			frame->type(),fullFrame->subclass(),fullFrame->destCallNo(),fullFrame->sourceCallNo());
-#endif
+
+		sendInval(fullFrame,addr);
 	    }
-#endif
 	    return 0;
     }
     // Generate local number
@@ -268,6 +268,31 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, const unsigned char*
     return tr;
 }
 
+// Find a complete transaction
+IAXTransaction* IAXEngine::findTransaction(const SocketAddr& addr, u_int16_t rCallNo)
+{
+    Lock lck(this);
+    ObjList* o = m_transList[rCallNo % m_transListCount]->skipNull();
+    for (; o; o = o->skipNext()) {
+	IAXTransaction* tr = static_cast<IAXTransaction*>(o->get());
+	if (tr->remoteCallNo() == rCallNo && addr == tr->remoteAddr())
+	    return tr->ref() ? tr : 0;
+    }
+    return 0;
+}
+
+void IAXEngine::sendInval(IAXFullFrame* frame, const SocketAddr& addr)
+{
+    if (!frame)
+	return;
+    DDebug(this,DebugInfo,"Sending INVAL for unmatched frame(%u,%u) with OSeq=%u ISeq=%u",frame->type(),frame->subclass(),
+	frame->oSeqNo(),frame->iSeqNo());
+    IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::Inval,frame->destCallNo(),
+	frame->sourceCallNo(),frame->iSeqNo(),frame->oSeqNo(),frame->timeStamp());
+    writeSocket(f->data().data(),f->data().length(),addr,f);
+    f->deref();
+}
+
 bool IAXEngine::process()
 {
     bool ok = false;
@@ -414,7 +439,7 @@ bool IAXEngine::processTrunkFrames(u_int32_t time)
 {
     Lock lock(&m_mutexTrunk);
     bool sent = false;
-    for (ObjList* l = m_trunkList.skipNull(); l; l = l->next()) {
+    for (ObjList* l = m_trunkList.skipNull(); l; l = l->skipNext()) {
 	IAXMetaTrunkFrame* frame = static_cast<IAXMetaTrunkFrame*>(l->get());
 	// Frame has mini frame(s) ?
 	if (!frame->timestamp())
