@@ -76,7 +76,8 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, IAXFullFrame* frame, u_int16_t
     m_expire(60),
     m_format(IAXFormat::Audio), m_formatVideo(IAXFormat::Video),
     m_capability(0), m_callToken(false),
-    m_trunkFrame(0)
+    m_trunkFrame(0),
+    m_trunkInOffsetTimeMs(0), m_trunkInLastTs(0), m_warnTrunkInTimestamp(true)
 {
     Debug(m_engine,DebugAll,"Transaction(%u,%u) incoming type=%u remote=%s:%d [%p]",
 	localCallNo(),remoteCallNo(),m_type,m_addr.host().c_str(),m_addr.port(),this);
@@ -139,7 +140,8 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno,
     m_expire(60),
     m_format(IAXFormat::Audio), m_formatVideo(IAXFormat::Video),
     m_capability(0), m_callToken(false),
-    m_trunkFrame(0)
+    m_trunkFrame(0),
+    m_trunkInOffsetTimeMs(0), m_trunkInLastTs(0), m_warnTrunkInTimestamp(true)
 {
     Debug(m_engine,DebugAll,"Transaction(%u,%u) outgoing type=%u remote=%s:%d [%p]",
 	localCallNo(),remoteCallNo(),m_type,m_addr.host().c_str(),m_addr.port(),this);
@@ -248,7 +250,8 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
     if (state() == Terminating) {
 	// Local terminate: Accept only Ack. Remote terminate: Accept none.
 	if (m_localReqEnd && frame->fullFrame()) {
-	    if (!(frame->type() == IAXFrame::IAX && frame->fullFrame()->subclass() == IAXControl::Ack))
+	     if (!(frame->type() == IAXFrame::IAX && (frame->fullFrame()->subclass() == IAXControl::Ack || 
+		frame->fullFrame()->subclass() == IAXControl::Inval || frame->fullFrame()->subclass() == IAXControl::VNAK)))
 		return 0;
 	}
 	else
@@ -277,10 +280,10 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
 	m_inDroppedFrames++;
 	return 0;
     }
-    bool fAck = frame->type() == IAXFrame::IAX && frame->fullFrame()->subclass() == IAXControl::Ack;
+    bool fAck = frame->type() == IAXFrame::IAX && 
+	(frame->fullFrame()->subclass() == IAXControl::Ack || frame->fullFrame()->subclass() == IAXControl::Inval);
     if (!fAck && !isFrameAcceptable(frame->fullFrame()))
 	return 0;
-    incrementSeqNo(frame->fullFrame(),true);
     // Video/Voice full frame: process data & format
     if (type() == New && (frame->type() == IAXFrame::Voice ||
 	frame->type() == IAXFrame::Video)) {
@@ -480,7 +483,7 @@ unsigned int IAXTransaction::sendMedia(const DataBlock& data, u_int32_t format,
     else
 	Debug(m_engine,DebugStub,
 	    "IAXTransaction::sendMedia() not implemented for type '%s'",fmt->typeName());
-    DDebug(m_engine,sent == data.length() ? DebugAll : DebugNote,
+    XDebug(m_engine,sent == data.length() ? DebugAll : DebugNote,
 	"Transaction(%u,%u) sent %u/%u media=%s mark=%u ts=%u [%p]",
 	localCallNo(),remoteCallNo(),sent,data.length(),
 	fmt->typeName(),mark,ts,this);
@@ -536,7 +539,7 @@ IAXEvent* IAXTransaction::getEvent(u_int64_t time)
 	if (state() == NewRemoteInvite_AuthSent && frame->ack())
 	    frame->adjustAuthTimeout(time + m_engine->authTimeout() * 1000);
 	// No response. Timeout ?
-	if (frame->timeout()) {
+	if (frame->timeout() && frame->timeForRetrans(time)) {
 	    if (m_state == Terminating)
 		// Client already notified: Terminate transaction
 		ev = terminate(IAXEvent::Timeout,true);
@@ -601,7 +604,7 @@ IAXEvent* IAXTransaction::getEvent(u_int64_t time)
     }
     // No pending outgoing frames. No valid requests. Clear incoming frames queue.
     //m_inDroppedFrames += m_inFrames.count();
-    m_inFrames.clear();
+    //m_inFrames.clear();
     return 0;
 }
 
@@ -837,6 +840,47 @@ void IAXTransaction::processCallToken(const DataBlock& callToken)
     sendFrame(frame);
 }
 
+// Update transaction incoming trunk data (used for trunk without timestamps)
+bool IAXTransaction::updateTrunkRecvTs(u_int32_t& frameTs, u_int32_t ts, u_int64_t currentTimeMs)
+{
+    Lock lck(this);
+    if (m_trunkInLastTs) {
+	bool ok = (ts > m_trunkInLastTs);
+	if (!ok) {
+	    // Allow frames older then 5 seconds to restart trunk
+	    if ((m_trunkInLastTs - ts) >= 5000) {
+		ok = true;
+		m_trunkInOffsetTimeMs = 0;
+	    }
+	    else
+		ok = (m_trunkInOffsetTimeMs == 0);
+	}
+	if (!ok) {
+	    if (m_warnTrunkInTimestamp) {
+		Debug(m_engine,DebugNote,
+		    "Transaction(%u,%u) ignoring trunked mini-frame without timestamps from %s:%d with ts=%u last=%u [%p]",
+		    localCallNo(),remoteCallNo(),remoteAddr().host().c_str(),remoteAddr().port(),
+		    ts,m_trunkInLastTs,this);
+		m_warnTrunkInTimestamp = false;
+	    }
+	    return false;
+	}
+	m_warnTrunkInTimestamp = true;
+    }
+    m_trunkInLastTs = ts;
+    if (!m_trunkInOffsetTimeMs) {
+	if (!currentTimeMs)
+	    currentTimeMs = Time::msecNow();
+	u_int64_t trunkStart = currentTimeMs - ts;
+	m_trunkInOffsetTimeMs = (int64_t)m_timeStamp - trunkStart;
+    }
+    if (m_trunkInOffsetTimeMs >= 0)
+	frameTs = ts - (u_int32_t)m_trunkInOffsetTimeMs;
+    else
+	frameTs = ts + (u_int32_t)(-m_trunkInOffsetTimeMs);
+    return true;
+}
+
 void IAXTransaction::print(bool printStats, bool printFrames, const char* location)
 {
     if (m_engine && !m_engine->debugAt(DebugAll))
@@ -940,8 +984,10 @@ bool IAXTransaction::incrementSeqNo(const IAXFullFrame* frame, bool inbound)
 bool IAXTransaction::isFrameAcceptable(const IAXFullFrame* frame)
 {
     int64_t delta = frame->oSeqNo() - m_iSeqNo;
-    if (!delta)
+    if (!delta) {
+	incrementSeqNo(frame,true);
 	return true;
+    }
     if (delta > 0) {
 	// We missed some frames before this one: Send VNAK
 	Debug(m_engine,DebugInfo,"Transaction(%u,%u). Received Frame(%u,%u) out of order! oseq=%u expecting %u. Send VNAK",
@@ -952,6 +998,8 @@ bool IAXTransaction::isFrameAcceptable(const IAXFullFrame* frame)
     }
     DDebug(m_engine,DebugInfo,"Transaction(%u,%u). Received late Frame(%u,%u) with oseq=%u expecting %u [%p]",
 	localCallNo(),remoteCallNo(),frame->type(),frame->subclass(),frame->oSeqNo(),m_iSeqNo,this);
+    if (frame->type() == IAXFrame::IAX && (frame->subclass() == IAXControl::Ping || frame->subclass() == IAXControl::Pong))
+	return true;
     sendAck(frame);	
     return false;
 }
@@ -1083,7 +1131,7 @@ IAXEvent* IAXTransaction::getEventResponse(IAXFrameOut* frame, bool& delFrame)
     if (findInFrameAck(frame)) {
 	frame->setAck();
 	// Terminating frame sent
-	if (m_state == Terminating)
+	if (m_state == Terminating && frame->type() == IAXFrame::IAX && frame->subclass() == IAXControl::Hangup)
 	    return terminate(IAXEvent::Terminated,true);
 	// Frame only need ACK
 	if (frame->ackOnly())
@@ -1399,6 +1447,7 @@ bool IAXTransaction::findInFrameTimestamp(const IAXFullFrame* frameOut, IAXFrame
 	    if (match)
 		break;
 	}
+	frame = 0;
     }
     if (frame) {
 	m_inFrames.remove(frame,true);
@@ -1409,10 +1458,12 @@ bool IAXTransaction::findInFrameTimestamp(const IAXFullFrame* frameOut, IAXFrame
 
 bool IAXTransaction::findInFrameAck(const IAXFullFrame* frameOut)
 {
+    if (frameOut && (frameOut->type() == IAXFrame::IAX && frameOut->subclass() == IAXControl::Ping))
+	return false;
     IAXFullFrame* frame = 0;
-    for (ObjList* l = m_inFrames.skipNull(); l; l = l->next()) {
+    for (ObjList* l = m_inFrames.skipNull(); l; l = l->skipNext()) {
 	frame = static_cast<IAXFullFrame*>(l->get());
-	if (frame && frame->type() == IAXFrame::IAX && frame->subclass() == IAXControl::Ack &&
+	if (frame->type() == IAXFrame::IAX && frame->subclass() == IAXControl::Ack &&
 	    frame->timeStamp() == frameOut->timeStamp() && frame->oSeqNo() == frameOut->iSeqNo())
 	    break;
 	frame = 0;
@@ -1429,7 +1480,8 @@ void IAXTransaction::ackInFrames()
     IAXFullFrame* ack = 0;
     for (ObjList* l = m_inFrames.skipNull(); l; l = l->next()) {
 	IAXFullFrame* frame = static_cast<IAXFullFrame*>(l->get());
-	if (frame && frame->type() == IAXFrame::IAX && frame->subclass() != IAXControl::Ack)
+	if (frame && !(frame->type() == IAXFrame::IAX && (frame->subclass() == IAXControl::Ack ||  frame->subclass() == IAXControl::Inval
+		|| frame->subclass() == IAXControl::LagRq || frame->subclass() ==  IAXControl::Ping)))
 	    ack = frame;
     }
     if (ack) {
@@ -1465,7 +1517,7 @@ void IAXTransaction::sendAck(const IAXFullFrame* frame)
 void IAXTransaction::sendInval()
 {
     IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::Inval,localCallNo(),
-	remoteCallNo(),m_oSeqNo++,m_iSeqNo,(u_int32_t)timeStamp());
+	remoteCallNo(),m_oSeqNo,m_iSeqNo,(u_int32_t)timeStamp());
     m_engine->writeSocket(f->data().data(),f->data().length(),remoteAddr(),f);
     f->deref();
 }
@@ -1516,6 +1568,10 @@ IAXEvent* IAXTransaction::processInternalIncomingRequest(const IAXFullFrame* fra
 	postFrame(IAXFrame::IAX,IAXControl::LagRp,0,0,frame->timeStamp(),true);
 	delFrame = true;
     }
+    if (frame->subclass() == IAXControl::Pong) {
+	sendAck(frame);
+	delFrame = true;
+    }
     return 0;
 }
 
@@ -1731,9 +1787,8 @@ void IAXTransaction::eventTerminated(IAXEvent* event)
 
 void IAXTransaction::adjustTStamp(u_int32_t& tStamp)
 {
-    if (tStamp)
-	return;
-    tStamp = (u_int32_t)timeStamp();
+    if (!tStamp)
+	tStamp = (u_int32_t)timeStamp();
     if (m_lastFullFrameOut) {
 	// adjust timestamp to be different from the last sent
 	int32_t delta = tStamp - m_lastFullFrameOut;
