@@ -721,6 +721,14 @@ public:
     static inline const char* typeName(int type)
 	{ return lookup(type,s_types); }
 
+    /**
+     * Get the text associated with a media type
+     * @param type The media type
+     * @return A string associated with the media type
+    */
+    static inline const String& typeNameStr(int type)
+	{ return s_typesList[type]; }
+
     /**
      * Keep the texts associated with the formats
     */
@@ -731,6 +739,11 @@ public:
      */
     static const TokenDict s_types[];
 
+    /**
+     * Keep the texts associated with a type also as String
+     */
+    static const String s_typesList[TypeCount];
+
 protected:
     int m_type;
     u_int32_t m_format;
@@ -1778,6 +1791,15 @@ public:
      */
     void processCallToken(const DataBlock& callToken);
 
+    /**
+     * Update transaction incoming trunk data (used for trunk without timestamps)
+     * @param frameTs Address of variable to be set with frame timestamp
+     * @param ts Trunk frame timestamp
+     * @param currentTimeMs Current time
+     * @return True if accepted
+     */
+    bool updateTrunkRecvTs(u_int32_t& frameTs, u_int32_t ts, u_int64_t currentTimeMs);
+
     /**
      * Print transaction data on stdin
      * @param printStats True to print media statistics
@@ -2223,6 +2245,9 @@ private:
     bool m_callToken;                           // Call token supported/expected
     // Meta trunking
     IAXMetaTrunkFrame* m_trunkFrame;		// Reference to a trunk frame if trunking is enabled for this transaction
+    int64_t m_trunkInOffsetTimeMs;		// Offset between transaction start and trunk start
+    u_int32_t m_trunkInLastTs;                  // Last received trunk timestamp
+    bool m_warnTrunkInTimestamp;                // Warn incoming trunk invalid timestamp
 };
 
 /**
@@ -2420,6 +2445,15 @@ public:
      */
     IAXTransaction* addFrame(const SocketAddr& addr, const unsigned char* buf, unsigned int len);
 
+    /**
+     * Find a complete transaction.
+     * This method is thread safe
+     * @param addr Remote address
+     * @param rCallNo Remote transaction call number
+     * @return Referrenced pointer to the transaction or 0
+     */
+    IAXTransaction* findTransaction(const SocketAddr& addr, u_int16_t rCallNo);
+
     /**
      * Process media from remote peer. Descendents must override this method
      * @param transaction IAXTransaction that owns the call leg
@@ -2602,6 +2636,13 @@ public:
      */
     void removeTrunkFrame(IAXMetaTrunkFrame* metaFrame);
 
+    /**
+     * Send an INVAL frame
+     * @param frame Frame for which to send an INVAL frame
+     * @param addr The address from where the call request was received
+     */
+    void sendInval(IAXFullFrame* frame, const SocketAddr& addr);
+
     /**
      * Send a trunk frame
      * @param metaFrame The trunk meta frame to sent
