@@ -31,7 +31,7 @@ using namespace TelEngine;
 static unsigned int fixValue(const NamedList& p, const char* param,
     unsigned int defVal, unsigned int min, unsigned int max, bool zero = false)
 {
-    unsigned int val = p.getIntValue(param);
+    unsigned int val = p.getIntValue(param,defVal);
     if (!val) {
 	if (!zero)
 	    val = defVal;
@@ -74,6 +74,8 @@ static const String s_entityCapsItem = "item";
 // Node values used by entity caps
 static const String s_googleTalkNode = "http://www.google.com/xmpp/client/caps";
 static const String s_googleMailNode = "http://mail.google.com/xmpp/client/caps";
+static const String s_googleAndroidNode = "http://www.android.com/gtalk/client/caps";
+static const String s_googleAndroidNode2 = "http://www.android.com/gtalk/client/caps2";
 
 // Stream read buffer
 #define JB_STREAMBUF                8192
@@ -102,9 +104,9 @@ static const String s_googleMailNode = "http://mail.google.com/xmpp/client/caps"
 #define JB_SRV_INTERVAL_MIN        10000
 #define JB_SRV_INTERVAL_MAX       120000
 // Ping
-#define JB_PING_INTERVAL          120000
+#define JB_PING_INTERVAL          600000
 #define JB_PING_INTERVAL_MIN       60000
-#define JB_PING_INTERVAL_MAX      600000
+#define JB_PING_INTERVAL_MAX     3600000
 #define JB_PING_TIMEOUT            30000
 #define JB_PING_TIMEOUT_MIN        10000
 #define JB_PING_TIMEOUT_MAX        JB_PING_INTERVAL_MIN
@@ -112,6 +114,11 @@ static const String s_googleMailNode = "http://mail.google.com/xmpp/client/caps"
 #define JB_IDLE_INTERVAL         3600000 // 1h
 #define JB_IDLE_INTERVAL_MIN      600000 // 10min
 #define JB_IDLE_INTERVAL_MAX    21600000 // 6h
+// Redirect
+#define JB_REDIRECT_COUNT              0
+#define JB_REDIRECT_COUNT_CLIENT       2
+#define JB_REDIRECT_MIN	               0
+#define JB_REDIRECT_MAX	              10
 
 
 /*
@@ -566,7 +573,20 @@ JBConnect::JBConnect(const JBStream& stream)
     m_engine(stream.engine()), m_stream(stream.toString()),
     m_streamType((JBStream::Type)stream.type())
 {
-    stream.connectAddr(m_address,m_port,m_localIp,m_status,m_srvs);
+    bool redir = false;
+    stream.connectAddr(m_address,m_port,m_localIp,m_status,m_srvs,&redir);
+    if (redir && m_address) {
+	char c = m_address[0];
+	if ((c < '0' || c > '9') && c != '[' && m_address[m_address.length() - 1] != ']') {
+	    // Redirect to domain: replace stream domain, clear address
+	    m_domain = m_address;
+	    m_address.clear();
+	}
+	else {
+	    // Redirect to IP address: clear stream domain
+	    m_domain.clear();
+	}
+    }
     if (m_engine)
 	m_engine->connectStatus(this,true);
 }
@@ -622,7 +642,7 @@ void JBConnect::connect()
 	}
 	advanceStatus();
     }
-    if (m_status == Srv) {
+    if (m_status == Srv && m_domain) {
 	if (!m_srvs.skipNull()) {
 	    // Get SRV records from remote party
 	    String query;
@@ -665,9 +685,11 @@ void JBConnect::connect()
 	}
 	advanceStatus();
     }
+    else if (m_status == Srv)
+	advanceStatus();
     if (m_status == Domain) {
 	// Try to resolve the domain
-	if (port)
+	if (port && m_domain)
 	    sock = connect(m_domain,port,stop);
 	advanceStatus();
     }
@@ -904,6 +926,7 @@ JBEngine::JBEngine(const char* name)
     m_pingInterval(JB_PING_INTERVAL), m_pingTimeout(JB_PING_TIMEOUT),
     m_idleTimeout(0), m_pptTimeoutC2s(0), m_pptTimeout(0),
     m_streamReadBuffer(JB_STREAMBUF), m_maxIncompleteXml(XMPP_MAX_INCOMPLETEXML),
+    m_redirectMax(JB_REDIRECT_COUNT),
     m_hasClientTls(true), m_printXml(0), m_initialized(false)
 {
     debugName(name);
@@ -951,11 +974,18 @@ void JBEngine::initialize(const NamedList& params)
     m_srvTimeout = fixValue(params,"stream_srvtimeout",
 	JB_SRV_INTERVAL,JB_SRV_INTERVAL_MIN,JB_SRV_INTERVAL_MAX);
     m_pingInterval = fixValue(params,"stream_pinginterval",
-	JB_PING_INTERVAL,JB_PING_INTERVAL_MIN,JB_PING_INTERVAL_MAX);
+	client ? JB_PING_INTERVAL : 0,JB_PING_INTERVAL_MIN,JB_PING_INTERVAL_MAX,true);
     m_pingTimeout = fixValue(params,"stream_pingtimeout",
-	JB_PING_TIMEOUT,JB_PING_TIMEOUT_MIN,JB_PING_TIMEOUT_MAX);
+	client ? JB_PING_TIMEOUT : 0,JB_PING_TIMEOUT_MIN,JB_PING_TIMEOUT_MAX,true);
+    if (!(m_pingInterval && m_pingTimeout))
+	m_pingInterval = m_pingTimeout = 0;
     m_idleTimeout = fixValue(params,"stream_idletimeout",
 	JB_IDLE_INTERVAL,JB_IDLE_INTERVAL_MIN,JB_IDLE_INTERVAL_MAX);
+    int defVal = JB_REDIRECT_COUNT;
+    if (client)
+	defVal = JB_REDIRECT_COUNT_CLIENT;
+    m_redirectMax = params.getIntValue("stream_redirectcount",
+	defVal,JB_REDIRECT_MIN,JB_REDIRECT_MAX);
     m_pptTimeoutC2s = params.getIntValue("stream_ppttimeout_c2s",10000,0,120000);
     m_pptTimeout = params.getIntValue("stream_ppttimeout",60000,0,180000);
     m_initialized = true;
@@ -2508,7 +2538,8 @@ bool JBEntityCapsList::processCaps(String& capsId, XmlElement* xml, JBStream* st
 	return true;
     // Hack for google (doesn't support disco info, supports only disco info with node)
     if (version == JBEntityCaps::Ver1_3 &&
-	(*node == s_googleTalkNode || *node == s_googleMailNode)) {
+	(*node == s_googleTalkNode || *node == s_googleMailNode ||
+	*node == s_googleAndroidNode || *node == s_googleAndroidNode2)) {
 	caps = new JBEntityCaps(capsId,version,*node,*ver);
 	if (ext) {
 	    ObjList* list = ext->split(' ',false);
