@@ -106,6 +106,7 @@ const String XMPPNamespace::s_array[Count] = {
     "jabber:x:data",                                       // XData
     "http://jabber.org/protocol/chatstates",               // ChatStates
     "http://yate.null.ro/yate/cluster",                    // YateCluster
+    "http://www.google.com/transport/raw-udp",             // JingleTransportGoogleRawUdp
 };
 
 const String XMPPError::s_array[Count] = {
@@ -774,6 +775,14 @@ XmlElement* XMPPUtils::createIqError(const char* from, const char* to, XmlElemen
     return iq;
 }
 
+// Create an 'iq' element with a ping child
+XmlElement* XMPPUtils::createPing(const char* id, const char* from, const char* to)
+{
+    XmlElement* iq = XMPPUtils::createIq(XMPPUtils::IqGet,from,to,id);
+    iq->addChild(XMPPUtils::createElement(XmlTag::Ping,XMPPNamespace::Ping));
+    return iq;
+}
+
 // Create an 'iq' element of type 'get' with a 'vcard' child
 XmlElement* XMPPUtils::createVCard(bool get, const char* from, const char* to, const char* id)
 {
@@ -845,11 +854,11 @@ XmlElement* XMPPUtils::createError(XmlElement* xml, int type, int error,
 }
 
 // Build a stream error element
-XmlElement* XMPPUtils::createStreamError(int error, const char* text)
+XmlElement* XMPPUtils::createStreamError(int error, const char* text, const char* content)
 {
     XmlElement* xml = createElement(XmlTag::Error);
     setStreamXmlns(*xml,false);
-    XmlElement* err = createElement(s_error[error],XMPPNamespace::StreamError);
+    XmlElement* err = createElement(s_error[error],XMPPNamespace::StreamError,content);
     xml->addChild(err);
     if (!TelEngine::null(text))
 	xml->addChild(createElement(XmlTag::Text,XMPPNamespace::StreamError,text));
@@ -929,7 +938,7 @@ XmlElement* XMPPUtils::findNextChild(const XmlElement& xml, XmlElement* start,
 }
 
 // Find an error child of a given element and return the associated code
-void XMPPUtils::decodeError(XmlElement* xml, int ns, String* error, String* text)
+void XMPPUtils::decodeError(XmlElement* xml, int ns, String* error, String* text, String* content)
 {
     if (!(xml && (error || text)))
 	return;
@@ -958,7 +967,9 @@ void XMPPUtils::decodeError(XmlElement* xml, int ns, String* error, String* text
 	    if (ch->unprefixedTag() == XMPPUtils::s_tag[XmlTag::Text])
 		continue;
 	    *error = ch->unprefixedTag();
-	    if (text) {
+	    if (content)
+		*content = ch->getText();
+	    else if (text) {
 		*text = ch->getText();
 		if (!TelEngine::null(text))
 		    return;
