@@ -190,6 +190,7 @@ public:
 	XData,                           // jabber:x:data
 	ChatStates,                      // http://jabber.org/protocol/chatstates
 	YateCluster,                     // http://yate.null.ro/yate/cluster
+	JingleTransportGoogleRawUdp,     // http://www.google.com/transport/raw-udp
 	// This value MUST be the last one: it's used as array bound
 	Count,
     };
@@ -1218,6 +1219,16 @@ public:
     static XmlElement* createIqError(const char* from, const char* to, XmlElement*& xml,
 	int type, int error, const char* text = 0);
 
+    /**
+     * Create an 'iq' element with a ping child
+     * @param id The 'id' attribute
+     * @param from The 'from' attribute
+     * @param to The 'to' attribute
+     * @return A valid XmlElement pointer
+     */
+    static XmlElement* createPing(const char* id = 0, const char* from = 0,
+	const char* to = 0);
+
     /**
      * Create an 'iq' element with a 'vcard' child
      * @param get True to set the iq's type to 'get', false to set it to 'set'
@@ -1290,9 +1301,10 @@ public:
      * Create a 'stream:error' element
      * @param error The XMPP defined condition
      * @param text Optional text to add to the error
+     * @param content Optional error condition element text
      * @return A valid XmlElement pointer
      */
-    static XmlElement* createStreamError(int error, const char* text = 0);
+    static XmlElement* createStreamError(int error, const char* text = 0, const char* content = 0);
 
     /**
      * Build a register query element
@@ -1520,9 +1532,10 @@ public:
      *  namespace if the element is a stream error or to stanza error namespace otherwise
      * @param error Optional string to be filled with error tag
      * @param text Optional string to be filled with error text
+     * @param content Optional string to be filled with error condition element text
      */
     static void decodeError(XmlElement* xml, int ns = XMPPNamespace::Count,
-	String* error = 0, String* text = 0);
+	String* error = 0, String* text = 0, String* content = 0);
 
     /**
      * Decode a stream error or stanza error
