@@ -726,10 +726,11 @@ public:
      * @param port The remote port
      * @param localip Local ip to bind
      * @param stat Current connect status
+     * @param isRedirect Optional pointer to be set to true if returned address is a redirect one
      * @param srvs List to copy stream SRV records
      */
     void connectAddr(String& addr, int& port, String& localip, int& stat,
-	ObjList& srvs) const;
+	ObjList& srvs, bool* isRedirect = 0) const;
 
     /**
      * Retrieve server host when connecting.
@@ -856,10 +857,12 @@ public:
      * @param error Termination reason. Set it to NoError to send stream end tag
      * @param reason Optional text to be added to the error stanza
      * @param final True if called from destructor
+     * @param genEvent True to generate terminated event
+     * @param content Optional sent error condition element text
      */
     void terminate(int location, bool destroy, XmlElement* xml,
 	int error = XMPPError::NoError, const char* reason = "",
-	bool final = false);
+	bool final = false, bool genEvent = true, const char* content = 0);
 
     /**
      * Outgoing stream connect terminated notification.
@@ -1013,6 +1016,13 @@ protected:
      */
     virtual void resetConnection(Socket* sock = 0);
 
+    /**
+     * Build a ping iq stanza
+     * @param stanzaId Stanza id 
+     * @return 0
+     */
+    virtual XmlElement* buildPing(const String& stanzaId);
+
     /**
      * Build a stream start XML element
      * @return XmlElement pointer
@@ -1199,6 +1209,28 @@ protected:
      */
     void setIdleTimer(u_int64_t msecNow = Time::msecNow());
 
+    /**
+     * Reset ping data
+     */
+    void resetPing();
+
+    /**
+     * Set the time of the next ping if there is any timeout and we don't have a ping in progress.
+     * Set the ping timeout if an element is returned
+     * @param force True to set it even if already set
+     * @return XmlElement containing the ping to send, 0 if no ping is going to be sent or 'force' is true
+     */
+    XmlElement* setNextPing(bool force);
+
+    /**
+     * Generate a stanza index from stream id and current stanza index
+     * Set the ping timeout if an element is returned
+     * @param buf Destination string
+     * @param extra Optional extra string
+     */
+    inline void generateIdIndex(String& buf, const char* extra = 0)
+	{ buf = id() + extra + String(++m_stanzaIndex);	}
+
     State m_state;                       // Stream state
     String m_id;                         // Stream id
     JabberID m_local;                    // Local peer's jid
@@ -1210,10 +1242,12 @@ protected:
     JBEvent* m_lastEvent;                // Last event generated by this stream
     ObjList m_events;                    // Queued events
     ObjList m_pending;                   // Pending outgoing elements
+    unsigned int m_stanzaIndex;          // Index used to generate IDs for stanzas
     // Timers
     u_int64_t m_setupTimeout;            // Overall stream setup timeout
     u_int64_t m_startTimeout;            // Incoming: wait stream start period
     u_int64_t m_pingTimeout;             // Sent ping timeout
+    u_int64_t m_pingInterval;            // Ping interval
     u_int64_t m_nextPing;                // Next ping
     u_int64_t m_idleTimeout;             // Stream idle timeout
     u_int64_t m_connectTimeout;          // Stream connect timeout
@@ -1268,6 +1302,8 @@ private:
     // Handle postponed termination. Return true if found
     // This method is not thread safe
     bool postponedTerminate();
+    // Reset redirect data
+    void setRedirect(const String& addr = String::empty(), int port = 0);
     // Reset postponed terminate data
     inline void resetPostponedTerminate() {
 	    m_ppTerminateTimeout = 0;
@@ -1344,6 +1380,10 @@ private:
     Compressor* m_compress;
     int m_connectStatus;                 // Current connect stream status
     ObjList m_connectSrvs;               // Current connect stream SRV records
+    unsigned int m_redirectMax;
+    unsigned int m_redirectCount;
+    String m_redirectAddr;
+    int m_redirectPort;
 };
 
 
@@ -1408,6 +1448,13 @@ public:
     virtual JBClientStream* clientStream()
 	{ return this; }
 
+    /**
+     * Build a ping iq stanza
+     * @param stanzaId Stanza id 
+     * @return Valid XmlElement pointer
+     */
+    virtual XmlElement* buildPing(const String& stanzaId);
+
     /**
      * Bind a resource to an incoming stream. This method should be called
      * after processing a Bind event
@@ -1931,6 +1978,13 @@ public:
 		XMPPError::Shutdown);
 	}
 
+    /**
+     * Retrieve maximum redirect counter for outgoing streams
+     * @return Maximum redirect counter for outgoing streams
+     */
+    inline unsigned int redirectMax() const
+	{ return m_redirectMax; }
+
     /**
      * Check if TLS is available for outgoing streams
      * @return True if TLS is available for outgoing streams
@@ -2193,6 +2247,7 @@ protected:
     unsigned int m_pptTimeout;           // Non client streams postpone stream termination intervals
     unsigned int m_streamReadBuffer;     // Stream read buffer length
     unsigned int m_maxIncompleteXml;     // Maximum length of an incomplete xml
+    unsigned int m_redirectMax;          // Max redirect counter for outgoing streams
     bool m_hasClientTls;                 // True if TLS is available for outgoing streams
     int m_printXml;                      // Print XML data to output
     bool m_initialized;                  // True if already initialized
