@@ -61,7 +61,6 @@ static const TokenDict s_operators_c[] =
     MAKEOP("&", And),
     MAKEOP("|", Or),
     MAKEOP("^", Xor),
-    MAKEOP(".", Cat),
     MAKEOP("@", As),
     MAKEOP("=", Assign),
     { 0, 0 }
@@ -146,12 +145,14 @@ bool ExpExtender::runAssign(ObjList& stack, const ExpOperation& oper, GenObject*
 
 
 ExpEvaluator::ExpEvaluator(const TokenDict* operators, const TokenDict* unaryOps)
-    : m_operators(operators), m_unaryOps(unaryOps), m_inError(false), m_extender(0)
+    : m_operators(operators), m_unaryOps(unaryOps),
+      m_inError(false), m_lineNo(1), m_extender(0)
 {
 }
 
 ExpEvaluator::ExpEvaluator(ExpEvaluator::Parser style)
-    : m_operators(0), m_unaryOps(0), m_inError(false), m_extender(0)
+    : m_operators(0), m_unaryOps(0),
+    m_inError(false), m_lineNo(1), m_extender(0)
 {
     switch (style) {
 	case C:
@@ -167,7 +168,7 @@ ExpEvaluator::ExpEvaluator(ExpEvaluator::Parser style)
 
 ExpEvaluator::ExpEvaluator(const ExpEvaluator& original)
     : m_operators(original.m_operators), m_unaryOps(original.unaryOps()),
-      m_inError(false), m_extender(0)
+      m_inError(false), m_lineNo(original.lineNumber()), m_extender(0)
 {
     extender(original.extender());
     for (ObjList* l = original.m_opcodes.skipNull(); l; l = l->skipNext()) {
@@ -197,9 +198,26 @@ char ExpEvaluator::skipWhites(const char*& expr)
 {
     if (!expr)
 	return 0;
-    while (*expr==' ' || *expr=='\t' || *expr=='\r' || *expr=='\n')
-	expr++;
-    return *expr;
+    for (; ; expr++) {
+	char c = *expr;
+	switch (c) {
+	    case ' ':
+	    case '\t':
+		continue;
+	    case '\r':
+		m_lineNo++;
+		if (expr[1] == '\n')
+		    expr++;
+		continue;
+	    case '\n':
+		m_lineNo++;
+		if (expr[1] == '\r')
+		    expr++;
+		continue;
+	    default:
+		return c;
+	}
+    }
 }
 
 bool ExpEvaluator::keywordChar(char c) const
@@ -208,7 +226,7 @@ bool ExpEvaluator::keywordChar(char c) const
 	('0' <= c && c <= '9') || (c == '_');
 }
 
-char ExpEvaluator::skipComments(const char*& expr, GenObject* context) const
+char ExpEvaluator::skipComments(const char*& expr, GenObject* context)
 {
     return skipWhites(expr);
 }
@@ -220,8 +238,7 @@ int ExpEvaluator::preProcess(const char*& expr, GenObject* context)
 
 ExpEvaluator::Opcode ExpEvaluator::getOperator(const char*& expr, const TokenDict* operators, bool caseInsensitive) const
 {
-    XDebug(this,DebugAll,"getOperator('%s',%p,%s)",expr,operators,String::boolText(caseInsensitive));
-    skipComments(expr);
+    XDebug(this,DebugAll,"getOperator('%.30s',%p,%s)",expr,operators,String::boolText(caseInsensitive));
     if (operators) {
 	bool kw = keywordChar(*expr);
 	for (const TokenDict* o = operators; o->token; o++) {
@@ -240,26 +257,35 @@ ExpEvaluator::Opcode ExpEvaluator::getOperator(const char*& expr, const TokenDic
     return OpcNone;
 }
 
-bool ExpEvaluator::gotError(const char* error, const char* text) const
+bool ExpEvaluator::gotError(const char* error, const char* text, unsigned int line) const
 {
     if (!error) {
 	if (!text)
 	    return false;
 	error = "unknown error";
     }
-    Debug(this,DebugWarn,"Evaluator error: %s%s%s",error,
-	(text ? " at: " : ""),
-	c_safe(text));
+    if (!line)
+	line = lineNumber();
+    String lineNo;
+    formatLineNo(lineNo,line);
+    Debug(this,DebugWarn,"Evaluator error: %s in %s%s%.50s",error,
+	lineNo.c_str(),(text ? " at: " : ""),c_safe(text));
     return false;
 }
 
-bool ExpEvaluator::gotError(const char* error, const char* text)
+bool ExpEvaluator::gotError(const char* error, const char* text, unsigned int line)
 {
     m_inError = true;
     return const_cast<const ExpEvaluator*>(this)->gotError(error,text);
 }
 
-bool ExpEvaluator::getInstruction(const char*& expr, Opcode nested)
+void ExpEvaluator::formatLineNo(String& buf, unsigned int line) const
+{
+    buf.clear();
+    buf << "line " << line;
+}
+
+bool ExpEvaluator::getInstruction(const char*& expr, char stop, GenObject* nested)
 {
     return false;
 }
@@ -268,7 +294,7 @@ bool ExpEvaluator::getOperand(const char*& expr, bool endOk)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getOperand '%s'",expr);
+    XDebug(this,DebugAll,"getOperand '%.30s'",expr);
     char c = skipComments(expr);
     if (!c)
 	// end of string
@@ -289,16 +315,21 @@ bool ExpEvaluator::getOperand(const char*& expr, bool endOk)
 	addOpcode(op);
 	return true;
     }
-    if (getString(expr) || getNumber(expr) || getFunction(expr) || getField(expr))
+    if (getSimple(expr) || getFunction(expr) || getField(expr))
 	return true;
     return gotError("Expecting operand",expr);
 }
 
+bool ExpEvaluator::getSimple(const char*& expr, bool constOnly)
+{
+    return getString(expr) || getNumber(expr);
+}
+
 bool ExpEvaluator::getNumber(const char*& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getNumber '%s'",expr);
+    XDebug(this,DebugAll,"getNumber '%.30s'",expr);
     char* endp = 0;
     long int val = ::strtol(expr,&endp,0);
     if (!endp || (endp == expr))
@@ -313,25 +344,70 @@ bool ExpEvaluator::getString(const char*& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getString '%s'",expr);
+    XDebug(this,DebugAll,"getString '%.30s'",expr);
     char c = skipComments(expr);
     if (c == '"' || c == '\'') {
-	char sep = c;
-	const char* start = ++expr;
-	while ((c = *expr++)) {
-	    if (c != sep)
-		continue;
-	    String str(start,expr-start-1);
-	    DDebug(this,DebugAll,"Found '%s'",str.safe());
+	String str;
+	if (getString(expr,str)) {
 	    addOpcode(str);
 	    return true;
 	}
-	expr--;
-	return gotError("Expecting string end");
     }
     return false;
 }
 
+bool ExpEvaluator::getString(const char*& expr, String& str)
+{
+    char sep = *expr++;
+    const char* start = expr;
+    while (char c = *expr++) {
+	if (c != '\\' && c != sep)
+	    continue;
+	String tmp(start,expr-start-1);
+	str += tmp;
+	if (c == sep) {
+	    DDebug(this,DebugAll,"Found '%s'",str.safe());
+	    return true;
+	}
+	tmp.clear();
+	if (!getEscape(expr,tmp,sep))
+	    break;
+	str += tmp;
+	start = expr;
+    }
+    expr--;
+    return gotError("Expecting string end");
+}
+
+bool ExpEvaluator::getEscape(const char*& expr, String& str, char sep)
+{
+    char c = *expr++;
+    switch (c) {
+	case '\0':
+	    return false;
+	case 'b':
+	    c = '\b';
+	    break;
+	case 'f':
+	    c = '\f';
+	    break;
+	case 'n':
+	    c = '\n';
+	    break;
+	case 'r':
+	    c = '\r';
+	    break;
+	case 't':
+	    c = '\t';
+	    break;
+	case 'v':
+	    c = '\v';
+	    break;
+    }
+    str = c;
+    return true;
+}
+
 int ExpEvaluator::getKeyword(const char* str) const
 {
     int len = 0;
@@ -347,7 +423,7 @@ bool ExpEvaluator::getFunction(const char*& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getFunction '%s'",expr);
+    XDebug(this,DebugAll,"getFunction '%.30s'",expr);
     skipComments(expr);
     int len = getKeyword(expr);
     const char* s = expr+len;
@@ -378,7 +454,7 @@ bool ExpEvaluator::getField(const char*& expr)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"getField '%s'",expr);
+    XDebug(this,DebugAll,"getField '%.30s'",expr);
     skipComments(expr);
     int len = getKeyword(expr);
     if (len <= 0)
@@ -394,11 +470,13 @@ bool ExpEvaluator::getField(const char*& expr)
 
 ExpEvaluator::Opcode ExpEvaluator::getOperator(const char*& expr)
 {
+    skipComments(expr);
     return getOperator(expr,m_operators);
 }
 
 ExpEvaluator::Opcode ExpEvaluator::getUnaryOperator(const char*& expr)
 {
+    skipComments(expr);
     return getOperator(expr,m_unaryOps);
 }
 
@@ -484,7 +562,19 @@ bool ExpEvaluator::getSeparator(const char*& expr, bool remove)
     return true;
 }
 
-bool ExpEvaluator::runCompile(const char*& expr, char stop, Opcode nested)
+bool ExpEvaluator::runCompile(const char*& expr, char stop, GenObject* nested)
+{
+    char buf[2];
+    const char* stopStr = 0;
+    if (stop) {
+	buf[0] = stop;
+	buf[1] = '\0';
+	stopStr = buf;
+    }
+    return runCompile(expr,stopStr,nested);
+}
+
+bool ExpEvaluator::runCompile(const char*& expr, const char* stop, GenObject* nested)
 {
     typedef struct {
 	Opcode code;
@@ -492,7 +582,9 @@ bool ExpEvaluator::runCompile(const char*& expr, char stop, Opcode nested)
     } StackedOpcode;
     StackedOpcode stack[10];
     unsigned int stackPos = 0;
-    DDebug(this,DebugInfo,"runCompile '%s' '%1s'",expr,&stop);
+#ifdef DEBUG
+    Debugger debug(DebugInfo,"runCompile()"," '%s' %p '%.30s'",TelEngine::c_safe(stop),nested,expr);
+#endif
     if (skipComments(expr) == ')')
 	return false;
     m_inError = false;
@@ -501,12 +593,14 @@ bool ExpEvaluator::runCompile(const char*& expr, char stop, Opcode nested)
 	addOpcode(OpcField,"*");
 	return true;
     }
+    char stopChar = stop ? stop[0] : '\0';
     for (;;) {
-	while (skipComments(expr) && getInstruction(expr,nested))
+	while (!stackPos && skipComments(expr) && (!stop || !::strchr(stop,*expr)) && getInstruction(expr,stopChar,nested))
 	    ;
 	if (inError())
 	    return false;
-	if (stop && (skipComments(expr) == stop))
+	char c = skipComments(expr);
+	if (c && stop && ::strchr(stop,c))
 	    return true;
 	if (!getOperand(expr))
 	    return false;
@@ -515,14 +609,15 @@ bool ExpEvaluator::runCompile(const char*& expr, char stop, Opcode nested)
 	    addOpcode(oper);
 	if (inError())
 	    return false;
-	char c = skipComments(expr);
-	if (!c || c == stop || getSeparator(expr,false)) {
+	c = skipComments(expr);
+	if (!c || (stop && ::strchr(stop,c)) || getSeparator(expr,false)) {
 	    while (stackPos)
 		addOpcode(stack[--stackPos].code);
 	    return true;
 	}
 	if (inError())
 	    return false;
+	skipComments(expr);
 	oper = getOperator(expr);
 	if (oper == OpcNone)
 	    return gotError("Operator or separator expected",expr);
@@ -545,9 +640,15 @@ bool ExpEvaluator::trySimplify()
 {
     DDebug(this,DebugInfo,"trySimplify");
     bool done = false;
-    for (unsigned int i = 0; i < m_opcodes.length(); i++) {
+    for (unsigned int i = 0; ; i++) {
 	ExpOperation* o = static_cast<ExpOperation*>(m_opcodes[i]);
-	if (!o || o->barrier())
+	if (!o) {
+	    if (i >= m_opcodes.length())
+		break;
+	    else
+		continue;
+	}
+	if (o->barrier())
 	    continue;
 	switch (o->opcode()) {
 	    case OpcLAnd:
@@ -578,10 +679,9 @@ bool ExpEvaluator::trySimplify()
 		    if (o->opcode() == OpcLAnd || o->opcode() == OpcAnd || o->opcode() == OpcMul) {
 			if ((op1->opcode() == OpcPush && !op1->number() && op2->opcode() == OpcField) ||
 			    (op2->opcode() == OpcPush && !op2->number() && op1->opcode() == OpcField)) {
-			    if (o->opcode() == OpcLAnd)
-				(m_opcodes+i)->set(new ExpOperation(false));
-			    else
-				(m_opcodes+i)->set(new ExpOperation((long int)0));
+			    ExpOperation* newOp = (o->opcode() == OpcLAnd) ? new ExpOperation(false) : new ExpOperation((long int)0);
+			    newOp->lineNumber(o->lineNumber());
+			    (m_opcodes+i)->set(newOp);
 			    m_opcodes.remove(op1);
 			    m_opcodes.remove(op2);
 			    i -= 2;
@@ -592,7 +692,9 @@ bool ExpEvaluator::trySimplify()
 		    if (o->opcode() == OpcLOr) {
 			if ((op1->opcode() == OpcPush && op1->number() && op2->opcode() == OpcField) ||
 			    (op2->opcode() == OpcPush && op2->number() && op1->opcode() == OpcField)) {
-			    (m_opcodes+i)->set(new ExpOperation(true));
+			    ExpOperation* newOp = new ExpOperation(true);
+			    newOp->lineNumber(o->lineNumber());
+			    (m_opcodes+i)->set(newOp);
 			    m_opcodes.remove(op1);
 			    m_opcodes.remove(op2);
 			    i -= 2;
@@ -606,7 +708,9 @@ bool ExpEvaluator::trySimplify()
 			pushOne(stack,op2->clone());
 			if (runOperation(stack,*o)) {
 			    // replace operators and operation with computed constant
-			    (m_opcodes+i)->set(popOne(stack));
+			    ExpOperation* newOp = popOne(stack);
+			    newOp->lineNumber(o->lineNumber());
+			    (m_opcodes+i)->set(newOp);
 			    m_opcodes.remove(op1);
 			    m_opcodes.remove(op2);
 			    i -= 2;
@@ -627,7 +731,9 @@ bool ExpEvaluator::trySimplify()
 			pushOne(stack,op->clone());
 			if (runOperation(stack,*o)) {
 			    // replace unary operator and operation with computed constant
-			    (m_opcodes+i)->set(popOne(stack));
+			    ExpOperation* newOp = popOne(stack);
+			    newOp->lineNumber(o->lineNumber());
+			    (m_opcodes+i)->set(newOp);
 			    m_opcodes.remove(op);
 			    i--;
 			    done = true;
@@ -649,9 +755,20 @@ bool ExpEvaluator::trySimplify()
     return done;
 }
 
+void ExpEvaluator::addOpcode(ExpOperation* oper, unsigned int line)
+{
+    if (!oper)
+	return;
+    DDebug(this,DebugAll,"addOpcode %u (%s), %u",oper->opcode(),getOperator(oper->opcode()),line);
+    if (!line)
+	line = lineNumber();
+    oper->lineNumber(line);
+    m_opcodes.append(oper);
+}
+
 ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, bool barrier)
 {
-    DDebug(this,DebugAll,"addOpcode %u",oper);
+    DDebug(this,DebugAll,"addOpcode %u (%s)",oper,getOperator(oper));
     if (oper == OpcAs) {
 	// the second operand is used just for the field name
 	ExpOperation* o = 0;
@@ -663,22 +780,25 @@ ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, bool barrier)
 	}
     }
     ExpOperation* op = new ExpOperation(oper,0,ExpOperation::nonInteger(),barrier);
+    op->lineNumber(lineNumber());
     m_opcodes.append(op);
     return op;
 }
 
 ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, long int value, bool barrier)
 {
-    DDebug(this,DebugAll,"addOpcode %u %lu",oper,value);
+    DDebug(this,DebugAll,"addOpcode %u (%s) %lu",oper,getOperator(oper),value);
     ExpOperation* op = new ExpOperation(oper,0,value,barrier);
+    op->lineNumber(lineNumber());
     m_opcodes.append(op);
     return op;
 }
 
 ExpOperation* ExpEvaluator::addOpcode(ExpEvaluator::Opcode oper, const String& name, long int value, bool barrier)
 {
-    DDebug(this,DebugAll,"addOpcode %u '%s' %ld",oper,name.c_str(),value);
+    DDebug(this,DebugAll,"addOpcode %u (%s) '%s' %ld",oper,getOperator(oper),name.c_str(),value);
     ExpOperation* op = new ExpOperation(oper,name,value,barrier);
+    op->lineNumber(lineNumber());
     m_opcodes.append(op);
     return op;
 }
@@ -687,6 +807,7 @@ ExpOperation* ExpEvaluator::addOpcode(const String& value)
 {
     DDebug(this,DebugAll,"addOpcode ='%s'",value.c_str());
     ExpOperation* op = new ExpOperation(value);
+    op->lineNumber(lineNumber());
     m_opcodes.append(op);
     return op;
 }
@@ -695,6 +816,7 @@ ExpOperation* ExpEvaluator::addOpcode(long int value)
 {
     DDebug(this,DebugAll,"addOpcode =%ld",value);
     ExpOperation* op = new ExpOperation(value);
+    op->lineNumber(lineNumber());
     m_opcodes.append(op);
     return op;
 }
@@ -703,10 +825,32 @@ ExpOperation* ExpEvaluator::addOpcode(bool value)
 {
     DDebug(this,DebugAll,"addOpcode =%s",String::boolText(value));
     ExpOperation* op = new ExpOperation(value);
+    op->lineNumber(lineNumber());
     m_opcodes.append(op);
     return op;
 }
 
+ExpOperation* ExpEvaluator::popOpcode()
+{
+    ObjList* l = &m_opcodes;
+    for (ObjList* p = l; p; p = p->next()) {
+	if (p->get())
+	    l = p;
+    }
+    return static_cast<ExpOperation*>(l->remove(false));
+}
+
+unsigned int ExpEvaluator::getLineOf(ExpOperation* op1, ExpOperation* op2, ExpOperation* op3)
+{
+    if (op1 && op1->lineNumber())
+	return op1->lineNumber();
+    if (op2 && op2->lineNumber())
+	return op2->lineNumber();
+    if (op3 && op3->lineNumber())
+	return op3->lineNumber();
+    return 0;
+}
+
 void ExpEvaluator::pushOne(ObjList& stack, ExpOperation* oper)
 {
     if (oper)
@@ -728,9 +872,15 @@ ExpOperation* ExpEvaluator::popOne(ObjList& stack)
 	return 0;
     }
     stack.remove(o,false);
-    XDebug(DebugAll,"Popped: %p%s%s",o,
+#ifdef DEBUG
+#ifdef XDEBUG
+    Debug(DebugAll,"popOne: %p%s%s",o,
 	(YOBJECT(ExpFunction,o) ? " function" : ""),
 	(YOBJECT(ExpWrapper,o) ? " wrapper" : ""));
+#else
+    Debug(DebugAll,"popOne: %p",o);
+#endif
+#endif
     return o;
 }
 
@@ -745,7 +895,16 @@ ExpOperation* ExpEvaluator::popAny(ObjList& stack)
 	stack.remove();
     }
     stack.remove(o,false);
-    DDebug(DebugInfo,"Popped: %p",o);
+#ifdef DEBUG
+#ifdef XDEBUG
+    Debug(DebugAll,"popAny: %p%s%s '%s'",o,
+	(YOBJECT(ExpFunction,o) ? " function" : ""),
+	(YOBJECT(ExpWrapper,o) ? " wrapper" : ""),
+	(o ? o->name().safe() : (const char*)0));
+#else
+    Debug(DebugAll,"popAny: %p '%s'",o,(o ? o->name().safe() : (const char*)0));
+#endif
+#endif
     return o;
 }
 
@@ -754,6 +913,8 @@ ExpOperation* ExpEvaluator::popValue(ObjList& stack, GenObject* context) const
     ExpOperation* oper = popOne(stack);
     if (!oper || (oper->opcode() != OpcField))
 	return oper;
+    XDebug(DebugAll,"ExpEvaluator::popValue() field '%s' [%p]",
+	oper->name().c_str(),this);
     bool ok = runField(stack,*oper,context);
     TelEngine::destruct(oper);
     return ok ? popOne(stack) : 0;
@@ -769,9 +930,34 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 	case OpcField:
 	    pushOne(stack,oper.clone());
 	    break;
+	case OpcCopy:
+	    {
+		Mutex* mtx = 0;
+		ScriptRun* runner = YOBJECT(ScriptRun,&oper);
+		if (runner) {
+		    if (runner->context())
+			mtx = runner->context()->mutex();
+		    if (!mtx)
+			mtx = runner;
+		}
+		pushOne(stack,oper.copy(mtx));
+	    }
+	    break;
 	case OpcNone:
 	case OpcLabel:
 	    break;
+	case OpcDrop:
+	    TelEngine::destruct(popOne(stack));
+	    break;
+	case OpcDup:
+	    {
+		ExpOperation* op = popValue(stack,context);
+		if (!op)
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
+		pushOne(stack,op->clone());
+		pushOne(stack,op);
+	    }
+	    break;
 	case OpcAnd:
 	case OpcOr:
 	case OpcXor:
@@ -796,13 +982,13 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		if (!op1 || !op2) {
 		    TelEngine::destruct(op1);
 		    TelEngine::destruct(op2);
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		}
 		switch (oper.opcode()) {
 		    case OpcDiv:
 		    case OpcMod:
-			if (!op2->number())
-			    return gotError("Division by zero");
+			if (!op2->valInteger())
+			    return gotError("Division by zero",oper.lineNumber());
 		    case OpcAdd:
 			if (op1->isInteger() && op2->isInteger())
 			    break;
@@ -821,46 +1007,46 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		long int val = 0;
 		switch (oper.opcode()) {
 		    case OpcAnd:
-			val = op1->number() & op2->number();
+			val = op1->valInteger() & op2->valInteger();
 			break;
 		    case OpcOr:
-			val = op1->number() | op2->number();
+			val = op1->valInteger() | op2->valInteger();
 			break;
 		    case OpcXor:
-			val = op1->number() ^ op2->number();
+			val = op1->valInteger() ^ op2->valInteger();
 			break;
 		    case OpcShl:
-			val = op1->number() << op2->number();
+			val = op1->valInteger() << op2->valInteger();
 			break;
 		    case OpcShr:
-			val = op1->number() >> op2->number();
+			val = op1->valInteger() >> op2->valInteger();
 			break;
 		    case OpcAdd:
-			val = op1->number() + op2->number();
+			val = op1->valInteger() + op2->valInteger();
 			break;
 		    case OpcSub:
-			val = op1->number() - op2->number();
+			val = op1->valInteger() - op2->valInteger();
 			break;
 		    case OpcMul:
-			val = op1->number() * op2->number();
+			val = op1->valInteger() * op2->valInteger();
 			break;
 		    case OpcDiv:
-			val = op1->number() / op2->number();
+			val = op1->valInteger() / op2->valInteger();
 			break;
 		    case OpcMod:
-			val = op1->number() % op2->number();
+			val = op1->valInteger() % op2->valInteger();
 			break;
 		    case OpcLt:
-			val = (op1->number() < op2->number()) ? 1 : 0;
+			val = (op1->valInteger() < op2->valInteger()) ? 1 : 0;
 			break;
 		    case OpcGt:
-			val = (op1->number() > op2->number()) ? 1 : 0;
+			val = (op1->valInteger() > op2->valInteger()) ? 1 : 0;
 			break;
 		    case OpcLe:
-			val = (op1->number() <= op2->number()) ? 1 : 0;
+			val = (op1->valInteger() <= op2->valInteger()) ? 1 : 0;
 			break;
 		    case OpcGe:
-			val = (op1->number() >= op2->number()) ? 1 : 0;
+			val = (op1->valInteger() >= op2->valInteger()) ? 1 : 0;
 			break;
 		    case OpcEq:
 			val = (*op1 == *op2) ? 1 : 0;
@@ -891,15 +1077,15 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		if (!op1 || !op2) {
 		    TelEngine::destruct(op1);
 		    TelEngine::destruct(op2);
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		}
 		bool val = false;
 		switch (oper.opcode()) {
 		    case OpcLAnd:
-			val = op1->number() && op2->number();
+			val = op1->valBoolean() && op2->valBoolean();
 			break;
 		    case OpcLOr:
-			val = op1->number() || op2->number();
+			val = op1->valBoolean() || op2->valBoolean();
 			break;
 		    default:
 			break;
@@ -917,7 +1103,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		if (!op1 || !op2) {
 		    TelEngine::destruct(op1);
 		    TelEngine::destruct(op2);
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		}
 		String val = *op1 + *op2;
 		TelEngine::destruct(op1);
@@ -933,7 +1119,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		if (!op1 || !op2) {
 		    TelEngine::destruct(op1);
 		    TelEngine::destruct(op2);
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		}
 		pushOne(stack,op1->clone(*op2));
 		TelEngine::destruct(op1);
@@ -946,27 +1132,27 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 	    {
 		ExpOperation* op = popValue(stack,context);
 		if (!op)
-		    return gotError("ExpEvaluator stack underflow");
-		long int val = op->number();
-		TelEngine::destruct(op);
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		switch (oper.opcode()) {
 		    case OpcNeg:
-			pushOne(stack,new ExpOperation(-val));
+			pushOne(stack,new ExpOperation(-op->valInteger()));
 			break;
 		    case OpcNot:
-			pushOne(stack,new ExpOperation(~val));
+			pushOne(stack,new ExpOperation(~op->valInteger()));
 			break;
 		    case OpcLNot:
-			pushOne(stack,new ExpOperation(!val));
+			pushOne(stack,new ExpOperation(!op->valBoolean()));
 			break;
 		    default:
-			pushOne(stack,new ExpOperation(val));
+			pushOne(stack,new ExpOperation(op->valInteger()));
 			break;
 		}
+		TelEngine::destruct(op);
 	    }
 	    break;
 	case OpcFunc:
-	    return runFunction(stack,oper,context) || gotError("Function call failed");
+	    return runFunction(stack,oper,context) ||
+		gotError("Function '" + oper.name() + "' call failed",oper.lineNumber());
 	case OpcIncPre:
 	case OpcDecPre:
 	case OpcIncPost:
@@ -974,17 +1160,17 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 	    {
 		ExpOperation* fld = popOne(stack);
 		if (!fld)
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		if (fld->opcode() != OpcField) {
 		    TelEngine::destruct(fld);
-		    return gotError("Expecting LValue in operator");
+		    return gotError("Expecting LValue in operator",oper.lineNumber());
 		}
 		ExpOperation* val = 0;
 		if (!(runField(stack,*fld,context) && (val = popOne(stack)))) {
 		    TelEngine::destruct(fld);
 		    return false;
 		}
-		long int num = val->number();
+		long int num = val->valInteger();
 		switch (oper.opcode()) {
 		    case OpcIncPre:
 			num++;
@@ -1010,7 +1196,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		TelEngine::destruct(fld);
 		if (!ok) {
 		    TelEngine::destruct(val);
-		    return gotError("Assignment failed");
+		    return gotError("Assignment failed",oper.lineNumber());
 		}
 		pushOne(stack,val);
 	    }
@@ -1022,12 +1208,12 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		if (!fld || !val) {
 		    TelEngine::destruct(fld);
 		    TelEngine::destruct(val);
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		}
 		if (fld->opcode() != OpcField) {
 		    TelEngine::destruct(fld);
 		    TelEngine::destruct(val);
-		    return gotError("Expecting LValue in assignment");
+		    return gotError("Expecting LValue in assignment",oper.lineNumber());
 		}
 		ExpOperation* op = val->clone(fld->name());
 		TelEngine::destruct(fld);
@@ -1035,7 +1221,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		TelEngine::destruct(op);
 		if (!ok) {
 		    TelEngine::destruct(val);
-		    return gotError("Assignment failed");
+		    return gotError("Assignment failed",oper.lineNumber());
 		}
 		pushOne(stack,val);
 	    }
@@ -1048,12 +1234,12 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		if (!fld || !val) {
 		    TelEngine::destruct(fld);
 		    TelEngine::destruct(val);
-		    return gotError("ExpEvaluator stack underflow");
+		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		}
 		if (fld->opcode() != OpcField) {
 		    TelEngine::destruct(fld);
 		    TelEngine::destruct(val);
-		    return gotError("Expecting LValue in assignment");
+		    return gotError("Expecting LValue in assignment",oper.lineNumber());
 		}
 		pushOne(stack,fld->clone());
 		pushOne(stack,fld);
@@ -1081,7 +1267,7 @@ bool ExpEvaluator::runFunction(ObjList& stack, const ExpOperation& oper, GenObje
 	for (long int i = oper.number(); i; i--) {
 	    ExpOperation* o = popValue(stack,context);
 	    if (!o)
-		return gotError("ExpEvaluator stack underflow");
+		return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 	    res = String((char)o->number()) + res;
 	    TelEngine::destruct(o);
 	}
@@ -1090,7 +1276,7 @@ bool ExpEvaluator::runFunction(ObjList& stack, const ExpOperation& oper, GenObje
     }
     if (oper.name() == YSTRING("now")) {
 	if (oper.number())
-	    return gotError("Function expects no arguments");
+	    return gotError("Function expects no arguments",oper.lineNumber());
 	pushOne(stack,new ExpOperation((long int)Time::secNow()));
 	return true;
     }
@@ -1223,51 +1409,101 @@ int ExpEvaluator::evaluate(Array& results, unsigned int index, GenObject* contex
     return -1;
 }
 
+void ExpEvaluator::dump(const ExpOperation& oper, String& res) const
+{
+    const char* name = getOperator(oper.opcode());
+    if (name) {
+	res << name;
+	return;
+    }
+    switch (oper.opcode()) {
+	case OpcPush:
+	case OpcCopy:
+	    if (oper.isInteger())
+		res << (int)oper.number();
+	    else
+		res << "'" << oper << "'";
+	    break;
+	case OpcField:
+	    res << oper.name();
+	    break;
+	case OpcFunc:
+	    res << oper.name() << "(" << (int)oper.number() << ")";
+	    break;
+	default:
+	    res << "[" << oper.opcode() << "]";
+	    if (oper.number() && oper.isInteger())
+		res << "(" << (int)oper.number() << ")";
+    }
+}
+
 void ExpEvaluator::dump(const ObjList& codes, String& res) const
 {
     for (const ObjList* l = codes.skipNull(); l; l = l->skipNext()) {
 	if (res)
 	    res << " ";
 	const ExpOperation* o = static_cast<const ExpOperation*>(l->get());
-	const char* oper = getOperator(o->opcode());
-	if (oper) {
-	    res << oper;
-	    continue;
-	}
-	switch (o->opcode()) {
-	    case OpcPush:
-		if (o->isInteger())
-		    res << (int)o->number();
-		else
-		    res << "'" << *o << "'";
-		break;
-	    case OpcField:
-		res << o->name();
-		break;
-	    case OpcFunc:
-		res << o->name() << "(" << (int)o->number() << ")";
-		break;
-	    default:
-		res << "[" << o->opcode() << "]";
-	}
+	dump(*o,res);
     }
 }
 
+long int ExpOperation::valInteger() const
+{
+    return isInteger() ? number() : 0;
+}
+
+bool ExpOperation::valBoolean() const
+{
+    return isInteger() ? (number() != 0) : !null();
+}
+
+ExpOperation* ExpOperation::clone(const char* name) const
+{
+    ExpOperation* op = new ExpOperation(*this,name);
+    op->lineNumber(lineNumber());
+    return op;
+}
+
 
 ExpOperation* ExpFunction::clone(const char* name) const
 {
     XDebug(DebugInfo,"ExpFunction::clone('%s') [%p]",name,this);
-    return new ExpFunction(name,number());
+    ExpFunction* op = new ExpFunction(name,number());
+    op->lineNumber(lineNumber());
+    return op;
 }
 
 
 ExpOperation* ExpWrapper::clone(const char* name) const
 {
-    XDebug(DebugInfo,"ExpWrapper::clone('%s') [%p]",name,this);
     RefObject* r = YOBJECT(RefObject,object());
+    XDebug(DebugInfo,"ExpWrapper::clone('%s') %s=%p [%p]",
+	name,(r ? "ref" : "obj"),object(),this);
     if (r)
 	r->ref();
-    return new ExpWrapper(object(),name);
+    ExpWrapper* op = new ExpWrapper(object(),name);
+    static_cast<String&>(*op) = *this;
+    op->lineNumber(lineNumber());
+    return op;
+}
+
+ExpOperation* ExpWrapper::copy(Mutex* mtx) const
+{
+    JsObject* jso = YOBJECT(JsObject,m_object);
+    if (!jso)
+	return ExpOperation::clone();
+    XDebug(DebugInfo,"ExpWrapper::copy(%p) [%p]",mtx,this);
+    ExpWrapper* op = new ExpWrapper(jso->copy(mtx),name());
+    static_cast<String&>(*op) = *this;
+    op->lineNumber(lineNumber());
+    return op;
+}
+
+bool ExpWrapper::valBoolean() const
+{
+    if (!m_object)
+	return false;
+    return !JsParser::isNull(*this);
 }
 
 void* ExpWrapper::getObject(const String& name) const
