@@ -304,6 +304,9 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
     else
 	m_sdpVersion = m_sdpSession = Time::secNow();
 
+    // override the address with the externally advertised if needed
+    if (addr && m_rtpNatAddr)
+	addr = m_rtpNatAddr;
     // no address means on hold or muted
     String origin;
     origin << "yate " << m_sdpSession << " " << m_sdpVersion;
@@ -324,9 +327,16 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	SDPMedia* m = static_cast<SDPMedia*>(ml->get());
 	int rfc2833 = 0;
 	if ((m_rfc2833 >= 0) && m->isAudio()) {
-	    rfc2833 = m->rfc2833().toInteger(m_rfc2833);
-	    if (rfc2833 < 96 || rfc2833 > 127)
-		rfc2833 = 101;
+	    if (!m_rtpForward) {
+		rfc2833 = m->rfc2833().toInteger(m_rfc2833);
+		if (rfc2833 < 96 || rfc2833 > 127)
+		    rfc2833 = 101;
+	    }
+	    else if (m->rfc2833().toBoolean(true)) {
+		rfc2833 = m->rfc2833().toInteger();
+		if (rfc2833 < 96 || rfc2833 > 127)
+		    rfc2833 = 0;
+	    }
 	}
 	String mline(m->fmtList());
 	ObjList* l = mline.split(',',false);
@@ -334,6 +344,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	mline << " " << (m->localPort() ? m->localPort().c_str() : "0") << " " << m->transport();
 	ObjList* map = m->mappings().split(',',false);
 	ObjList rtpmap;
+	ObjList* dest = &rtpmap;
 	String frm;
 	int ptime = 0;
 	ObjList* f = l;
@@ -421,27 +432,27 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 			frm << " " << payload;
 			String* temp = new String("rtpmap:");
 			*temp << payload << " " << map;
-			rtpmap.append(temp);
+			dest = dest->append(temp);
 			if (mode) {
 			    temp = new String("fmtp:");
 			    *temp << payload << " mode=" << mode;
-			    rtpmap.append(temp);
+			    dest = dest->append(temp);
 			}
 			if (*s == "g729") {
 			    temp = new String("fmtp:");
 			    *temp << payload << " annexb=" <<
 				((0 != l->find("g729b")) ? "yes" : "no");
-			    rtpmap.append(temp);
+			    dest = dest->append(temp);
 			}
 			else if (*s == "amr") {
 			    temp = new String("fmtp:");
 			    *temp << payload << " octet-align=0";
-			    rtpmap.append(temp);
+			    dest = dest->append(temp);
 			}
 			else if (*s == "amr-o") {
 			    temp = new String("fmtp:");
 			    *temp << payload << " octet-align=1";
-			    rtpmap.append(temp);
+			    dest = dest->append(temp);
 			}
 		    }
 		}
@@ -455,7 +466,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	    frm << " " << rfc2833;
 	    String* s = new String;
 	    *s << "rtpmap:" << rfc2833 << " telephone-event/8000";
-	    rtpmap.append(s);
+	    dest = dest->append(s);
 	}
 
 	if (frm.null()) {
@@ -477,7 +488,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	if (ptime) {
 	    String* temp = new String("ptime:");
 	    *temp << ptime;
-	    rtpmap.append(temp);
+	    dest = dest->append(temp);
 	}
 
 	sdp->addLine("m",mline + frm);
