@@ -88,7 +88,7 @@ SignallingFactory::~SignallingFactory()
     s_mutex.unlock();
 }
 
-SignallingComponent* SignallingFactory::build(const String& type, const NamedList* name)
+SignallingComponent* SignallingFactory::build(const String& type, NamedList* name)
 {
     if (type.null())
 	return 0;
@@ -129,7 +129,7 @@ SignallingComponent* SignallingFactory::build(const String& type, const NamedLis
     return 0;
 }
 
-void* SignallingFactory::buildInternal(const String& type, const NamedList* name)
+void* SignallingFactory::buildInternal(const String& type, NamedList* name)
 {
     SignallingComponent* c = build(type,name);
     if (!c)
@@ -145,8 +145,8 @@ void* SignallingFactory::buildInternal(const String& type, const NamedList* name
 }
 
 
-SignallingComponent::SignallingComponent(const char* name, const NamedList* params)
-    : m_engine(0), m_compType("unknown")
+SignallingComponent::SignallingComponent(const char* name, const NamedList* params, const char* type)
+    : m_engine(0), m_compType(type)
 {
     if (params) {
 	name = params->getValue(YSTRING("debugname"),name);
@@ -184,6 +184,28 @@ bool SignallingComponent::initialize(const NamedList* config)
     return true;
 }
 
+bool SignallingComponent::resolveConfig(const String& cmpName, NamedList& params, const NamedList* config)
+{
+    if (!config)
+	return false;
+    String name = config->getValue(cmpName,params);
+    if (!(name && !name.toBoolean(false)))
+	return false;
+    static_cast<String&>(params) = name;
+    NamedString* param = config->getParam(params);
+    NamedPointer* ptr = YOBJECT(NamedPointer,param);
+    NamedList* ifConfig = ptr ? YOBJECT(NamedList,ptr->userData()) : 0;
+    if (ifConfig)
+	params.copyParams(*ifConfig);
+    else {
+	if (config->hasSubParams(params + "."))
+	    params.copySubParams(*config,params + ".");
+	else
+	    params.addParam("local-config","true");
+    }
+    return true;
+}
+
 bool SignallingComponent::control(NamedList& params)
 {
     return false;
@@ -341,7 +363,7 @@ bool SignallingEngine::find(const SignallingComponent* component)
     return m_components.find(component) != 0;
 }
 
-SignallingComponent* SignallingEngine::build(const String& type, const NamedList& params, bool init, bool ref)
+SignallingComponent* SignallingEngine::build(const String& type, NamedList& params, bool init, bool ref)
 {
     XDebug(this,DebugAll,"Engine building '%s' of type %s [%p]",
 	params.c_str(),type.c_str(),this);
@@ -710,6 +732,7 @@ static const TokenDict s_dict_formatCCITT[] = {
 	{"alaw",          0x03},         // Recomendation G.711 A-law 
 	{"g721",          0x04},         // Recomendation G.721 32kbit/s ADPCM and I.460
 	{"h221",          0x05},         // Recomendation H.221 and H.242
+	{"h223",          0x06},         // Recomendation H.223 and H.245 videoconference
 	{"non-CCITT",     0x07},         // Non CCITT standardized rate adaption
 	{"v120",          0x08},         // Recomendation V.120
 	{"x31",           0x09},         // Recomendation X.31 HDLC flag stuffing
