@@ -76,8 +76,8 @@ bool SS7Layer3::initialize(const NamedList* config)
 	setNI(SS7MSU::getNetIndicator(config->getValue(YSTRING("netindicator")),SS7MSU::National));
     if (engine() && !user()) {
 	NamedList params("ss7router");
-	if (config)
-	    static_cast<String&>(params) = config->getValue(YSTRING("router"),params);
+	if (!resolveConfig(YSTRING("router"),params,config))
+	    params.addParam("local-config","true");
 	if (params.toBoolean(true))
 	    SS7Layer3::attach(YOBJECT(SS7Router,engine()->build("SS7Router",params,true,false)));
     }
@@ -507,10 +507,10 @@ void SS7Layer3::printRoutes()
 
 
 SS7MTP3::SS7MTP3(const NamedList& params)
-    : SignallingComponent(params.safe("SS7MTP3"),&params),
+    : SignallingComponent(params.safe("SS7MTP3"),&params,"ss7-mtp3"),
       SignallingDumpable(SignallingDumper::Mtp3),
       Mutex(true,"SS7MTP3"),
-      m_total(0), m_active(0), m_inhibit(false), m_warnDown(true),
+      m_total(0), m_active(0), m_slcShift(false), m_inhibit(false), m_warnDown(true),
       m_checklinks(true), m_forcealign(true), m_checkT1(0), m_checkT2(0)
 {
 #ifdef DEBUG
@@ -553,6 +553,7 @@ SS7MTP3::SS7MTP3(const NamedList& params)
     }
     Debug(this,level,"Point code types are '%s' [%p]",stype.safe(),this);
 
+    m_slcShift = params.getBoolValue(YSTRING("slcshift"),false);
     m_inhibit = !params.getBoolValue(YSTRING("autostart"),true);
     m_checklinks = params.getBoolValue(YSTRING("checklinks"),m_checklinks);
     m_forcealign = params.getBoolValue(YSTRING("forcealign"),m_forcealign);
@@ -822,6 +823,9 @@ bool SS7MTP3::control(Operation oper, NamedList* params)
 {
     bool ok = operational();
     if (params) {
+	// cannot change SLS to SLC shift while active
+	if (m_active == 0)
+	    m_slcShift = params->getBoolValue(YSTRING("slcshift"),m_slcShift);
 	m_checklinks = params->getBoolValue(YSTRING("checklinks"),m_checklinks);
 	m_forcealign = params->getBoolValue(YSTRING("forcealign"),m_forcealign);
 	const String& inh = (*params)[YSTRING("inhibit")];
@@ -922,6 +926,7 @@ bool SS7MTP3::initialize(const NamedList* config)
     countLinks();
     m_warnDown = true;
     if (config && (0 == m_total)) {
+	m_slcShift = config->getBoolValue(YSTRING("slcshift"),m_slcShift);
 	m_checklinks = config->getBoolValue(YSTRING("checklinks"),m_checklinks);
 	m_forcealign = config->getBoolValue(YSTRING("forcealign"),m_forcealign);
 	unsigned int n = config->length();
@@ -943,7 +948,10 @@ bool SS7MTP3::initialize(const NamedList* config)
 	    if (linkConfig)
 		params.copyParams(*linkConfig);
 	    else {
-		params.copySubParams(*config,params + ".");
+		if (config->hasSubParams(params + "."))
+		    params.copySubParams(*config,params + ".");
+		else
+		    params.addParam("local-config","true");
 		linkConfig = &params;
 	    }
 	    SS7Layer2* link = YSIGCREATE(SS7Layer2,&params);
@@ -982,11 +990,12 @@ int SS7MTP3::transmitMSU(const SS7MSU& msu, const SS7Label& label, int sls)
 {
     bool maint = (msu.getSIF() == SS7MSU::MTN) || (msu.getSIF() == SS7MSU::MTNS);
     bool mgmt = (msu.getSIF() == SS7MSU::SNM);
+    bool regular = !maint && !mgmt;
     Lock lock(this);
     if (!(maint || m_active || (mgmt && m_checked))) {
 	if (m_warnDown) {
 	    m_warnDown = false;
-	    Debug(this,DebugMild,"Could not transmit %s MSU, %s",
+	    Debug(this,m_total ? DebugInfo : DebugMild,"Could not transmit %s MSU, %s",
 		msu.getServiceName(),
 		m_total ? "all links are down" : "no data links attached");
 	}
@@ -994,8 +1003,11 @@ int SS7MTP3::transmitMSU(const SS7MSU& msu, const SS7Label& label, int sls)
     }
 
     // TODO: support ranges with holes
-    if (!maint && !mgmt)
+    if (regular && sls >= 0) {
+	if (m_slcShift)
+	    sls = sls >> 1;
 	sls = sls % m_total;
+    }
 
     // Try to find a link with the given SLS
     ObjList* l = (sls >= 0) ? &m_links : 0;
@@ -1013,7 +1025,7 @@ int SS7MTP3::transmitMSU(const SS7MSU& msu, const SS7Label& label, int sls)
 			(m_inhibit ? " while inhibited" : ""),this);
 		    dump(msu,true,sls);
 		    m_warnDown = true;
-		    return sls;
+		    return (regular && m_slcShift) ? (sls << 1) : sls;
 		}
 		return -1;
 	    }
@@ -1045,11 +1057,11 @@ int SS7MTP3::transmitMSU(const SS7MSU& msu, const SS7Label& label, int sls)
 		(m_inhibit ? " while inhibited" : ""),this);
 	    dump(msu,true,sls);
 	    m_warnDown = true;
-	    return sls;
+	    return (regular && m_slcShift) ? (sls << 1) : sls;
 	}
     }
 
-    Debug(this,((sls == -2) ? DebugWarn : DebugMild),
+    Debug(this,((sls == -2) ? DebugWarn : DebugInfo),
 	"Could not find any link to send %s MSU",msu.getServiceName());
     return -1;
 }
