@@ -260,6 +260,11 @@ int SS7Route::transmitMSU(const SS7Router* router, const SS7MSU& msu,
 int SS7Route::transmitInternal(const SS7Router* router, const SS7MSU& msu,
     const SS7Label& label, int sls, State states, const SS7Layer3* source)
 {
+#ifdef DEBUG
+    bool info = true;
+#else
+    bool info = false;
+#endif
     int offs = 0;
     bool userPart = (msu.getSIF() > SS7MSU::MTNS);
     if (userPart)
@@ -281,16 +286,19 @@ int SS7Route::transmitInternal(const SS7Router* router, const SS7MSU& msu,
 		m_congCount++;
 		m_congBytes += msu.length();
 	    }
-#ifdef DEBUG
-	    String addr;
-	    addr << label;
-	    Debug(router,DebugAll,"MSU %s size %u sent on '%s' SLS %d%s",
-		addr.c_str(),msu.length(),l3->toString().c_str(),res,
-		(cong ? " (congested)" : ""));
-#endif
+	    if (info) {
+		String addr;
+		addr << label;
+		Debug(router,DebugInfo,"MSU %s size %u sent on %s:%d%s",
+		    addr.c_str(),msu.length(),l3->toString().c_str(),res,
+		    (cong ? " (congested)" : ""));
+	    }
 	    return res;
 	}
+	info = true;
     }
+    Debug(router,DebugMild,"Could not send %s MSU size %u on any linkset",
+	msu.getServiceName(),msu.length());
     return -1;
 }
 
@@ -340,14 +348,14 @@ void SS7Route::reroute()
  * SS7Router
  */
 SS7Router::SS7Router(const NamedList& params)
-    : SignallingComponent(params.safe("SS7Router"),&params),
+    : SignallingComponent(params.safe("SS7Router"),&params,"ss7-router"),
       Mutex(true,"SS7Router"),
       m_changes(0), m_transfer(false), m_phase2(false), m_started(false),
       m_restart(0), m_isolate(0), m_statsMutex(false,"SS7RouterStats"),
       m_trafficOk(0), m_trafficSent(0), m_routeTest(0), m_testRestricted(false),
       m_transferSilent(false), m_checkRoutes(false), m_autoAllowed(false),
       m_sendUnavail(true), m_sendProhibited(true),
-      m_rxMsu(0), m_txMsu(0), m_fwdMsu(0), m_congestions(0),
+      m_rxMsu(0), m_txMsu(0), m_fwdMsu(0), m_failMsu(0), m_congestions(0),
       m_mngmt(0)
 {
 #ifdef DEBUG
@@ -390,7 +398,10 @@ SS7Router::SS7Router(const NamedList& params)
 	if (mConfig)
 	    mParams.copyParams(*mConfig);
 	else {
-	    mParams.copySubParams(params,mParams + ".");
+	    if (params.hasSubParams(mParams + "."))
+		mParams.copySubParams(params,mParams + ".");
+	    else
+		mParams.addParam("local-config","true");
 	    mConfig = &mParams;
 	}
 	attach(m_mngmt = YSIGCREATE(SS7Management,&mParams));
@@ -399,8 +410,8 @@ SS7Router::SS7Router(const NamedList& params)
 
 SS7Router::~SS7Router()
 {
-    Debug(this,DebugInfo,"SS7Router destroyed, rx=%lu, tx=%lu, fwd=%lu, cong=%lu",
-	m_rxMsu,m_txMsu,m_fwdMsu,m_congestions);
+    Debug(this,DebugInfo,"SS7Router destroyed, rx=%lu, tx=%lu, fwd=%lu, fail=%lu, cong=%lu",
+	m_rxMsu,m_txMsu,m_fwdMsu,m_failMsu,m_congestions);
 }
 
 bool SS7Router::initialize(const NamedList* config)
@@ -807,6 +818,20 @@ int SS7Router::routeMSU(const SS7MSU& msu, const SS7Label& label, SS7Layer3* net
 	    m_congestions++;
 	m_statsMutex.unlock();
     }
+    else {
+	m_statsMutex.lock();
+	m_failMsu++;
+	m_statsMutex.unlock();
+	if (!route) {
+	    String tmp;
+	    tmp << label.dpc();
+	    Debug(this,DebugMild,"No route to %s was found for %s MSU size %u",
+		tmp.c_str(),msu.getServiceName(),msu.length());
+	}
+	else
+	    Debug(this,DebugAll,"Failed to send %s MSU size %u on %s route %u",
+		msu.getServiceName(),msu.length(),route->stateName(),route->packed());
+    }
     return slsTx;
 }
 
@@ -850,8 +875,13 @@ HandledMSU SS7Router::receivedMSU(const SS7MSU& msu, const SS7Label& label, SS7L
     }
     if ((msu.getSIF() > SS7MSU::MTNS) && !m_started)
 	return HandledMSU::Failure;
+    bool maint = (msu.getSIF() == SS7MSU::MTN) || (msu.getSIF() == SS7MSU::MTNS);
+    if (!maint) {
+	m_statsMutex.lock();
+	m_rxMsu++;
+	m_statsMutex.unlock();
+    }
     lock();
-    m_rxMsu++;
     ObjList* l;
     HandledMSU ret;
     do {
@@ -896,7 +926,7 @@ HandledMSU SS7Router::receivedMSU(const SS7MSU& msu, const SS7Label& label, SS7L
 	    break;
     }
     // maintenance must stop here, others may be transferred out
-    if ((msu.getSIF() == SS7MSU::MTN) || (msu.getSIF() == SS7MSU::MTNS))
+    if (maint)
 	return HandledMSU::Rejected;
     unsigned int dpc = label.dpc().pack(label.type());
     // if packet was for this node as set in router don't process any further
@@ -1855,6 +1885,7 @@ bool SS7Router::control(NamedList& params)
 	    // fall through
 	case SS7Router::Status:
 	    printRoutes();
+	    printStats();
 	    return operational();
 	case SS7Router::Advertise:
 	    if (!(m_transfer && (m_started || m_phase2)))
@@ -1974,6 +2005,17 @@ bool SS7Router::control(NamedList& params)
     return false;
 }
 
+void SS7Router::printStats()
+{
+    String tmp;
+    m_statsMutex.lock();
+    tmp << "Rx=" << (unsigned int)m_rxMsu << ", Tx=" << (unsigned int)m_txMsu;
+    tmp << ", Fwd=" << (unsigned int)m_fwdMsu << ", Fail=" << (unsigned int)m_failMsu;
+    tmp << ", Cong=" << (unsigned int)m_congestions;
+    m_statsMutex.unlock();
+    Output("Statistics for '%s': %s",debugName(),tmp.c_str());
+}
+
 // Detach management. Call SignallingComponent::detach()
 void SS7Router::destroyed()
 {
