@@ -558,10 +558,22 @@ public:
     /**
      * Append a new child to this XmlParent
      * @param child The child to append
-     * @return XmlNoError if the child was successfully added
+     * @return NoError if the child was successfully added
      */
     virtual XmlSaxParser::Error addChild(XmlChild* child) = 0;
 
+    /**
+     * Append a new child of this XmlParent, release the object on failure
+     * @param child The child to append
+     * @return The child on success, 0 on failure
+     */
+    inline XmlChild* addChildSafe(XmlChild* child) {
+	    XmlSaxParser::Error err = addChild(child);
+	    if (err != XmlSaxParser::NoError)
+		TelEngine::destruct(child);
+	    return child;
+	}
+
     /**
      * Remove a child
      * @param child The child to remove
@@ -898,6 +910,12 @@ public:
     inline XmlChild* pop()
 	{ return static_cast<XmlChild*>(m_list.remove(false)); }
 
+    /**
+     * Remove the first XmlElement from list and returns it if completed
+     * @return XmlElement pointer or 0 if no XmlElement is found or the first one is not completed
+     */
+    XmlElement* popElement();
+
     /**
      * Remove a child. Reset the parent of not deleted xml element
      * @param child The child to remove
@@ -1094,6 +1112,14 @@ public:
      */
     XmlElement(const char* name, bool complete = true);
 
+    /**
+     * Constructor. Create a new element with a text child
+     * @param name The name of the element
+     * @param value Element text child value
+     * @param complete False to build an incomplete element
+     */
+    XmlElement(const char* name, const char* value, bool complete = true);
+
     /**
      * Copy constructor
      * @param orig Original XmlElement
@@ -1316,6 +1342,21 @@ public:
 	    return start ? XmlFragment::findElement(start->skipNext(),name,ns,noPrefix) : 0;
 	}
 
+    /**
+     * Retrieve a child's text
+     * @param name Name (tag) of the child
+     * @param ns Optional child namespace
+     * @param noPrefix True to compare the tag without namespace prefix, false to
+     *  include namespace prefix when comparing the given tag.
+     *  This parameter is ignored ns is not 0
+     * @return Pointer to child's text, 0 if the child was not found
+     */
+    inline const String* childText(const String& name, const String* ns = 0,
+	bool noPrefix = true) const {
+	    XmlElement* c = findFirstChild(&name,ns,noPrefix);
+	    return c ? &(c->getText()) : 0;
+	}
+
     /**
      * @return The first XmlText found in this XmlElement children
      */
