@@ -56,12 +56,19 @@ namespace { // anonymous
 // Maximum number of frames we are willing to decode in a packet
 #define MAX_PKT_FRAMES  4
 
+// Discontinuous Transmission (DTX)
+static bool s_discontinuous = false;
+
+// Default mode
+static Mode s_mode = MR122;
+
+
 class AmrPlugin : public Plugin, public TranslatorFactory
 {
 public:
     AmrPlugin();
     ~AmrPlugin();
-    virtual void initialize() {}
+    virtual void initialize();
     virtual bool isBusy() const;
     virtual DataTranslator* create(const DataFormat& sFormat, const DataFormat& dFormat);
     virtual const TranslatorCaps* getCapabilities() const;
@@ -70,7 +77,7 @@ public:
 class AmrTrans : public DataTranslator
 {
 public:
-    AmrTrans(const char* sFormat, const char* dFormat, void* amrState, bool octetAlign = false);
+    AmrTrans(const char* sFormat, const char* dFormat, void* amrState, bool octetAlign, bool encoding);
     virtual ~AmrTrans();
     virtual unsigned long Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags);
     inline bool valid() const
@@ -78,10 +85,13 @@ public:
     static inline const char* alignName(bool align)
 	{ return align ? "octet aligned" : "bandwidth efficient"; }
 protected:
+    void filterBias(short* buf, unsigned int len);
     bool dataError(const char* text = 0);
-    virtual bool pushData(unsigned long& tStamp) = 0;
+    virtual bool pushData(unsigned long& tStamp, unsigned long& flags) = 0;
     void* m_amrState;
     DataBlock m_data;
+    int m_bias;
+    bool m_encoding;
     bool m_showError;
     bool m_octetAlign;
     Mode m_cmr;
@@ -92,13 +102,15 @@ class AmrEncoder : public AmrTrans
 {
 public:
     inline AmrEncoder(const char* sFormat, const char* dFormat, bool octetAlign, bool discont = false)
-	: AmrTrans(sFormat,dFormat,::Encoder_Interface_init(discont ? 1 : 0),octetAlign),
-	 m_mode(MR122)
+	: AmrTrans(sFormat,dFormat,::Encoder_Interface_init(discont ? 1 : 0),octetAlign,true),
+	 m_mode(s_mode), m_silent(false)
 	{ }
     virtual ~AmrEncoder();
+    virtual bool control(NamedList& params);
 protected:
-    virtual bool pushData(unsigned long& tStamp);
+    virtual bool pushData(unsigned long& tStamp, unsigned long& flags);
     Mode m_mode;
+    bool m_silent;
 };
 
 // Decoding specific class
@@ -106,11 +118,11 @@ class AmrDecoder : public AmrTrans
 {
 public:
     inline AmrDecoder(const char* sFormat, const char* dFormat, bool octetAlign)
-	: AmrTrans(sFormat,dFormat,::Decoder_Interface_init(),octetAlign)
+	: AmrTrans(sFormat,dFormat,::Decoder_Interface_init(),octetAlign,false)
 	{ }
     virtual ~AmrDecoder();
 protected:
-    virtual bool pushData(unsigned long& tStamp);
+    virtual bool pushData(unsigned long& tStamp, unsigned long& flags);
 };
 
 // Module data
@@ -130,8 +142,18 @@ static int modeBits[16] = {
     -1, -1, -1, -1, -1, -1, 0
 };
 
-// Discontinuous Transmission (DTX)
-bool s_discontinuous = false;
+// Table for bitrate to mode conversion
+static const TokenDict s_modes[] = {
+    { "4.75", MR475 },
+    { "5.15", MR515 },
+    { "5.90", MR59  },
+    { "6.70", MR67  },
+    { "7.40", MR74  },
+    { "7.95", MR795 },
+    { "10.2", MR102 },
+    { "12.2", MR122 },
+    { 0, 0 }
+};
 
 // Helper function, gets a number of bits and advances pointer, return -1 for error
 static int getBits(unsigned const char*& ptr, int& len, int& bpos, unsigned char bits)
@@ -159,13 +181,13 @@ static int getBits(unsigned const char*& ptr, int& len, int& bpos, unsigned char
 
 
 // Arbitrary type transcoder constructor
-AmrTrans::AmrTrans(const char* sFormat, const char* dFormat, void* amrState, bool octetAlign)
+AmrTrans::AmrTrans(const char* sFormat, const char* dFormat, void* amrState, bool octetAlign, bool encoding)
     : DataTranslator(sFormat,dFormat),
-      m_amrState(amrState), m_showError(true),
-      m_octetAlign(octetAlign), m_cmr(MR122)
+      m_amrState(amrState), m_bias(0), m_encoding(encoding), m_showError(true),
+      m_octetAlign(octetAlign), m_cmr(s_mode)
 {
-    Debug(MODNAME,DebugAll,"AmrTrans::AmrTrans('%s','%s',%p,%s) [%p]",
-	sFormat,dFormat,amrState,String::boolText(octetAlign),this);
+    Debug(MODNAME,DebugAll,"AmrTrans::AmrTrans('%s','%s',%p,%s,%s) [%p]",
+	sFormat,dFormat,amrState,String::boolText(octetAlign),String::boolText(encoding),this);
     count++;
 }
 
@@ -185,15 +207,40 @@ unsigned long AmrTrans::Consume(const DataBlock& data, unsigned long tStamp, uns
     if (data.null() && (flags & DataSilent))
 	return getTransSource()->Forward(data,tStamp,flags);
     ref();
+    if (m_encoding && (tStamp != invalidStamp()) && !m_data.null())
+	tStamp -= (m_data.length() / 2);
     m_data += data;
-    if (!tStamp)
-	tStamp = timeStamp() + SAMPLES_FRAME;
-    while (pushData(tStamp))
+    if (m_encoding) {
+	// the AMR encoder errors on biased silence so suppress it
+	unsigned int len = data.length();
+	if (len)
+	    filterBias((short*)m_data.data(m_data.length() - len),len / 2);
+    }
+    while (pushData(tStamp,flags))
 	;
     deref();
     return invalidStamp();
 }
 
+// High pass filter to get rid of the bias - for example when transcoding through A-Law
+void AmrTrans::filterBias(short* buf, unsigned int len)
+{
+    if (!buf)
+	return;
+    while (len--) {
+	int val = *buf;
+	// work on integers using sample * 16
+	m_bias = (m_bias * 63 + val * 16) / 64;
+	// substract the averaged bias and saturate
+	val -= m_bias / 16;
+	if (val > 32767)
+	    val = 32676;
+	else if (val < -32767)
+	    val = -32767;
+	*buf++ = val;
+    }
+}
+
 // Data error, report error 1st time and clear buffer
 bool AmrTrans::dataError(const char* text)
 {
@@ -218,7 +265,7 @@ AmrEncoder::~AmrEncoder()
 }
 
 // Encode accumulated slin data and push it to the consumer
-bool AmrEncoder::pushData(unsigned long& tStamp)
+bool AmrEncoder::pushData(unsigned long& tStamp, unsigned long& flags)
 {
     if (m_data.length() < BUFFER_SIZE)
 	return false;
@@ -227,6 +274,17 @@ bool AmrEncoder::pushData(unsigned long& tStamp)
     int len = ::Encoder_Interface_Encode(m_amrState,m_mode,(short*)m_data.data(),unpacked,0);
     if ((len <= 0) || (len >= MAX_AMRNB_SIZE))
 	return dataError("encoder");
+    Mode mode = (Mode)((unpacked[0] >> 3) & 0x0f);
+    if (mode > MRDTX) {
+	// invalid mode returned in frame - don't send the the data at all
+	m_data.cut(-BUFFER_SIZE);
+	tStamp += SAMPLES_FRAME;
+	return (0 != m_data.length());
+    }
+    bool silent = (mode == MRDTX);
+    if (m_silent && !silent)
+	flags |= DataMark;
+    m_silent = silent;
     unpacked[len] = 0;
     XDebug(MODNAME,DebugAll,"Encoded mode %d frame to %d bytes first %02x [%p]",
 	m_mode,len,unpacked[0],this);
@@ -255,12 +313,30 @@ bool AmrEncoder::pushData(unsigned long& tStamp)
     }
     m_data.cut(-BUFFER_SIZE);
     DataBlock outData(buffer,len,false);
-    getTransSource()->Forward(outData,tStamp);
+    getTransSource()->Forward(outData,tStamp,flags);
     outData.clear(false);
     tStamp += SAMPLES_FRAME;
+    flags &= ~DataMark;
     return (0 != m_data.length());
 }
 
+// Execute control operations
+bool AmrEncoder::control(NamedList& params)
+{
+    bool ok = false;
+    int mode = params[YSTRING("mode")].toInteger(s_modes,-1);
+    if (mode >= MR475 && mode <= MR122) {
+	m_mode = (Mode)mode;
+	ok = true;
+    }
+    mode = params[YSTRING("cmr")].toInteger(s_modes,-1);
+    if (mode >= MR475 && mode <= MR122) {
+	m_cmr = (Mode)mode;
+	ok = true;
+    }
+    return AmrTrans::control(params) || ok;
+}
+
 
 // Decoder cleanup
 AmrDecoder::~AmrDecoder()
@@ -271,7 +347,7 @@ AmrDecoder::~AmrDecoder()
 }
 
 // Decode AMR data and push it to the consumer
-bool AmrDecoder::pushData(unsigned long& tStamp)
+bool AmrDecoder::pushData(unsigned long& tStamp, unsigned long& flags)
 {
     if (m_data.length() < 2)
 	return false;
@@ -340,9 +416,10 @@ bool AmrDecoder::pushData(unsigned long& tStamp)
 	    (good ? RxTypes::RX_SPEECH_GOOD : RxTypes::RX_SPEECH_DEGRADED);
 	::Decoder_Interface_Decode(m_amrState,unpacked,buffer,type);
 	DataBlock outData(buffer,BUFFER_SIZE,false);
-	getTransSource()->Forward(outData,tStamp);
+	getTransSource()->Forward(outData,tStamp,flags);
 	outData.clear(false);
 	tStamp += SAMPLES_FRAME;
+	flags &= ~DataMark;
     }
     if (bpos)
 	len--;
@@ -405,6 +482,16 @@ const TranslatorCaps* AmrPlugin::getCapabilities() const
     return caps;
 }
 
+void AmrPlugin::initialize()
+{
+    Output("Initializing module AMR-NB");
+    Configuration cfg(Engine::configFile("amrnbcodec"));
+    int mode = cfg.getIntValue("general","mode",s_modes,MR122);
+    if (mode >= MR475 && mode <= MR122)
+	s_mode = (Mode)mode;
+    s_discontinuous = cfg.getBoolValue("general","discontinuous",false);
+}
+
 
 INIT_PLUGIN(AmrPlugin);
 
