@@ -84,6 +84,7 @@ public:
     };
     ForkSlave(ForkMaster* master, const char* id);
     virtual ~ForkSlave();
+    virtual void destroyed();
     virtual void disconnected(bool final, const char* reason);
     inline void clearMaster()
 	{ m_master = 0; }
@@ -115,17 +116,6 @@ protected:
     bool m_hasRelays;
 };
 
-class ForkRelay : public MessageHandler
-{
-public:
-    inline ForkRelay(const char* name, const char* match, int priority)
-	: MessageHandler(name,priority),
-	  m_match(match)
-	{ }
-    virtual bool received(Message& msg);
-private:
-    String m_match;
-};
 
 static TokenDict s_calltypes[] = {
     { "regular",    ForkSlave::Regular    },
@@ -144,6 +134,20 @@ UNLOAD_PLUGIN(unloadNow)
 }
 
 
+class ForkRelay : public MessageHandler
+{
+public:
+    inline ForkRelay(const char* name, const char* match, int priority)
+	: MessageHandler(name,priority,__plugin.name()),
+	  m_match(match)
+	{ }
+    virtual bool received(Message& msg)
+	{ return __plugin.msgToSlaves(msg,m_match); }
+private:
+    String m_match;
+};
+
+
 ForkMaster::ForkMaster(ObjList* targets)
     : m_index(0), m_answered(false), m_rtpForward(false), m_rtpStrict(false),
       m_fake(false), m_targets(targets), m_exec(0),
@@ -221,6 +225,7 @@ bool ForkMaster::forkSlave(String* dest)
     NamedList* params = YOBJECT(NamedList,dest);
     if (params)
 	msgCopy.copyParams(*params);
+    const char* error = "failure";
     if (m_execNext) {
 	RefPointer<CallEndpoint> peer = getPeer();
 	if (!peer) {
@@ -233,8 +238,12 @@ bool ForkMaster::forkSlave(String* dest)
 	msgCopy.userData(peer);
 	msgCopy.setParam("id",peer->id());
 	msgCopy.clearParam("cdrtrack");
-	if (!Engine::dispatch(msgCopy))
+	if (!Engine::dispatch(msgCopy)) {
+	    error = msgCopy.getValue("error",error);
+	    Debug(&__plugin,DebugNote,"Call '%s' failed non-fork to target '%s', error '%s'",
+		getPeerId().c_str(),dest->c_str(),error);
 	    return false;
+	}
 	clear(false);
 	return true;
     }
@@ -243,7 +252,6 @@ bool ForkMaster::forkSlave(String* dest)
     ForkSlave* slave = new ForkSlave(this,tmp);
     msgCopy.setParam("id",tmp);
     msgCopy.userData(slave);
-    const char* error = "failure";
     bool autoring = false;
     if (Engine::dispatch(msgCopy)) {
 	ok = true;
@@ -284,8 +292,11 @@ bool ForkMaster::forkSlave(String* dest)
 	    Engine::enqueue(ring);
 	}
     }
-    else
+    else {
+	Debug(&__plugin,DebugNote,"Call '%s' failed on '%s' target '%s', error '%s'",
+	    getPeerId().c_str(),tmp.c_str(),dest->c_str(),error);
 	slave->lostMaster(error);
+    }
     slave->deref();
     return ok;
 }
@@ -353,6 +364,7 @@ bool ForkMaster::callContinue()
 	if (!dest)
 	    break;
 	if (dest->startSkip("|",false)) {
+	    m_execNext = false;
 	    if (*dest) {
 		String tmp(*dest);
 		int tout = 0;
@@ -378,6 +390,8 @@ bool ForkMaster::callContinue()
 	    dest->destruct();
 	    if (forks)
 		break;
+	    m_timer = 0;
+	    m_timerDrop = false;
 	    continue;
 	}
 	if (forkSlave(dest))
@@ -618,10 +632,22 @@ ForkSlave::~ForkSlave()
     DDebug(&__plugin,DebugAll,"ForkSlave::~ForkSlave() '%s'",id().c_str());
 }
 
+void ForkSlave::destroyed()
+{
+    CallEndpoint::commonMutex().lock();
+    RefPointer<ForkMaster> master = m_master;
+    m_master = 0;
+    CallEndpoint::commonMutex().unlock();
+    if (master)
+	master->lostSlave(this,0);
+    CallEndpoint::destroyed();
+}
+
 void ForkSlave::disconnected(bool final, const char* reason)
 {
     CallEndpoint::commonMutex().lock();
     RefPointer<ForkMaster> master = m_master;
+    m_master = 0;
     CallEndpoint::commonMutex().unlock();
     CallEndpoint::disconnected(final,reason);
     if (master)
@@ -629,12 +655,6 @@ void ForkSlave::disconnected(bool final, const char* reason)
 }
 
 
-bool ForkRelay::received(Message& msg)
-{
-    return __plugin.msgToSlaves(msg,m_match);
-}
-
-
 ForkModule::ForkModule()
     : Module("callfork","misc"),
       m_hasRelays(false)
@@ -719,7 +739,7 @@ void ForkModule::statusParams(String& str)
 
 bool ForkModule::msgExecute(Message& msg)
 {
-    CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
+    CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (!ch)
 	return false;
     String dest(msg.getParam("callto"));
