@@ -244,30 +244,6 @@ private:
     RefPointer<ConfConsumer> m_cons;
 };
 
-// Handler for call.conference message to join both legs of a call in conference
-class ConfHandler : public MessageHandler
-{
-public:
-    inline ConfHandler(unsigned int priority)
-	: MessageHandler("call.conference",priority)
-	{ }
-    virtual ~ConfHandler()
-	{ }
-    virtual bool received(Message& msg);
-};
-
-// Handler for chan.hangup message
-class HangupHandler : public MessageHandler
-{
-public:
-    inline HangupHandler(unsigned int priority)
-	: MessageHandler("chan.hangup",priority)
-	{ }
-    virtual ~HangupHandler()
-	{ }
-    virtual bool received(Message& msg);
-};
-
 // The driver just holds all the channels (not conferences)
 class ConferenceDriver : public Driver
 {
@@ -299,6 +275,30 @@ UNLOAD_PLUGIN(unloadNow)
     return true;
 }
 
+// Handler for call.conference message to join both legs of a call in conference
+class ConfHandler : public MessageHandler
+{
+public:
+    inline ConfHandler(unsigned int priority)
+	: MessageHandler("call.conference",priority,__plugin.name())
+	{ }
+    virtual ~ConfHandler()
+	{ }
+    virtual bool received(Message& msg);
+};
+
+// Handler for chan.hangup message
+class HangupHandler : public MessageHandler
+{
+public:
+    inline HangupHandler(unsigned int priority)
+	: MessageHandler("chan.hangup",priority,__plugin.name())
+	{ }
+    virtual ~HangupHandler()
+	{ }
+    virtual bool received(Message& msg);
+};
+
 // Count the position of the most significant 1 bit - pretty close to logarithm
 static unsigned int binLog(unsigned int x)
 {
@@ -1377,7 +1377,7 @@ bool ConferenceDriver::msgExecute(Message& msg, String& dest)
     bool counted = msg.getBoolValue("counted",!utility);
     if (!checkRoom(dest,msg.getBoolValue("existing"),counted))
 	return false;
-    CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
+    CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (ch) {
 	ConfChan *c = new ConfChan(dest,msg,counted,utility);
 	c->initChan();
