@@ -63,6 +63,7 @@ static int s_waitFlush = WAIT_FLUSH;
 static int s_timeout = MSG_TIMEOUT;
 static bool s_timebomb = false;
 static bool s_pluginSafe = true;
+static const char* s_trackName = 0;
 
 static const char* s_cmds[] = {
     "info",
@@ -263,6 +264,7 @@ private:
     String m_script, m_args;
     ObjList m_waiting;
     ObjList m_relays;
+    String m_trackName;
     String m_reason;
 };
 
@@ -280,17 +282,36 @@ private:
     ExtModReceiver* m_receiver;
 };
 
+class ExtModHandler;
+
+class ExtModulePlugin : public Plugin
+{
+public:
+    ExtModulePlugin();
+    ~ExtModulePlugin();
+    virtual void initialize();
+    virtual bool isBusy() const;
+private:
+    ExtModHandler *m_handler;
+};
+
+INIT_PLUGIN(ExtModulePlugin);
+
 class ExtModHandler : public MessageHandler
 {
 public:
-    ExtModHandler(const char *name, unsigned prio) : MessageHandler(name,prio) { }
+    ExtModHandler(const char *name, unsigned prio)
+	: MessageHandler(name,prio,__plugin.name())
+	{ }
     virtual bool received(Message &msg);
 };
 
 class ExtModCommand : public MessageHandler
 {
 public:
-    ExtModCommand(const char *name) : MessageHandler(name) { }
+    ExtModCommand(const char *name)
+	: MessageHandler(name,100,__plugin.name())
+	{ }
     virtual bool received(Message &msg);
 private:
     bool complete(const String& partLine, const String& partWord, String& rval) const;
@@ -311,17 +332,6 @@ protected:
     int m_role;
 };
 
-class ExtModulePlugin : public Plugin
-{
-public:
-    ExtModulePlugin();
-    ~ExtModulePlugin();
-    virtual void initialize();
-    virtual bool isBusy() const;
-private:
-    ExtModHandler *m_handler;
-};
-
 
 static bool runProgram(const char *script, const char *args)
 {
@@ -707,7 +717,7 @@ ExtModReceiver::ExtModReceiver(const char* script, const char* args, File* ain,
       m_in(0), m_out(0), m_ain(ain), m_aout(aout),
       m_chan(chan), m_watcher(0), m_selfWatch(false), m_reenter(false), m_setdata(true),
       m_timeout(s_timeout), m_timebomb(s_timebomb), m_restart(false),
-      m_script(script), m_args(args)
+      m_script(script), m_args(args), m_trackName(s_trackName)
 {
     Debug(DebugAll,"ExtModReceiver::ExtModReceiver(\"%s\",\"%s\") [%p]",script,args,this);
     m_script.trimBlanks();
@@ -724,7 +734,7 @@ ExtModReceiver::ExtModReceiver(const char* name, Stream* io, ExtModChan* chan, i
       m_in(io), m_out(io), m_ain(0), m_aout(0),
       m_chan(chan), m_watcher(0), m_selfWatch(false), m_reenter(false), m_setdata(true),
       m_timeout(s_timeout), m_timebomb(s_timebomb), m_restart(false),
-      m_script(name)
+      m_script(name), m_trackName(s_trackName)
 {
     Debug(DebugAll,"ExtModReceiver::ExtModReceiver(\"%s\",%p,%p) [%p]",name,io,chan,this);
     m_script.trimBlanks();
@@ -1287,7 +1297,7 @@ bool ExtModReceiver::processLine(const char* line)
 	lock();
 	bool ok = id && !m_dead && !m_relays.find(id);
 	if (ok) {
-	    MessageRelay *r = new MessageRelay(id,this,0,prio);
+	    MessageRelay *r = new MessageRelay(id,this,0,prio,m_trackName);
 	    if (fname)
 		r->setFilter(fname,fvalue);
 	    m_relays.append(r);
@@ -1353,6 +1363,7 @@ bool ExtModReceiver::processLine(const char* line)
 	int col = id.find(':');
 	if (col > 0) {
 	    String val(id.substr(col+1));
+	    val.trimBlanks();
 	    id = id.substr(0,col);
 	    bool ok = false;
 	    Lock mylock(this);
@@ -1370,6 +1381,13 @@ bool ExtModReceiver::processLine(const char* line)
 		val = m_chan->disconn();
 		ok = true;
 	    }
+	    else if (id == "trackparam") {
+		if (val.null())
+		    val = m_trackName;
+		else
+		    m_trackName = val;
+		ok = true;
+	    }
 	    else if (id == "reason") {
 		m_reason = val;
 		ok = true;
@@ -1540,7 +1558,7 @@ bool ExtModHandler::received(Message& msg)
     static const Regexp r("^external/\\([^/]*\\)/\\([^ ]*\\)\\(.*\\)$");
     if (!dest.matches(r))
 	return false;
-    CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
+    CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     String t = dest.matchString(1);
     int typ = 0;
     if (t == "nochan")
@@ -1826,6 +1844,8 @@ void ExtModulePlugin::initialize()
     s_cfg.load();
     s_timeout = s_cfg.getIntValue("general","timeout",MSG_TIMEOUT);
     s_timebomb = s_cfg.getBoolValue("general","timebomb",false);
+    s_trackName = s_cfg.getBoolValue("general","trackparam",false) ?
+	name().c_str() : (const char*)0;
     int wf = s_cfg.getIntValue("general","waitflush",WAIT_FLUSH);
     if (wf < 1)
 	wf = 1;
@@ -1869,8 +1889,6 @@ void ExtModulePlugin::initialize()
     }
 }
 
-INIT_PLUGIN(ExtModulePlugin);
-
 }; // anonymous namespace
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
