@@ -212,13 +212,6 @@ private:
     int m_pages;
 };
 
-class FaxHandler : public MessageHandler
-{
-public:
-    FaxHandler(const char *name) : MessageHandler(name) { }
-    virtual bool received(Message &msg);
-};
-
 // Driver and plugin
 class FaxDriver : public Driver
 {
@@ -241,6 +234,15 @@ static const TokenDict s_types[] = {
     { 0, 0 }
 };
 
+class FaxHandler : public MessageHandler
+{
+public:
+    FaxHandler(const char *name)
+	: MessageHandler(name,100,plugin.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
 FaxSource::FaxSource(FaxWrapper* wrapper, const char* format)
     : DataSource(format), m_wrap(wrapper)
 {
@@ -804,7 +806,7 @@ bool FaxDriver::msgExecute(Message& msg, String& dest)
     dest = dest.matchString(2);
 
     RefPointer<FaxChan> fc;
-    CallEndpoint* ce = static_cast<CallEndpoint *>(msg.userData());
+    CallEndpoint* ce = YOBJECT(CallEndpoint,msg.userData());
     if (ce) {
 	fc = new FaxChan(true,dest,transmit,msg);
 	fc->initChan();
@@ -823,6 +825,7 @@ bool FaxDriver::msgExecute(Message& msg, String& dest)
 	Message m("call.route");
 	fc->complete(m);
 	fc->setParams(m,fc->guessType(msg),FaxChan::guessT38(msg));
+	m.copyParams(msg,msg[YSTRING("copyparams")]);
 	m.userData(fc);
 	String callto = msg.getValue("caller");
 	if (callto)
