@@ -86,26 +86,7 @@ private:
     static int s_nextid;
 };
 
-class MOHHandler : public MessageHandler
-{
-public:
-    MOHHandler() : MessageHandler("call.execute") { }
-    virtual bool received(Message &msg);
-};
-
-class AttachHandler : public MessageHandler
-{
-public:
-    AttachHandler() : MessageHandler("chan.attach") { }
-    virtual bool received(Message &msg);
-};
-
-class StatusHandler : public MessageHandler
-{
-public:
-    StatusHandler() : MessageHandler("engine.status") { }
-    virtual bool received(Message &msg);
-};
+class MOHHandler;
 
 class MOHPlugin : public Plugin
 {
@@ -117,6 +98,35 @@ private:
     MOHHandler *m_handler;
 };
 
+INIT_PLUGIN(MOHPlugin);
+
+class MOHHandler : public MessageHandler
+{
+public:
+    MOHHandler()
+	: MessageHandler("call.execute",100,__plugin.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+class AttachHandler : public MessageHandler
+{
+public:
+    AttachHandler()
+	: MessageHandler("chan.attach",100,__plugin.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+class StatusHandler : public MessageHandler
+{
+public:
+    StatusHandler()
+	: MessageHandler("engine.status",100,__plugin.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
 
 MOHSource::MOHSource(const String &name, const String &command_line, unsigned int rate)
     : ThreadedSource("slin"),
@@ -328,7 +338,7 @@ bool MOHHandler::received(Message &msg)
     if (!dest.matches(r))
 	return false;
     String name = dest.matchString(1);
-    CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
+    CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (ch) {
 	MOHChan* mc = new MOHChan(name,msg);
 	if (ch->connect(mc,msg.getValue("reason"))) {
@@ -458,8 +468,6 @@ void MOHPlugin::initialize()
     }
 }
 
-INIT_PLUGIN(MOHPlugin);
-
 }; // anonymous namespace
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
