@@ -27,46 +27,6 @@
 using namespace TelEngine;
 namespace { // anonymous
 
-// chan.connect handler used to connect two channels
-class ConnHandler : public MessageHandler
-{
-public:
-    ConnHandler(int prio = 90)
-	: MessageHandler("chan.connect",prio)
-	{ }
-    virtual bool received(Message &msg);
-};
-
-// call.execute handler used to 'steal' a channel
-class ChanPickup : public MessageHandler
-{
-public:
-    ChanPickup(int prio = 100)
-	: MessageHandler("call.execute",prio)
-	{ }
-    virtual bool received(Message &msg);
-};
-
-// chan.attach handler used for detaching data nodes by message
-class AttachHandler : public MessageHandler
-{
-public:
-    AttachHandler(int prio = 100)
-	: MessageHandler("chan.attach",prio)
-	{ }
-    virtual bool received(Message &msg);
-};
-
-// chan.record handler used for detaching data nodes by message
-class RecordHandler : public MessageHandler
-{
-public:
-    RecordHandler(int prio = 100)
-	: MessageHandler("chan.record",prio)
-	{ }
-    virtual bool received(Message &msg);
-};
-
 class PbxModule : public Module
 {
 public:
@@ -81,6 +41,47 @@ static PbxModule s_module;
 static const String s_masquerade("chan.masquerade");
 
 
+// chan.connect handler used to connect two channels
+class ConnHandler : public MessageHandler
+{
+public:
+    ConnHandler(int prio = 90)
+	: MessageHandler("chan.connect",prio,s_module.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+// call.execute handler used to 'steal' a channel
+class ChanPickup : public MessageHandler
+{
+public:
+    ChanPickup(int prio = 100)
+	: MessageHandler("call.execute",prio,s_module.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+// chan.attach handler used for detaching data nodes by message
+class AttachHandler : public MessageHandler
+{
+public:
+    AttachHandler(int prio = 100)
+	: MessageHandler("chan.attach",prio,s_module.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+// chan.record handler used for detaching data nodes by message
+class RecordHandler : public MessageHandler
+{
+public:
+    RecordHandler(int prio = 100)
+	: MessageHandler("chan.record",prio,s_module.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+
 // Utility function to get a pointer to a call endpoint (or its peer) by id
 static CallEndpoint* locateChan(const String& id, bool peer = false)
 {
@@ -124,7 +125,7 @@ bool ChanPickup::received(Message& msg)
 	return false;
 
     // It's ours. Get the channels
-    RefPointer<CallEndpoint> caller(static_cast<CallEndpoint*>(msg.userData()));
+    RefPointer<CallEndpoint> caller(YOBJECT(CallEndpoint,msg.userData()));
     RefPointer<CallEndpoint> called(locateChan(callto,msg.getBoolValue("pickup_peer",true)));
 
     if (!caller) {
