@@ -39,6 +39,9 @@
 using namespace TelEngine;
 namespace { // anonymous
 
+#define DEF_HISTORY 10
+#define MAX_HISTORY 50
+
 typedef struct {
     const char* name;
     const char* args;
@@ -69,6 +72,12 @@ static const char* s_level[] =
     0
 };
 
+static const char* s_debug[] =
+{
+    "threshold",
+    0
+};
+
 static const char* s_oview[] =
 {
     "overview",
@@ -89,15 +98,20 @@ static const char* s_rnow[] =
 
 static const CommandInfo s_cmdInfo[] =
 {
+    // Unauthenticated commands
     { "quit", 0, 0, "Disconnect this control session from Yate" },
+    { "echo", "[on|off]", s_bools, "Show or turn remote echo on or off" },
     { "help", "[command]", 0, "Provide help on all or given command" },
+    { "auth", "password", 0, "Authenticate so you can access privileged commands" },
+
+    // User commands
     { "status", "[overview] [modulename]", s_oview, "Shows status of all or selected modules or channels" },
     { "uptime", 0, 0, "Show information on how long Yate has run" },
-    { "echo", "[on|off]", s_bools, "Show or turn remote echo on or off" },
     { "machine", "[on|off]", s_bools, "Show or turn machine output mode on or off" },
     { "output", "[on|off]", s_bools, "Show or turn local output on or off" },
     { "color", "[on|off]", s_bools, "Show status or turn local colorization on or off" },
-    { "auth", "password", 0, "Authenticate so you can access priviledged commands" },
+
+    // Admin commands
     { "debug", "[module] [level|on|off]", s_level, "Show or change debugging level globally or per module" },
 #ifdef HAVE_COREDUMPER
     { "coredump", "[filename]", 0, "Dumps memory image of running Yate to a file" },
@@ -105,9 +119,10 @@ static const CommandInfo s_cmdInfo[] =
     { "drop", "{chan|*|all} [reason]", s_dall, "Drops one or all active calls" },
     { "call", "chan target", 0, "Execute an outgoing call" },
     { "control", "chan [operation] [param=val] [param=...]", 0, "Apply arbitrary control operations to a channel or entity" },
-    { "reload", 0, 0, "Reloads module configuration files" },
+    { "reload", "[plugin]", 0, "Reloads module configuration files" },
     { "restart", "[now]", s_rnow, "Restarts the engine if executing supervised" },
     { "stop", "[exitcode]", 0, "Stops the engine with optionally provided exit code" },
+    { "alias", "[name [command...]]", 0, "Create an alias for a longer command" },
     { 0, 0, 0, 0 }
 };
 
@@ -122,60 +137,116 @@ static void completeWords(String& str, const char** list, const char* partial =
     }
 }
 
-static Configuration s_cfg;
-
-static bool s_output = false;
-
-/* I need this here because i'm gonna use it in both classes */
-Socket s_sock;
-Mutex s_mutex(true,"RManager");
+static Mutex s_mutex(true,"RManager");
 
 //we gonna create here the list with all the new connections.
-static ObjList connectionlist;
-    
+static ObjList s_connList;
+
+// the incomming connections listeners list
+static ObjList s_listeners;
+
+class Connection;
+class RManagerThread;
+
+class SockRef : public RefObject
+{
+public:
+    inline SockRef(Socket** sock)
+	: m_sock(sock)
+	{ }
+    void* getObject(const String& name) const
+    {
+	if (name == "Socket*")
+	    return m_sock;
+	return RefObject::getObject(name);
+    }
+private:
+    Socket** m_sock;
+};
+
+class RManagerListener : public RefObject
+{
+    friend class RManagerThread;
+public:
+    inline RManagerListener(const NamedList& sect)
+	: m_cfg(sect)
+	{ }
+    ~RManagerListener();
+    void init();
+    inline NamedList& cfg()
+	{ return m_cfg; }
+private:
+    void run();
+    bool initSocket();
+    Connection* checkCreate(Socket* sock, const char* addr);
+    NamedList m_cfg;
+    Socket m_socket;
+    String m_address;
+};
+
 class RManagerThread : public Thread
 {
 public:
-    RManagerThread() : Thread("RManager Listener") { }
-    virtual void run();
+    inline RManagerThread(RManagerListener* listener)
+	: Thread("RManager Listener"), m_listener(listener)
+	{ }
+    virtual void run()
+	{ m_listener->run(); }
 private:
+    RefPointer<RManagerListener> m_listener;
 };
 
 class Connection : public GenObject, public Thread
 {
 public:
-    Connection(Socket* sock, const char* addr);
+    enum Level {
+	None = 0,
+	User,
+	Admin
+    };
+    Connection(Socket* sock, const char* addr, RManagerListener* listener);
     ~Connection();
 
     virtual void run();
     bool processTelnetChar(unsigned char c);
     bool processChar(unsigned char c);
-    bool processLine(const char *line);
+    bool processLine(const char *line, bool saveLine = true);
     bool autoComplete();
     void errorBeep();
     void clearLine();
+    void writeBuffer();
+    void writeBufferTail(bool eraseOne = false);
     void writeStr(const char *str, int len = -1);
     void writeDebug(const char *str, int level);
+    void writeEvent(const char *str, int level);
     void writeStr(const Message &msg, bool received);
     inline void writeStr(const String &s)
 	{ writeStr(s.safe(),s.length()); }
     inline const String& address() const
 	{ return m_address; }
-    static Connection *checkCreate(Socket* sock, const char* addr = 0);
+    inline const NamedList& cfg() const
+	{ return m_listener->cfg(); }
+    void checkTimer(u_int64_t time);
 private:
-    bool m_auth;
+    NamedList m_aliases;
+    Level m_auth;
     bool m_debug;
     bool m_output;
     bool m_colorize;
     bool m_machine;
+    int m_threshold;
     Socket* m_socket;
     unsigned char m_lastch;
     unsigned char m_escmode;
     bool m_echoing;
     bool m_beeping;
+    u_int64_t m_timeout;
     String m_buffer;
     String m_address;
-    String m_lastcmd;
+    ObjList m_history;
+    RefPointer<RManagerListener> m_listener;
+    unsigned int m_cursorPos;
+    unsigned int m_histLen;
 };
 
 class RManager : public Plugin
@@ -195,10 +266,11 @@ public:
     virtual void dispatched(const Message& msg, bool handled);
 };
 
+
 static void dbg_remote_func(const char *buf, int level)
 {
     s_mutex.lock();
-    ObjList *p = &connectionlist;
+    ObjList* p = &s_connList;
     for (; p; p=p->next()) {
 	Connection *con = static_cast<Connection *>(p->get());
 	if (con)
@@ -207,36 +279,127 @@ static void dbg_remote_func(const char *buf, int level)
     s_mutex.unlock();
 }
 
-void RManagerThread::run()
+
+RManagerListener::~RManagerListener()
+{
+    DDebug("RManager",DebugInfo,"No longer listening '%s' on %s",
+	m_cfg.c_str(),m_address.c_str());
+    s_mutex.lock();
+    s_listeners.remove(this,false);
+    s_mutex.unlock();
+}
+
+void RManagerListener::init()
+{
+    if (initSocket()) {
+	s_mutex.lock();
+	s_listeners.append(this);
+	s_mutex.unlock();
+    }
+    deref();
+}
+
+bool RManagerListener::initSocket()
+{
+    // check configuration
+    int port = m_cfg.getIntValue("port",5038);
+    const char* host = c_safe(m_cfg.getValue("addr","127.0.0.1"));
+    if (!(port && *host))
+	return false;
+
+    m_socket.create(AF_INET, SOCK_STREAM);
+    if (!m_socket.valid()) {
+	Debug("RManager",DebugGoOn,"Unable to create the listening socket: %s",
+	    strerror(m_socket.error()));
+	return false;
+    }
+
+    if (!m_socket.setBlocking(false)) {
+	Debug("RManager",DebugGoOn, "Failed to set listener to nonblocking mode: %s",
+	    strerror(m_socket.error()));
+	return false;
+    }
+
+    SocketAddr sa(AF_INET);
+    sa.host(host);
+    sa.port(port);
+    m_address << sa.host() << ":" << sa.port();
+    m_socket.setReuse();
+    if (!m_socket.bind(sa)) {
+	Debug("RManager",DebugGoOn,"Failed to bind to %s : %s",
+	    m_address.c_str(),strerror(m_socket.error()));
+	return false;
+    }
+    if (!m_socket.listen(2)) {
+	Debug("RManager",DebugGoOn,"Unable to listen on socket: %s",
+	    strerror(m_socket.error()));
+	return false;
+    }
+    Debug("RManager",DebugInfo,"Starting listener '%s' on %s",
+	m_cfg.c_str(),m_address.c_str());
+    RManagerThread* t = new RManagerThread(this);
+    if (t->startup())
+	return true;
+    delete t;
+    return false;
+}
+
+void RManagerListener::run()
 {
     for (;;)
     {
 	Thread::idle(true);
 	SocketAddr sa;
-	Socket* as = s_sock.accept(sa);
+	Socket* as = m_socket.accept(sa);
 	if (!as) {
-	    if (!s_sock.canRetry())
-		Debug("RManager",DebugWarn, "Accept error: %s", strerror(s_sock.error()));
+	    if (!m_socket.canRetry())
+		Debug("RManager",DebugWarn, "Accept error: %s",strerror(m_socket.error()));
 	    continue;
 	} else {
 	    String addr(sa.host());
 	    addr << ":" << sa.port();
-	    if (!Connection::checkCreate(as,addr))
+	    if (!checkCreate(as,addr))
 		Debug("RManager",DebugWarn,"Connection rejected for %s",addr.c_str());
 	}
     }
 }
 
-Connection *Connection::checkCreate(Socket* sock, const char* addr)
+Connection* RManagerListener::checkCreate(Socket* sock, const char* addr)
 {
-    if (!sock)
-	return 0;
     if (!sock->valid()) {
 	delete sock;
 	return 0;
     }
+    const NamedString* secure = m_cfg.getParam("context");
+    if (TelEngine::null(secure))
+	secure = m_cfg.getParam("domain");
+    if (TelEngine::null(secure))
+	secure = 0;
+    if (secure) {
+	Message m("socket.ssl");
+	m.addParam("server",String::boolText(true));
+	m.addParam(secure->name(),*secure);
+	m.copyParam(m_cfg,"verify");
+	SockRef* s = new SockRef(&sock);
+	m.userData(s);
+	TelEngine::destruct(s);
+	if (!(Engine::dispatch(m) && sock)) {
+	    Debug("RManager",DebugWarn, "Failed to switch '%s' to SSL for %s '%s'",
+		cfg().c_str(),secure->name().c_str(),secure->c_str());
+	    delete sock;
+	    return 0;
+	}
+    }
+    else if (!sock->setBlocking(false)) {
+	Debug("RManager",DebugGoOn, "Failed to set tcp socket to nonblocking mode: %s",
+	    strerror(sock->error()));
+	delete sock;
+	return 0;
+    }
     // should check IP address here
-    Connection *conn = new Connection(sock,addr);
+    Output("Remote%s connection from %s to %s",
+	(secure ? " secure" : ""),addr,m_address.c_str());
+    Connection* conn = new Connection(sock,addr,this);
     if (conn->error()) {
 	conn->destruct();
 	return 0;
@@ -245,14 +408,17 @@ Connection *Connection::checkCreate(Socket* sock, const char* addr)
     return conn;
 }
 
-Connection::Connection(Socket* sock, const char* addr)
+
+Connection::Connection(Socket* sock, const char* addr, RManagerListener* listener)
     : Thread("RManager Connection"),
-      m_auth(false), m_debug(false), m_output(s_output), m_colorize(false), m_machine(false),
+      m_aliases(""),
+      m_auth(None), m_debug(false), m_output(false), m_colorize(false), m_machine(false),
+      m_threshold(DebugAll),
       m_socket(sock), m_lastch(0), m_escmode(0), m_echoing(false), m_beeping(false),
-      m_address(addr)
+      m_timeout(0), m_address(addr), m_listener(listener), m_cursorPos(0), m_histLen(DEF_HISTORY)
 {
     s_mutex.lock();
-    connectionlist.append(this);
+    s_connList.append(this);
     s_mutex.unlock();
 }
 
@@ -261,7 +427,7 @@ Connection::~Connection()
     m_debug = false;
     m_output = false;
     s_mutex.lock();
-    connectionlist.remove(this,false);
+    s_connList.remove(this,false);
     s_mutex.unlock();
     Output("Closing connection to %s",m_address.c_str());
     delete m_socket;
@@ -272,22 +438,34 @@ void Connection::run()
 {
     if (!m_socket)
 	return;
-    if (!m_socket->setBlocking(false)) {
-	Debug("RManager",DebugGoOn, "Failed to set tcp socket to nonblocking mode: %s",strerror(m_socket->error()));
-	return;
-    }
 
     // For the sake of responsiveness try to turn off the tcp assembly timer
     int arg = 1;
-    if (s_cfg.getBoolValue("general","interactive",false) &&
+    if (cfg().getBoolValue("interactive",false) &&
 	!m_socket->setOption(SOL_SOCKET, TCP_NODELAY, &arg, sizeof(arg)))
 	Debug("RManager",DebugMild, "Failed to set tcp socket to TCP_NODELAY mode: %s", strerror(m_socket->error()));
 
-    Output("Remote connection from %s",m_address.c_str());
-    m_auth = !s_cfg.getValue("general","password");
-    String hdr = s_cfg.getValue("general","header","YATE ${version}-${release} (http://YATE.null.ro) ready.");
+    if (cfg().getValue("userpass")) {
+	int tout = cfg().getIntValue("timeout",30000);
+	if (tout > 0) {
+	    if (tout < 5000)
+		tout = 5000;
+	    m_timeout = Time::now() + 1000 * tout;
+	}
+    }
+    else {
+	m_auth = cfg().getValue("password") ? User : Admin;
+	m_output = cfg().getBoolValue("output",false);
+	if (Admin == m_auth)
+	    m_debug = cfg().getBoolValue("debug",false);
+    }
+    m_histLen = cfg().getIntValue("maxhistory",DEF_HISTORY);
+    if (m_histLen > MAX_HISTORY)
+	m_histLen = MAX_HISTORY;
+    String hdr = cfg().getValue("header","YATE ${version}-${release} (http://YATE.null.ro) ready on ${nodename}.");
     Engine::runParams().replaceParams(hdr);
-    if (s_cfg.getBoolValue("general","telnet",true)) {
+    if (cfg().getBoolValue("telnet",true)) {
+	m_colorize = cfg().getBoolValue("color",false);
 	// WILL SUPPRESS GO AHEAD, WILL ECHO - and enough BS and blanks to hide them
 	writeStr("\377\373\003\377\373\001\r      \b\b\b\b\b\b");
     }
@@ -295,8 +473,16 @@ void Connection::run()
 	writeStr("\r" + hdr + "\r\n");
 	hdr.clear();
     }
+    NamedIterator iter(cfg());
+    while (const NamedString* s = iter.get()) {
+	if (s->null() || !s->name().startsWith("alias:"))
+	    continue;
+	String name = s->name().substr(6).trimSpaces();
+	if (name)
+	    m_aliases.setParam(name,*s);
+    }
     unsigned char buffer[128];
-    for (;;) {
+    while (m_socket && m_socket->valid()) {
 	Thread::check();
 	bool readok = false;
 	bool error = false;
@@ -328,7 +514,7 @@ void Connection::run()
 	    Debug("RManager",DebugWarn,"socket select error %d on %d",errno,m_socket->handle());
 	    return;
 	}
-    }	
+    }
 }
 
 // generates a beep - just one per processed buffer
@@ -346,6 +532,30 @@ void Connection::clearLine()
     writeStr("\r\033[K\r");
 }
 
+// write just the tail of the current buffer, get back the cursor
+void Connection::writeBufferTail(bool eraseOne)
+{
+    String tail = m_buffer.substr(m_cursorPos);
+    if (eraseOne)
+	tail += " ";
+    writeStr(tail);
+    // now write enough backspaces to get back
+    tail.assign('\b',tail.length());
+    writeStr(tail);
+}
+
+// write the current buffer, leave the cursor in the right place
+void Connection::writeBuffer()
+{
+    if (m_cursorPos == m_buffer.length()) {
+	writeStr(m_buffer);
+	return;
+    }
+    if (m_cursorPos)
+	writeStr(m_buffer,m_cursorPos);
+    writeBufferTail();
+}
+
 // process incoming telnet characters
 bool Connection::processTelnetChar(unsigned char c)
 {
@@ -433,6 +643,8 @@ bool Connection::processTelnetChar(unsigned char c)
 // process incoming terminal characters
 bool Connection::processChar(unsigned char c)
 {
+    bool atEol = (m_buffer.length() == m_cursorPos);
+    XDebug(DebugAll,"cur=%u len=%u '%s'",m_cursorPos,m_buffer.length(),m_buffer.safe());
     switch (c) {
 	case '\0':
 	    m_escmode = 0;
@@ -444,6 +656,7 @@ bool Connection::processChar(unsigned char c)
 	    m_escmode = 0;
 	    if (m_buffer.null())
 		return false;
+	// fall through
 	case '\r':
 	    m_escmode = 0;
 	    if (m_echoing)
@@ -451,6 +664,7 @@ bool Connection::processChar(unsigned char c)
 	    if (processLine(m_buffer))
 		return true;
 	    m_buffer.clear();
+	    m_cursorPos = 0;
 	    return false;
 	case 0x03: // ^C, BREAK
 	    m_escmode = 0;
@@ -462,23 +676,27 @@ bool Connection::processChar(unsigned char c)
 		errorBeep();
 		return false;
 	    }
-	    return processLine("quit");
+	    return processLine("quit",false);
 	case 0x1C: // ^backslash
 	    if (m_buffer)
 		break;
-	    return processLine("reload");
+	    return processLine("reload",false);
 	case 0x05: // ^E
 	    m_escmode = 0;
 	    m_echoing = !m_echoing;
 	    return false;
 	case 0x0F: // ^O
 	    m_escmode = 0;
-	    // cycle [no output] -> [output] -> [debug (only if auth)]
+	    if (m_auth < User) {
+		errorBeep();
+		return false;
+	    }
+	    // cycle [no output] -> [output] -> [debug (only if admin)]
 	    if (m_debug)
 		m_output = m_debug = false;
 	    else if (m_output) {
 		m_output = false;
-		if ((m_debug = m_auth))
+		if ((m_debug = (m_auth >= Admin)))
 		    Debugger::enableOutput(true);
 	    }
 	    else
@@ -488,27 +706,28 @@ bool Connection::processChar(unsigned char c)
 	    if (!m_echoing)
 		break;
 	    writeStr("\033[H\033[2J");
-	    writeStr(m_buffer);
+	    writeBuffer();
 	    return false;
 	case 0x12: // ^R
 	    if (!m_echoing)
 		break;
 	    clearLine();
-	    writeStr(m_buffer);
+	    writeBuffer();
 	    return false;
 	case 0x15: // ^U
 	    if (m_buffer.null())
 		break;
 	    m_escmode = 0;
 	    m_buffer.clear();
+	    m_cursorPos = 0;
 	    if (m_echoing)
 		clearLine();
 	    return false;
 	case 0x17: // ^W
-	    if (m_buffer.null())
+	    if (m_cursorPos <= 0)
 		errorBeep();
 	    else {
-		int i = m_buffer.length()-1;
+		int i = m_cursorPos-1;
 		for (; i > 0; i--)
 		    if (m_buffer[i] != ' ')
 			break;
@@ -518,28 +737,45 @@ bool Connection::processChar(unsigned char c)
 			break;
 		    }
 		m_escmode = 0;
-		m_buffer.assign(m_buffer,i);
+		m_buffer = m_buffer.substr(0,i) + m_buffer.substr(m_cursorPos);
+		m_cursorPos = i;
 		if (m_echoing) {
 		    clearLine();
-		    writeStr(m_buffer);
+		    writeBuffer();
 		}
 	    }
 	    return false;
 	case 0x7F: // DEL
 	case 0x08: // ^H, BACKSPACE
-	    if (m_buffer.null()) {
+	    if (!m_cursorPos) {
 		errorBeep();
 		return false;
 	    }
 	    m_escmode = 0;
-	    m_buffer.assign(m_buffer,m_buffer.length()-1);
-	    if (m_echoing)
-		writeStr("\b \b");
+	    if (atEol) {
+		m_buffer.assign(m_buffer,--m_cursorPos);
+		if (m_echoing)
+		    writeStr("\b \b");
+	    }
+	    else {
+		m_buffer = m_buffer.substr(0,m_cursorPos-1) + m_buffer.substr(m_cursorPos);
+		m_cursorPos--;
+		if (m_echoing) {
+		    writeStr("\b");
+		    writeBufferTail(true);
+		}
+	    }
 	    return false;
 	case 0x09: // ^I, TAB
 	    m_escmode = 0;
 	    if (m_buffer.null())
-		return processLine("help");
+		return processLine("help",false);
+	    if (!atEol) {
+		if (m_echoing)
+		    writeStr(m_buffer.c_str()+m_cursorPos,m_buffer.length()-m_cursorPos);
+		m_cursorPos = m_buffer.length();
+		return false;
+	    }
 	    if (!autoComplete())
 		errorBeep();
 	    return false;
@@ -562,27 +798,98 @@ bool Connection::processChar(unsigned char c)
 		m_escmode = c;
 		return false;
 	}
+	char escMode = m_escmode;
+	m_escmode = 0;
 	DDebug("RManager",DebugInfo,"ANSI '%s%c' last '%s%c'",
 	    (c >= ' ') ? "" : "^", (c >= ' ') ? c : c+0x40,
-	    (m_escmode >= ' ') ? "" : "^", (m_escmode >= ' ') ? m_escmode : m_escmode+0x40
+	    (escMode >= ' ') ? "" : "^", (escMode >= ' ') ? escMode : escMode+0x40
 	);
-	m_escmode = 0;
 	switch (c) {
 	    case 'A': // Up arrow
+		{
+		    String* s = static_cast<String*>(m_history.remove(false));
+		    if (m_buffer)
+			m_history.append(new String(m_buffer));
+		    m_buffer = s;
+		    TelEngine::destruct(s);
+		}
+		clearLine();
+		m_cursorPos = m_buffer.length();
+		writeBuffer();
+		return false;
 	    case 'B': // Down arrow
-		if (m_lastcmd.null()) {
+		{
+		    ObjList* l = &m_history;
+		    while (l->skipNext())
+			l = l->skipNext();
+		    String* s = static_cast<String*>(l->get());
+		    m_history.remove(s,false);
+		    if (m_buffer)
+			m_history.insert(new String(m_buffer));
+		    m_buffer = s;
+		    TelEngine::destruct(s);
+		}
+		clearLine();
+		m_cursorPos = m_buffer.length();
+		writeBuffer();
+		return false;
+	    case 'C': // Right arrow
+		if (atEol || m_buffer.null()) {
 		    errorBeep();
 		    return false;
 		}
-		else {
-		    String str = m_lastcmd;
-		    if (m_buffer)
-			m_lastcmd = m_buffer;
-		    m_buffer = str;
-		}
-		clearLine();
-		writeStr(m_buffer);
+		if (m_echoing)
+		    writeStr(m_buffer.c_str()+m_cursorPos,1);
+		m_cursorPos++;
 		return false;
+	    case 'D': // Left arrow
+		if (!m_cursorPos || m_buffer.null()) {
+		    errorBeep();
+		    return false;
+		}
+		if (m_echoing)
+		    writeStr("\b");
+		m_cursorPos--;
+		return false;
+	    case 'H': // Home
+		if (m_echoing)
+		    writeStr("\r");
+		m_cursorPos = 0;
+		return false;
+	    case 'F': // End
+		if (atEol)
+		    return false;
+		if (m_echoing && m_buffer)
+		    writeStr(m_buffer.c_str()+m_cursorPos);
+		m_cursorPos = m_buffer.length();
+		return false;
+	    case '~':
+		switch (escMode) {
+		    case '1': // Home
+			if (m_echoing)
+			    writeStr("\r");
+			m_cursorPos = 0;
+			return false;
+		    case '4': // End
+			if (atEol)
+			    return false;
+			if (m_echoing && m_buffer)
+			    writeStr(m_buffer.c_str()+m_cursorPos);
+			m_cursorPos = m_buffer.length();
+			return false;
+		    case '3': // Delete
+			if (atEol || m_buffer.null()) {
+			    errorBeep();
+			    return false;
+			}
+			m_buffer = m_buffer.substr(0,m_cursorPos) + m_buffer.substr(m_cursorPos+1);
+			writeBufferTail(true);
+			return false;
+		    //case '2': // Insert
+		    //case '5': // Page up
+		    //case '6': // Page down
+		}
+		break;
 	}
 	c = 0;
     }
@@ -591,14 +898,28 @@ bool Connection::processChar(unsigned char c)
 	return false;
     }
     if (m_echoing && (c == ' ')) {
-	if (m_buffer.null() || m_buffer.endsWith(" ")) {
+	if (m_buffer.null() || (atEol && m_buffer.endsWith(" "))) {
 	    errorBeep();
 	    return false;
 	}
     }
-    m_buffer += (char)c;
-    if (m_echoing)
-	writeStr((char*)&c,1);
+    if (atEol) {
+	// append char
+	m_buffer += (char)c;
+	m_cursorPos = m_buffer.length();
+	if (m_echoing)
+	    writeStr((char*)&c,1);
+    }
+    else {
+	// insert char
+	String tmp((char)c);
+	m_buffer = m_buffer.substr(0,m_cursorPos) + tmp + m_buffer.substr(m_cursorPos);
+	m_cursorPos++;
+	if (m_echoing) {
+	    writeStr(tmp);
+	    writeBufferTail(m_cursorPos);
+	}
+    }
     return false;
 }
 
@@ -653,6 +974,11 @@ bool Connection::autoComplete()
 		if (cmd.startsWith(partWord))
 		    m.retValue().append(cmd,"\t");
 	    }
+	    NamedIterator iter(m_aliases);
+	    while (const NamedString* s = iter.get()) {
+		if (s->name().startsWith(partWord))
+		    m.retValue().append(s->name(),"\t");
+	    }
 	}
 	else {
 	    const CommandInfo* info = s_cmdInfo;
@@ -681,16 +1007,36 @@ bool Connection::autoComplete()
 	m.addParam("partword",partWord);
     if ((partLine == "status") || (partLine == "debug") || (partLine == "drop"))
 	m.setParam("complete","channels");
-    Regexp r("^debug [^ ]\\+$");
-    if (r.matches(partLine))
+    static const Regexp r("^debug \\([^ ]\\+\\)$");
+    if (partLine == "debug")
+	completeWords(m.retValue(),s_debug,partWord);
+    else while (partLine.matches(r)) {
+	String tmp = partLine.matchString(1);
+	const char** lvl = s_level;
+	for (; *lvl; lvl++) {
+	    if (tmp == *lvl)
+		break;
+	}
+	if (*lvl)
+	    break;
+	for (lvl = s_debug; *lvl; lvl++) {
+	    if (tmp == *lvl)
+		break;
+	}
+	if (*lvl)
+	    break;
 	completeWords(m.retValue(),s_level,partWord);
-    Engine::dispatch(m);
+	break;
+    }
+    if (m_auth >= Admin)
+	Engine::dispatch(m);
     if (m.retValue().null())
 	return false;
     if (m.retValue().find('\t') < 0) {
 	m_buffer = m_buffer.substr(0,keepLen)+m.retValue()+" ";
+	m_cursorPos = m_buffer.length();
 	clearLine();
-	writeStr(m_buffer);
+	writeBuffer();
 	return true;
     }
     // more options returned - list them and display the prompt again
@@ -715,13 +1061,14 @@ bool Connection::autoComplete()
     }
     TelEngine::destruct(l);
     m_buffer += maxMatch.substr(partWord.length());
+    m_cursorPos = m_buffer.length();
     writeStr("\r\n");
-    writeStr(m_buffer);
+    writeBuffer();
     return true;
 }
 
 // execute received input line
-bool Connection::processLine(const char *line)
+bool Connection::processLine(const char *line, bool saveLine)
 {
     DDebug("RManager",DebugInfo,"processLine = '%s'",line);
     String str(line);
@@ -729,17 +1076,112 @@ bool Connection::processLine(const char *line)
     if (str.null())
 	return false;
 
-    m_lastcmd = str;
+    if (saveLine) {
+	m_history.remove(str);
+	while (GenObject* obj = m_history[m_histLen])
+	    m_history.remove(obj);
+	m_history.insert(new String(str));
+    }
+
     line = 0;
     m_buffer.clear();
 
+    if (str.startSkip("quit"))
+    {
+	writeStr(m_machine ? "%%=quit\r\n" : "Goodbye!\r\n");
+	return true;
+    }
+    else if (str.startSkip("echo"))
+    {
+	str >> m_echoing;
+	str = "Remote echo: ";
+	str += (m_echoing ? "on\r\n" : "off\r\n");
+	writeStr(str);
+	return false;
+    }
+    else if (str.startSkip("help") || str.startSkip("?"))
+    {
+	if (str)
+	{
+	    Message m("engine.help");
+	    const CommandInfo* info = s_cmdInfo;
+	    for (; info->name; info++) {
+		if (str == info->name) {
+		    str = "  ";
+		    str << info->name;
+		    if (info->args)
+			str << " " << info->args;
+		    str << "\r\n" << info->desc << "\r\n";
+		    writeStr(str);
+		    return false;
+		}
+	    }
+	    m.addParam("line",str);
+	    if ((m_auth >= Admin) && Engine::dispatch(m))
+		writeStr(m.retValue());
+	    else
+		writeStr("No help for '"+str+"'\r\n");
+	}
+	else
+	{
+	    Message m("engine.help",0,true);
+	    m.retValue() = "Available commands:\r\n";
+	    const CommandInfo* info = s_cmdInfo;
+	    for (; info->name; info++) {
+		m.retValue() << "  " << info->name;
+		if (info->args)
+		    m.retValue() << " " << info->args;
+		m.retValue() << "\r\n";
+	    }
+	    if (m_auth >= Admin)
+		Engine::dispatch(m);
+	    writeStr(m.retValue());
+	}
+	return false;
+    }
+    else if (str.startSkip("auth"))
+    {
+	if (m_auth >= Admin) {
+	    writeStr(m_machine ? "%%=auth:success\r\n" : "You are already authenticated as admin!\r\n");
+	    return false;
+	}
+	const char* pass = cfg().getValue("password");
+	if (pass && (str == pass)) {
+	    Output("Authenticated admin connection %s",m_address.c_str());
+	    m_auth = Admin;
+	    m_timeout = 0;
+	    writeStr(m_machine ? "%%=auth:success\r\n" : "Authenticated successfully as admin!\r\n");
+	}
+	else if ((pass = cfg().getValue("userpass")) && (str == pass)) {
+	    if (m_auth < User) {
+		Output("Authenticated user connection %s",m_address.c_str());
+		m_auth = User;
+		m_timeout = 0;
+		writeStr(m_machine ? "%%=auth:success\r\n" : "Authenticated successfully as user!\r\n");
+	    }
+	    else
+		writeStr(m_machine ? "%%=auth:success\r\n" : "You are already authenticated as user!\r\n");
+	}
+	else
+	    writeStr(m_machine ? "%%=auth:fail=badpass\r\n" : "Bad authentication password!\r\n");
+	return false;
+    }
+    if (m_auth < User) {
+	writeStr(m_machine ? "%%=*:fail=noauth\r\n" : "Not authenticated!\r\n");
+	return false;
+    }
     if (str.startSkip("status"))
     {
 	Message m("engine.status");
 	if (str.startSkip("overview"))
 	    m.addParam("details",String::boolText(false));
-	if (str.null() || (str == "rmanager"))
-	    m.retValue() << "name=rmanager,type=misc;conn=" << connectionlist.count() << "\r\n";
+	if (str.null() || (str == "rmanager")) {
+	    s_mutex.lock();
+	    m.retValue() << "name=rmanager,type=misc"
+		<< ";listeners=" << s_listeners.count()
+		<< ",conn=" << s_connList.count() << "\r\n";
+	    s_mutex.unlock();
+	}
 	if (!str.null()) {
 	    m.addParam("module",str);
 	    str = ":" + str;
@@ -750,11 +1192,24 @@ bool Connection::processLine(const char *line)
 	writeStr(str);
 	return false;
     }
-    else if (str.startSkip("echo"))
+    else if (str.startSkip("uptime"))
     {
-	str >> m_echoing;
-	str = "Remote echo: ";
-	str += (m_echoing ? "on\r\n" : "off\r\n");
+	str.clear();
+	u_int32_t t = SysUsage::secRunTime();
+	if (m_machine) {
+	    str << "%%=uptime:" << (unsigned int)t;
+	    (str << ":").append(SysUsage::runTime(SysUsage::UserTime));
+	    (str << ":").append(SysUsage::runTime(SysUsage::KernelTime));
+	}
+	else {
+	    char buf[72];
+	    ::sprintf(buf,"%u %02u:%02u:%02u (%u)",
+		t / 86400, (t / 3600) % 24,(t / 60) % 60,t % 60,t);
+	    str << "Uptime: " << buf;
+	    (str << " user: ").append(SysUsage::runTime(SysUsage::UserTime));
+	    (str << " kernel: ").append(SysUsage::runTime(SysUsage::KernelTime));
+	}
+	str << "\r\n";
 	writeStr(str);
 	return false;
     }
@@ -782,85 +1237,7 @@ bool Connection::processLine(const char *line)
 	writeStr(str);
 	return false;
     }
-    else if (str.startSkip("uptime"))
-    {
-	str.clear();
-	u_int32_t t = SysUsage::secRunTime();
-	if (m_machine) {
-	    str << "%%=uptime:" << (unsigned int)t;
-	    (str << ":").append(SysUsage::runTime(SysUsage::UserTime));
-	    (str << ":").append(SysUsage::runTime(SysUsage::KernelTime));
-	}
-	else {
-	    char buf[64];
-	    ::sprintf(buf,"%u:%02u:%02u (%u)",t / 3600,(t / 60) % 60,t % 60,t);
-	    str << "Uptime: " << buf;
-	    (str << " user: ").append(SysUsage::runTime(SysUsage::UserTime));
-	    (str << " kernel: ").append(SysUsage::runTime(SysUsage::KernelTime));
-	}
-	str << "\r\n";
-	writeStr(str);
-	return false;
-    }
-    else if (str.startSkip("quit"))
-    {
-	writeStr(m_machine ? "%%=quit\r\n" : "Goodbye!\r\n");
-	return true;
-    }
-    else if (str.startSkip("help") || str.startSkip("?"))
-    {
-	Message m("engine.help");
-	if (str)
-	{
-	    const CommandInfo* info = s_cmdInfo;
-	    for (; info->name; info++) {
-		if (str == info->name) {
-		    str = "  ";
-		    str << info->name;
-		    if (info->args)
-			str << " " << info->args;
-		    str << "\r\n" << info->desc << "\r\n";
-		    writeStr(str);
-		    return false;
-		}
-	    }
-	    m.addParam("line",str);
-	    if (Engine::dispatch(m))
-		writeStr(m.retValue());
-	    else
-		writeStr("No help for '"+str+"'\r\n");
-	}
-	else
-	{
-	    m.retValue() = "Available commands:\r\n";
-	    const CommandInfo* info = s_cmdInfo;
-	    for (; info->name; info++) {
-		m.retValue() << "  " << info->name;
-		if (info->args)
-		    m.retValue() << " " << info->args;
-		m.retValue() << "\r\n";
-	    }
-	    Engine::dispatch(m);
-	    writeStr(m.retValue());
-	}
-	return false;
-    }
-    else if (str.startSkip("auth"))
-    {
-	if (m_auth) {
-	    writeStr(m_machine ? "%%=auth:success\r\n" : "You are already authenticated!\r\n");
-	    return false;
-	}
-	if (str == s_cfg.getValue("general","password")) {
-	    Output("Authenticated connection %s",m_address.c_str());
-	    m_auth = true;
-	    writeStr(m_machine ? "%%=auth:success\r\n" : "Authenticated successfully!\r\n");
-	}
-	else
-	    writeStr(m_machine ? "%%=auth:fail=badpass\r\n" : "Bad authentication password!\r\n");
-	return false;
-    }
-    if (!m_auth) {
+    if (m_auth < Admin) {
 	writeStr(m_machine ? "%%=*:fail=noauth\r\n" : "Not authenticated!\r\n");
 	return false;
     }
@@ -924,6 +1301,15 @@ bool Connection::processLine(const char *line)
 	    str >> dbg;
 	    dbg = debugLevel(dbg);
 	}
+	if (str.startSkip("threshold")) {
+	    int thr = m_threshold;
+	    str >> thr;
+	    if (thr < DebugConf)
+		thr = DebugConf;
+	    else if (thr > DebugAll)
+		thr = DebugAll;
+	    m_threshold = thr;
+	}
 	else if (str.isBoolean()) {
 	    str >> m_debug;
 	    if (m_debug)
@@ -953,11 +1339,13 @@ bool Connection::processLine(const char *line)
 	}
 	if (m_machine) {
 	    str = "%%=debug:level=";
-	    str << debugLevel() << ":local=" << m_debug << "\r\n";
+	    str << debugLevel() << ":local=" << m_debug;
+	    str << ":threshold=" << m_threshold << "\r\n";
 	}
 	else {
 	    str = "Debug level: ";
-	    str << debugLevel() << " local: " << (m_debug ? "on\r\n" : "off\r\n");
+	    str << debugLevel() << " local: " << (m_debug ? "on" : "off");
+	    str << " threshold: " << m_threshold << "\r\n";
 	}
 	writeStr(str);
     }
@@ -973,7 +1361,7 @@ bool Connection::processLine(const char *line)
 	Message m("chan.control");
 	m.addParam("targetid",id);
 	m.addParam("component",id);
-	Regexp r("^\\(.* \\)\\?\\([^= ]\\+\\)=\\([^=]*\\)$");
+	static const Regexp r("^\\(.* \\)\\?\\([^= ]\\+\\)=\\([^=]*\\)$");
 	while (ctrl) {
 	    if (!ctrl.matches(r)) {
 		m.setParam("operation",ctrl);
@@ -1025,8 +1413,9 @@ bool Connection::processLine(const char *line)
 #endif
     else if (str.startSkip("reload"))
     {
+	str.trimSpaces();
 	writeStr(m_machine ? "%%=reload\r\n" : "Reinitializing...\r\n");
-	Engine::init();
+	Engine::init(str);
     }
     else if (str.startSkip("restart"))
     {
@@ -1050,12 +1439,70 @@ bool Connection::processLine(const char *line)
 	writeStr(m_machine ? "%%=shutdown\r\n" : "Engine shutting down - bye!\r\n");
 	Engine::halt(code);
     }
+    else if (str.startSkip("alias"))
+    {
+	str.trimSpaces();
+	if (str.null()) {
+	    NamedIterator iter(m_aliases);
+	    while (const NamedString* s = iter.get())
+		str << s->name() << "=" << *s << "\r\n";
+	    writeStr(str);
+	    return false;
+	}
+	int sep = str.find(' ');
+	if (sep > 0) {
+	    String val = str.substr(sep+1);
+	    str = str.substr(0,sep);
+	    m_aliases.setParam(str,val);
+	    writeStr("Alias " + str + " set to: " + val + "\r\n");
+	}
+	else {
+	    m_aliases.clearParam(str);
+	    writeStr("Alias " + str + " removed\r\n");
+	}
+    }
     else
     {
+	str.trimSpaces();
+	int sep = str.find(' ');
+	const String* cmd = m_aliases.getParam(str.substr(0,sep));
+	if (cmd) {
+	    if (!saveLine) {
+		writeStr("Error: possible alias loop in '" + str + "'\r\n");
+		return false;
+	    }
+	    if (sep > 0)
+		str = str.substr(sep+1);
+	    else
+		str.clear();
+	    static const Regexp s_paramSep("^\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *\\([^ ]*\\)\\? *");
+	    str.matches(s_paramSep);
+	    str = str.replaceMatches(*cmd);
+	    for (;;) {
+		sep = str.find("$()");
+		if (sep < 0)
+		    return processLine(str,false);
+		if (processLine(str.substr(0,sep),false))
+		    return true;
+		str = str.substr(sep+3);
+	    }
+	}
 	Message m("engine.command");
 	m.addParam("line",str);
-	if (Engine::dispatch(m))
+	if (Engine::dispatch(m)) {
 	    writeStr(m.retValue());
+	    const ObjList* l = YOBJECT(ObjList,m.userData());
+	    if (l)
+		l = l->skipNull();
+	    for (; l; l = l->skipNext()) {
+		const GenObject* o = l->get();
+		const CapturedEvent* ev = YOBJECT(CapturedEvent,o);
+		if (ev)
+		    writeEvent(ev->c_str(),ev->level());
+		else
+		    writeEvent(o->toString(),-1);
+	    }
+	}
 	else
 	    writeStr((m_machine ? "%%=syntax:" : "Cannot understand: ") + str + "\r\n");
     }
@@ -1069,34 +1516,43 @@ void Connection::writeStr(const Message &msg, bool received)
 	return;
     String s = msg.encode(received,"");
     s << "\r\n";
+    if (m_echoing && m_buffer)
+	clearLine();
     writeStr(s.c_str());
+    if (m_echoing && m_buffer)
+	writeBuffer();
 }
 
 // write debugging messages to the remote console
 void Connection::writeDebug(const char *str, int level)
+{
+    if ((m_debug && (m_threshold >= level)) || (m_output && (level < 0)))
+	writeEvent(str,level);
+}
+
+// unconditionally write an event to the remote console
+void Connection::writeEvent(const char *str, int level)
 {
     if (null(str))
 	return;
-    if (m_debug || (m_output && (level < 0))) {
-	if (m_echoing && m_buffer)
-	    clearLine();
-	const char* col = m_colorize ? debugColor(level) : 0;
-	if (col)
-	    writeStr(col,::strlen(col));
-	int len = ::strlen(str);
-	for (; len > 0; len--) {
-	    if ((unsigned char)str[len-1] >= ' ')
-		break;
-	}
-	writeStr(str,len);
-	writeStr("\r\n",2);
-	if (col)
-	    col = debugColor(-2);
-	if (col)
-	    writeStr(col,::strlen(col));
-	if (m_echoing && m_buffer)
-	    writeStr(m_buffer);
+    if (m_echoing && m_buffer)
+	clearLine();
+    const char* col = m_colorize ? debugColor(level) : 0;
+    if (col)
+	writeStr(col,::strlen(col));
+    int len = ::strlen(str);
+    for (; len > 0; len--) {
+	if ((unsigned char)str[len-1] >= ' ')
+	    break;
     }
+    writeStr(str,len);
+    writeStr("\r\n",2);
+    if (col)
+	col = debugColor(-2);
+    if (col)
+	writeStr(col,::strlen(col));
+    if (m_echoing && m_buffer)
+	writeBuffer();
 }
 
 // write arbitrary string to the remote console
@@ -1113,22 +1569,38 @@ void Connection::writeStr(const char *str, int len)
     }
 }
 
+void Connection::checkTimer(u_int64_t time)
+{
+    if (!m_timeout)
+	return;
+    if (time < m_timeout)
+	return;
+    m_timeout = 0;
+    if (m_socket)
+	m_socket->terminate();
+}
+
 
 void RHook::dispatched(const Message& msg, bool handled)
 {
+    u_int64_t t = 0;
+    if (msg == "engine.timer")
+	t = msg.msgTime();
     s_mutex.lock();
-    ObjList *p = &connectionlist;
-    for (; p; p=p->next()) {
-	Connection *con = static_cast<Connection *>(p->get());
-	if (con)
-	    con->writeStr(msg,handled);
+    ObjList* p = s_connList.skipNull();
+    for (; p; p = p->skipNext()) {
+	Connection* c = static_cast<Connection*>(p->get());
+	if (t)
+	    c->checkTimer(t);
+	c->writeStr(msg,handled);
     }
     s_mutex.unlock();
 };
 
 
 RManager::RManager()
-    : m_first(true)
+    : Plugin("rmanager"),
+      m_first(true)
 {
     Output("Loaded module RManager");
     Debugger::setIntOut(dbg_remote_func);
@@ -1137,66 +1609,37 @@ RManager::RManager()
 RManager::~RManager()
 {
     Output("Unloading module RManager");
-    s_sock.terminate();
+    s_connList.clear();
+    s_listeners.clear();
     Debugger::setIntOut(0);
 }
 
 bool RManager::isBusy() const
 {
-    return (connectionlist.count() != 0);
+    Lock mylock(s_mutex);
+    return (s_connList.count() != 0);
 }
 
 void RManager::initialize()
 {
-    Output("Initializing module RManager");
-    s_cfg = Engine::configFile("rmanager");
-    s_cfg.load();
-    s_output = s_cfg.getBoolValue("general","output",false);
-
-    if (s_sock.valid())
-	return;
-
-    // check configuration
-    int port = s_cfg.getIntValue("general","port",5038);
-    const char *host = c_safe(s_cfg.getValue("general","addr","127.0.0.1"));
-    if (!(port && *host))
-	return;
-
-    s_sock.create(AF_INET, SOCK_STREAM);
-    if (!s_sock.valid()) {
-	Debug("RManager",DebugGoOn,"Unable to create the listening socket: %s",strerror(s_sock.error()));
-	return;
-    }
-
-    if (!s_sock.setBlocking(false)) {
-	Debug("RManager",DebugGoOn, "Failed to set listener to nonblocking mode: %s",strerror(s_sock.error()));
-	return;
-    }
-
-    const int reuseFlag = 1;
-    s_sock.setOption(SOL_SOCKET,SO_REUSEADDR,&reuseFlag,sizeof(reuseFlag));
-
-    SocketAddr sa(AF_INET);
-    sa.host(host);
-    sa.port(port);
-    s_sock.setReuse();
-    if (!s_sock.bind(sa)) {
-	Debug("RManager",DebugGoOn,"Failed to bind to %s:%u : %s",sa.host().c_str(),sa.port(),strerror(s_sock.error()));
-	s_sock.terminate();
-	return;
-    }
-    if (!s_sock.listen(2)) {
-	Debug("RManager",DebugGoOn,"Unable to listen on socket: %s", strerror(s_sock.error()));
-	s_sock.terminate();
-	return;
-    }
-    
-    // don't bother to install handlers until we are listening
     if (m_first) {
-	m_first = false;
-	Engine::self()->setHook(new RHook);
-	RManagerThread *mt = new RManagerThread;
-	mt->startup();
+	Output("Initializing module RManager");
+	Configuration cfg;
+	cfg = Engine::configFile("rmanager");
+	// in server mode assume a default empty "general" section exists
+	if (!(cfg.load() || Engine::clientMode()))
+	    (new RManagerListener(NamedList("general")))->init();
+	for (unsigned int i = 0; i < cfg.sections(); i++) {
+	    NamedList* s = cfg.getSection(i);
+	    if (s)
+		(new RManagerListener(*s))->init();
+	}
+	Lock mylock(s_mutex);
+	// don't bother to install handlers until we are listening
+	if (s_listeners.count()) {
+	    m_first = false;
+	    Engine::self()->setHook(new RHook);
+	}
     }
 }
 
