@@ -468,26 +468,6 @@ private:
     String m_groupName;                  // Group's name (saved to be printed in destructor)
 };
 
-// engine.start handler (start detectors on lines expectind data before ring)
-class EngineStartHandler : public MessageHandler
-{
-public:
-    inline EngineStartHandler()
-	: MessageHandler("engine.start",100)
-	{}
-    virtual bool received(Message& msg);
-};
-
-// chan.notify handler (notify lines on detector events)
-class ChanNotifyHandler : public MessageHandler
-{
-public:
-    inline ChanNotifyHandler()
-	: MessageHandler("chan.notify",100)
-	{}
-    virtual bool received(Message& msg);
-};
-
 
 /**
  * Module data and functions
@@ -503,6 +483,27 @@ static const char* s_groupStatusDetail = "format=Type|Lines";
 static const char* s_recStatusDetail = "format=Status|Address|Peer";
 
 
+// engine.start handler (start detectors on lines expectind data before ring)
+class EngineStartHandler : public MessageHandler
+{
+public:
+    inline EngineStartHandler()
+	: MessageHandler("engine.start",100,plugin.name())
+	{}
+    virtual bool received(Message& msg);
+};
+
+// chan.notify handler (notify lines on detector events)
+class ChanNotifyHandler : public MessageHandler
+{
+public:
+    inline ChanNotifyHandler()
+	: MessageHandler("chan.notify",100,plugin.name())
+	{}
+    virtual bool received(Message& msg);
+};
+
+
 // Decode a line address into group name and circuit code
 // Set first to decode group name until first '/'
 // Return:
@@ -1905,7 +1906,7 @@ void AnalogChannel::evAlarm(bool alarm, const char* alarms)
     // Wait if answered
     if (!m_alarmTimer.started()) {
 	Debug(this,DebugNote,
-	    "Line is out of order alarms=%s. Starting timer for " FMT64 " ms [%p]",
+	    "Line is out of order alarms=%s. Starting timer for " FMT64U " ms [%p]",
 	    alarms,m_alarmTimer.interval(),this);
 	m_alarmTimer.start();
     }
@@ -2695,7 +2696,7 @@ void AnalogDriver::initialize()
 
 bool AnalogDriver::msgExecute(Message& msg, String& dest)
 {
-    Channel* peer = static_cast<Channel*>(msg.userData());
+    CallEndpoint* peer = YOBJECT(CallEndpoint,msg.userData());
     ModuleLine* line = 0;
     String cause;
     const char* error = "failure";
