@@ -186,18 +186,6 @@ private:
     CpuMonitor m_system;
 };
 
-class QueryHandler : public MessageHandler
-{
-public:
-    inline QueryHandler(unsigned int priority = 100)
-	: MessageHandler("monitor.query",priority)
-	{ }
-    virtual ~QueryHandler()
-	{ }
-    virtual bool received(Message& msg);
-};
-
-
 class CpuModule : public Module
 {
 public:
@@ -218,6 +206,17 @@ static int s_defaultHysteresis = 2;
 static int s_bufLen = 4096;
 static int s_smooth = 33;
 
+class QueryHandler : public MessageHandler
+{
+public:
+    inline QueryHandler(unsigned int priority = 100)
+	: MessageHandler("monitor.query",priority,s_module.name())
+	{ }
+    virtual ~QueryHandler()
+	{ }
+    virtual bool received(Message& msg);
+};
+
 static TokenDict s_monitors[] = {
     {"userLoad",   CpuUpdater::YateUser},
     {"kernelLoad", CpuUpdater::YateKernel},
@@ -373,7 +372,7 @@ void Cpu::updateYateLoad()
     bool updateLoad = true;
     if (user < m_yateUser || ker < m_yateSystem || time < m_lastYateCheck) {
 	Debug(&s_module,DebugInfo,"Negative values for yate CPU update "
-	    "cu = "FMT64" lu="FMT64" ck="FMT64" lk="FMT64" ct="FMT64" lt="FMT64" ",
+	    "cu = "FMT64U" lu="FMT64U" ck="FMT64U" lk="FMT64U" ct="FMT64U" lt="FMT64U" ",
 	      user,m_yateUser,ker,m_yateSystem,time,m_lastYateCheck);
 	updateLoad = false;
     }
