@@ -180,9 +180,9 @@ protected:
     virtual bool received(Message& msg, int id);
 private:
     static int getPriority(const String& name);
-    static void addHandler(const char *name, int type);
-    static void addHandler(AAAHandler* handler);
-    static void addHandler(FallBackHandler* handler);
+    void addHandler(const char *name, int type);
+    void addHandler(AAAHandler* handler);
+    void addHandler(FallBackHandler* handler);
     bool m_init;
     AccountsModule *m_accountsmodule;
 };
@@ -427,6 +427,8 @@ bool AAAHandler::received(Message& msg)
     {
 	case Regist:
 	{
+	    if (!msg.getBoolValue(YSTRING("register_register"),true))
+		return false;
 	    if (s_critical)
 		return failure(&msg);
 	    Message m("database");
@@ -439,7 +441,7 @@ bool AAAHandler::received(Message& msg)
 	break;
 	case Auth:
 	{
-	    if (!msg.getBoolValue("auth_register",true))
+	    if (!msg.getBoolValue(YSTRING("auth_register"),true))
 		return false;
 	    Message m("database");
 	    prepareQuery(m,account,query,true);
@@ -459,6 +461,8 @@ bool AAAHandler::received(Message& msg)
 	break;
 	case PreRoute:
 	{
+	    if (!msg.getBoolValue(YSTRING("preroute_register"),true))
+		return false;
 	    if (s_critical)
 		return failure(&msg);
 	    Message m("database");
@@ -474,6 +478,8 @@ bool AAAHandler::received(Message& msg)
 	break;
 	case Route:
 	{
+	    if (!msg.getBoolValue(YSTRING("route_register"),true))
+		return false;
 	    if (s_critical)
 		return failure(&msg);
 	    Message m("database");
@@ -500,6 +506,8 @@ bool AAAHandler::received(Message& msg)
 	break;
 	case UnRegist:
 	{
+	    if (!msg.getBoolValue(YSTRING("register_register"),true))
+		return false;
 	    // no error check needed on unregister - we return false
 	    Message m("database");
 	    prepareQuery(m,account,query,true);
@@ -956,6 +964,10 @@ int RegistModule::getPriority(const String& name)
 
 void RegistModule::addHandler(AAAHandler* handler)
 {
+    String trackName(name());
+    if (trackName && handler->priority())
+	trackName << ":" << handler->priority();
+    handler->trackName(trackName);
     handler->loadAccount();
     s_handlers.append(handler);
     handler->loadQuery();
@@ -965,6 +977,10 @@ void RegistModule::addHandler(AAAHandler* handler)
 
 void RegistModule::addHandler(FallBackHandler* handler)
 {
+    String trackName(name());
+    if (trackName && handler->priority())
+	trackName << ":" << handler->priority();
+    handler->trackName(trackName);
     s_handlers.append(handler);
     Engine::install(handler);
 }
@@ -1162,8 +1178,8 @@ bool AccountsModule::received(Message &msg, int id)
 void AccountsModule::initialize()
 {
     if (s_cfg.getBoolValue("general","accounts")) {
-	Engine::install(new MessageRelay("user.notify",this,Notify,100));
-	Engine::install(new MessageRelay("engine.timer",this,Timer,100));
+	Engine::install(new MessageRelay("user.notify",this,Notify,100,module.name()));
+	Engine::install(new MessageRelay("engine.timer",this,Timer,100,module.name()));
     }
 }
 
