@@ -51,20 +51,32 @@ class TransportWorker
     friend class TransportThread;
 public:
     inline TransportWorker()
-	: m_thread(0)
+	: m_thread(0), m_threadMutex(true,"TransportThread")
 	{ }
     virtual ~TransportWorker() { stop(); }
     virtual bool readData() = 0;
     virtual bool connectSocket() = 0;
     virtual bool needConnect() = 0;
+    virtual void reset() = 0;
     bool running();
     bool start(Thread::Priority prio = Thread::Normal);
     inline void resetThread()
-	{ m_thread = 0; }
+	{
+	    Lock myLock(m_threadMutex);
+	    m_thread = 0;
+	}
+    virtual const char* getTransportName() = 0;
+    inline bool hasThread()
+	{
+	    Lock myLock(m_threadMutex);
+	    return m_thread != 0;
+	}
+    void exitThread();
 protected:
     void stop();
 private:
     TransportThread* m_thread;
+    Mutex m_threadMutex;
 };
 
 class SockRef : public RefObject
@@ -87,13 +99,23 @@ class TransportThread : public Thread
 {
     friend class TransportWorker;
 public:
-    inline TransportThread(TransportWorker* worker, Priority prio = Normal)
-	: Thread("SignallingTransporter",prio), m_worker(worker)
+    inline TransportThread(TransportWorker* worker, String* tName, Priority prio = Normal)
+	: Thread(*tName,prio), m_worker(worker), m_exit(false), m_threadName(tName), m_cleanWorker(true)
 	{ }
     virtual ~TransportThread();
     virtual void run();
+    void exitThread()
+	{
+	    m_cleanWorker = false;
+	    m_exit = true;
+	}
+    void resetWorker()
+	{ m_worker = 0; }
 private:
     TransportWorker* m_worker;
+    bool m_exit;
+    String* m_threadName;
+    bool m_cleanWorker;
 };
 
 class ListenerThread : public Thread
@@ -112,18 +134,20 @@ private:
     bool m_stream;
 };
 
-class TReader : public TransportWorker, public Mutex
+class TReader : public TransportWorker, public Mutex, public RefObject
 {
 public:
     inline TReader()
 	: Mutex(true,"TReader"),
 	  m_sending(true,"TReader::sending"), m_canSend(true), m_reconnect(false),
-          m_tryAgain(0), m_interval(CONN_RETRY_MIN)
+	  m_tryAgain(0), m_interval(CONN_RETRY_MIN), m_downTime(0)
 	{ }
     virtual ~TReader();
     virtual void listen(int maxConn) = 0;
     virtual bool sendMSG(const DataBlock& header, const DataBlock& msg, int streamId = 0) = 0;
     virtual void setSocket(Socket* s) = 0;
+    virtual bool getSocketParams(const String& params, NamedList& result) = 0;
+    virtual void reset() = 0;
     void reconnect()
 	{ m_reconnect = true; }
     Mutex m_sending;
@@ -132,6 +156,7 @@ protected:
     bool m_reconnect;
     u_int64_t m_tryAgain;
     u_int32_t m_interval;
+    u_int64_t m_downTime;
 };
 
 class Transport : public SIGTransport
@@ -151,7 +176,8 @@ public:
 	Down
     };
     virtual bool initialize(const NamedList* config);
-    Transport(const NamedList& params);
+    Transport(const NamedList& params, String* mutexName);
+    Transport(TransportType type, String* mutexName);
     ~Transport();
     inline unsigned char getVersion(unsigned char* buf) const
 	{ return buf[0]; }
@@ -172,6 +198,8 @@ public:
 	{ return m_state; }
     inline void resetListener()
 	{ m_listener = 0; }
+    inline void startReading()
+	{ if (m_reader) m_reader->start(); }
     bool addSocket(Socket* socket,SocketAddr& adress);
     virtual bool reliable() const
 	{ return m_type == Sctp || m_type == Tcp; }
@@ -182,20 +210,29 @@ public:
 	{ }
     virtual void reconnect(bool force);
     bool connectSocket();
+    void resetReader(TReader* reader);
     u_int32_t getMsgLen(unsigned char* buf);
     bool addAddress(const NamedList &param, Socket* socket);
     bool bindSocket();
-    static SignallingComponent* create(const String& type,const NamedList& params);
+    static SignallingComponent* create(const String& type, NamedList& params);
     virtual bool transmitMSG(const DataBlock& header, const DataBlock& msg, int streamId = 0);
+    virtual bool getSocketParams(const String& params, NamedList& result);
+    virtual void destroyed();
+
+    virtual bool hasThread();
+    virtual void stopThread();
 private:
     TReader* m_reader;
+    Mutex m_readerMutex;
     bool m_streamer;
     int m_type;
     int m_state;
     ListenerThread* m_listener;
-    const NamedList m_config;
+    NamedList m_config;
     bool m_endpoint;
     bool m_supportEvents;
+    bool m_listenNotify;
+    String* m_mutexName;
 };
 
 class StreamReader : public TReader
@@ -212,7 +249,13 @@ public:
     virtual void listen(int maxConn)
 	{ }
     virtual bool sendBuffer(int streamId = 0);
-    void connectionDown();
+    virtual bool getSocketParams(const String& params, NamedList& result);
+    virtual void reset()
+	{ if (m_transport) m_transport->resetReader(this); }
+    void connectionDown(bool stop = true);
+    void stopThread();
+    virtual const char* getTransportName()
+	{ return m_transport ? m_transport->debugName() : ""; }
 private:
     Transport* m_transport;
     Socket* m_socket;
@@ -235,14 +278,23 @@ public:
     virtual bool needConnect()
 	{ return m_transport && m_transport->status() == Transport::Down && !m_transport->listen(); }
     virtual bool sendMSG(const DataBlock& header, const DataBlock& msg, int streamId = 0);
+    virtual bool getSocketParams(const String& params, NamedList& result);
     virtual void setSocket(Socket* s);
     virtual void listen(int maxConn)
 	{ m_socket->listen(maxConn); }
+    virtual void reset()
+	{ m_transport->resetReader(this); }
     bool bindSocket();
+    void reconnectSocket();
+    void updateTransportStatus(int status);
+    virtual const char* getTransportName()
+	{ return m_transport ? m_transport->debugName() : ""; }
 private:
-    Socket* m_socket;
     Transport* m_transport;
+    Socket* m_socket;
     SocketAddr m_remote;
+    u_int32_t m_reconnectInterval;
+    u_int64_t m_reconnectTryAgain;
 };
 
 class TransportModule : public Module
@@ -257,6 +309,21 @@ private:
 
 static TransportModule plugin;
 YSIGFACTORY2(Transport);
+static long s_maxDownAllowed = 10000000;
+static ObjList s_names;
+Mutex s_namesMutex(false,"TransportNames");
+
+static void addName(String* name)
+{
+    Lock myLock(s_namesMutex);
+    s_names.append(name);
+}
+
+static void removeName(String* name)
+{
+    Lock myLock(s_namesMutex);
+    s_names.remove(name);
+}
 
 static const TokenDict s_transType[] = {
     { "none",  Transport::None },
@@ -336,6 +403,7 @@ bool ListenerThread::init(const NamedList& param)
 		Debug("ListenerThread",DebugConf,"Could not obtain SctpSocket");
 		return false;
 	    }
+	    delete m_socket;
 	    m_socket = soc;
 	    m_socket->create(AF_INET,m_stream ? SOCK_STREAM : SOCK_SEQPACKET,IPPROTO_SCTP);
 	    break;
@@ -400,10 +468,12 @@ void ListenerThread::run()
 		DDebug("ListenerThread",DebugNote, "Accept error: %s", strerror(m_socket->error()));
 	    continue;
 	} else {
-	    if (!m_transport->addSocket(newSoc,address))
+	    if (!m_transport->addSocket(newSoc,address)) {
 		DDebug("ListenerThread",DebugNote,"Connection rejected for %s",
 		    address.host().c_str());
-		Debug(DebugStub,"See if should be done peeloff for new incomming connections");
+		newSoc->terminate();
+		delete newSoc;
+	    }
 	}
     }
     if (m_transport) {
@@ -447,16 +517,15 @@ bool ListenerThread::addAddress(const NamedList &param)
 
 TransportThread::~TransportThread()
 {
+    DDebug("TransportThread",DebugAll,"Destroying TransportThread [%p]",this);
     if (m_worker)
 	m_worker->resetThread();
-    DDebug(DebugAll,"Destroying Transport Thread [%p]",this);
+    removeName(m_threadName);
 }
 
 void TransportThread::run()
 {
-    if (!m_worker)
-	return;
-    while (true) {
+    while (!m_exit) {
 	bool ret = false;
 	if (m_worker->needConnect())
 	    ret = m_worker->connectSocket();
@@ -467,24 +536,35 @@ void TransportThread::run()
 	else
 	    Thread::msleep(5,true);
     }
+    if (!m_worker)
+	return;
+    m_worker->resetThread();
+    if (m_cleanWorker)
+	m_worker->reset();
+    m_worker = 0;
 }
 
 TReader::~TReader()
 {
-    Debug(DebugAll,"Destroying TReader [%p]",this);
+   DDebug(DebugAll,"Destroying TReader [%p]",this);
 }
 
 /** TransportWorker class */
 
 bool TransportWorker::running()
 {
+    Lock myLock(m_threadMutex);
     return m_thread && m_thread->running();
 }
 
 bool TransportWorker::start(Thread::Priority prio)
 {
-    if (!m_thread)
-	m_thread = new TransportThread(this,prio);
+    Lock myLock(m_threadMutex);
+    if (!m_thread) {
+	String* name = new String(getTransportName());
+	addName(name);
+	m_thread = new TransportThread(this,name,prio);
+    }
     if (m_thread->running() || m_thread->startup())
 	return true;
     m_thread->cancel(true);
@@ -494,43 +574,79 @@ bool TransportWorker::start(Thread::Priority prio)
 
 void TransportWorker::stop()
 {
+    Lock myLock(m_threadMutex);
     if (!(m_thread && m_thread->running()))
 	return;
-    m_thread->cancel();
-    for (unsigned int i = 2 * Thread::idleMsec(); i--; ) {
+    m_thread->exitThread();
+
+    if (m_thread == Thread::current()) {
+	m_thread->resetWorker();
+	m_thread = 0;
+	DDebug(DebugWarn,"Stopping TransportWorker from itself!! %p ", this);
+	return;
+    }
+    myLock.drop();
+    while (true) {
 	Thread::msleep(1);
 	if (!m_thread)
 	    return;
     }
-    m_thread->cancel(true);
-    m_thread = 0;
+}
+
+void TransportWorker::exitThread()
+{
+    Lock myLock(m_threadMutex);
+    if (!m_thread)
+	return;
+    m_thread->exitThread();
 }
 
 /**
  * Transport class
  */
 
-SignallingComponent* Transport::create(const String& type, const NamedList& name)
+SignallingComponent* Transport::create(const String& type, NamedList& name)
 {
     if (type != "SIGTransport")
 	return 0;
     Configuration cfg(Engine::configFile("sigtransport"));
     cfg.load();
 
+    NamedString* listenNotify = name.getParam(YSTRING("listen-notify"));
     const char* sectName = name.getValue("basename");
     NamedList* config = cfg.getSection(sectName);
-    if (!config) {
-	DDebug(&plugin,DebugWarn,"No section '%s' in configuration",c_safe(sectName));
+    if (!name.getBoolValue(YSTRING("local-config"),false))
+	config = &name;
+    else if (!config) {
+	Debug("SIGTransport",DebugWarn,"No section %s in configuration!",sectName);
 	return 0;
-    }
-    return new Transport(*config);
+    } else
+	name.copyParams(*config);
+
+    if (listenNotify)
+	config->setParam(listenNotify->name(),*listenNotify);
+    String* mName = new String("TransportReader:");
+    mName->append(*config);
+    addName(mName);
+    return new Transport(*config,mName);
 }
 
-Transport::Transport(const NamedList &param)
-    : m_reader(0), m_state(Down), m_listener(0), m_config(param), m_endpoint(true), m_supportEvents(true)
+Transport::Transport(const NamedList &param, String* mutexName)
+    : m_reader(0), m_readerMutex(true,*mutexName), m_streamer(false), m_state(Down),
+    m_listener(0), m_config(param), m_endpoint(true), m_supportEvents(true),
+    m_listenNotify(true), m_mutexName(mutexName)
 {
-    setName("Transport");
-    Debug(this,DebugAll,"Transport created (%p)",this);
+    setName("Transport:" + param);
+    DDebug(this,DebugAll,"Transport created (%p)",this);
+    m_listenNotify = param.getBoolValue("listen-notify",true);
+}
+
+Transport::Transport(TransportType type, String* mutexName)
+    : m_reader(0), m_readerMutex(true,*mutexName), m_streamer(true), m_type(type), m_state(Down),
+    m_listener(0), m_config(""), m_endpoint(true), m_supportEvents(true),
+    m_listenNotify(false), m_mutexName(mutexName)
+{
+    DDebug(this,DebugInfo,"Creating new Transport [%p]",this);
 }
 
 Transport::~Transport()
@@ -540,7 +656,31 @@ Transport::~Transport()
     while (m_listener)
 	Thread::yield();
     Debug(this,DebugAll,"Destroying Transport [%p]",this);
-    delete m_reader;
+    TelEngine::destruct(m_reader);
+    removeName(m_mutexName);
+}
+
+void Transport::destroyed()
+{
+    m_readerMutex.lock();
+    TReader* tmp = m_reader;
+    m_reader = 0;
+    m_readerMutex.unlock();
+    TelEngine::destruct(tmp);
+}
+
+void Transport::resetReader(TReader* caller)
+{
+    if (!caller)
+	return;
+    if (m_reader) {
+	m_readerMutex.lock();
+	if (caller == m_reader)
+	    m_reader = 0;
+	m_readerMutex.unlock();
+    }
+    if (m_listener)
+	TelEngine::destruct(caller);
 }
 
 bool Transport::control(const NamedList &param)
@@ -563,38 +703,43 @@ bool Transport::control(const NamedList &param)
 
 void Transport::reconnect(bool force)
 {
+    Lock lock(m_readerMutex);
     if (!m_reader) {
 	Debug(this,DebugWarn,"Request to reconnect but the transport is not initialized!!");
 	return;
     }
-    if (m_state == Up && !force)
+    if (m_state == Up && !force) {
+	Debug(this,DebugInfo,
+	      "Skipped transport restart. Transport is UP and force restart was not requested.");
 	return;
+    }
     Debug(this,DebugInfo,"Transport reconnect requested");
     m_reader->reconnect();
 }
 
+bool Transport::getSocketParams(const String& params, NamedList& result)
+{
+    if (m_reader)
+	return m_reader->getSocketParams(params,result);
+    return false;
+}
+
 bool Transport::initialize(const NamedList* params)
 {
-    Configuration cfg(Engine::configFile("sigtransport"));
-    cfg.load();
-    const char* sectName = params->getValue("basename");
-    NamedList* config = cfg.getSection(sectName);
-    if (!config) {
-	DDebug(&plugin,DebugWarn,"No section '%s' in configuration",c_safe(sectName));
-	return false;
-    }
-    m_type = lookup(config->getValue("type","sctp"),s_transType);
-    m_streamer = config->getBoolValue("stream",streamDefault());
-    m_endpoint = config->getBoolValue("endpoint",false);
+
+    m_type = lookup(m_config.getValue("type","sctp"),s_transType);
+    m_streamer = m_config.getBoolValue("stream",streamDefault());
+    m_endpoint = m_config.getBoolValue("endpoint",false);
     if (!m_endpoint && m_streamer) {
 	m_listener = new ListenerThread(this);
-	if (!m_listener->init(*config)) {
+	if (!m_listener->init(m_config)) {
 	    DDebug(this,DebugNote,"Unable to start listener");
 	    return false;
 	}
 	m_listener->startup();
 	return true;
     }
+    Lock lock(m_readerMutex);
     if (m_streamer)
 	m_reader = new StreamReader(this,0);
     else {
@@ -608,6 +753,7 @@ bool Transport::initialize(const NamedList* params)
 	bindSocket();
     }
     m_reader->start();
+    lock.drop();
     return true;
 }
 
@@ -677,7 +823,7 @@ bool Transport::bindSocket()
     addr.host(address);
     addr.port(port);
     if (!socket->bind(addr)) {
-	Debug(DebugMild,"Unable to bind to %s:%u: %d: %s",
+	Debug(this,DebugMild,"Unable to bind to %s:%u: %d: %s",
 	    addr.host().c_str(),addr.port(),errno,strerror(errno));
 	socket->terminate();
 	delete socket;
@@ -689,10 +835,13 @@ bool Transport::bindSocket()
 	delete socket;
 	return false;
     }
+    Lock lock(m_readerMutex);
+    if (!m_reader)
+	return false;
     m_reader->setSocket(socket);
     int linger = m_config.getIntValue("linger",0);
     socket->setLinger(linger);
-    setStatus(Up);
+
     if (m_type == Sctp) {
 	// Send a dummy MGMT NTFY message to create the connection
 	static const unsigned char dummy[8] =
@@ -702,7 +851,8 @@ bool Transport::bindSocket()
 	if (m_reader->sendMSG(hdr,DataBlock::empty(),1))
 	    m_reader->listen(1);
 	hdr.clear(false);
-    }
+    } else
+	setStatus(Up);
     return true;
 }
 
@@ -846,6 +996,13 @@ bool Transport::connectSocket()
 	     Debug(this,DebugNote,"Socket conected to %d addresses",o.count());
     }
     sock->setBlocking(false);
+    Lock mylock(m_readerMutex);
+    if (!m_reader) {
+	Debug(this,DebugFail,"Connect socket null reader");
+	sock->terminate();
+	delete sock;
+	return false;
+    }
     m_reader->setSocket(sock);
     setStatus(Up);
     return true;
@@ -858,7 +1015,11 @@ void Transport::setStatus(int status)
     DDebug(this,DebugInfo,"State change: %s -> %s [%p]",
 	lookup(m_state,s_transStatus,"?"),lookup(status,s_transStatus,"?"),this);
     m_state = status;
+    Lock mylock(m_readerMutex);
+    if (!m_reader)
+	return;
     m_reader->m_canSend = true;
+    mylock.drop();
     notifyLayer((status == Up) ? SignallingInterface::LinkUp : SignallingInterface::LinkDown);
 }
 
@@ -871,30 +1032,52 @@ u_int32_t Transport::getMsgLen(unsigned char* buf)
 
 bool Transport::transmitMSG(const DataBlock& header, const DataBlock& msg, int streamId)
 {
-    if (!m_reader) {
-	DDebug(this,DebugMild,"Cannot send message, no reader set [%p]",this);
+    Lock lock(m_readerMutex);
+    RefPointer<TReader> reader = m_reader;
+    if (!reader)
 	return false;
-    }
-    bool ret = m_reader->sendMSG(header,msg,streamId);
-    return ret;
+    lock.drop();
+    return reader->sendMSG(header,msg,streamId);;
 }
 
-bool Transport::addSocket(Socket* socket,SocketAddr& adress)
+bool Transport::addSocket(Socket* socket,SocketAddr& socketAddress)
 {
+    if (m_listenNotify) {
+	String* name = new String("Transport:");
+	name->append(socketAddress.host() + ":");
+	name->append((int)socketAddress.port());
+	addName(name);
+	Transport* newTrans = new Transport((TransportType)m_type,name);
+	if (!transportNotify(newTrans,socketAddress)) {
+	    DDebug(this,DebugInfo,"New transport wasn't accepted!");
+	    return false;
+	}
+	if (!newTrans->addSocket(socket,socketAddress)) {
+	    newTrans->setStatus(Down);
+	    return false;
+	}
+	return true;
+    }
+    Lock lock(m_readerMutex);
     if (transType() == Up)
 	return false;
-    SocketAddr addr(AF_INET);
-    String address, adr = m_config.getValue("remote");
-    int port = defPort();
-    resolveAddress(adr,address,port);
-    addr.host(address);
-    addr.port(port);
+    if (m_reader) {
+	StreamReader* sr = static_cast<StreamReader*>(m_reader);
+	m_reader = 0;
+	TelEngine::destruct(sr);
+    }
+    if (!m_config.c_str()) {
+	m_config.assign(String(socketAddress.host().c_str()) << ":" << socketAddress.port());
+	setName(m_config);
+    }
+    socket->setBlocking(false);
     switch (m_type) {
 	case Sctp :
 	{
 	    Socket* sock = 0;
 	    Message m("socket.sctp");
-	    m.addParam("handle",String(socket->handle()));
+	    m.addParam("handle",String(socket->detach()));
+	    delete socket;
 	    SockRef* s = new SockRef(&sock);
 	    m.userData(s);
 	    TelEngine::destruct(s);
@@ -915,10 +1098,11 @@ bool Transport::addSocket(Socket* socket,SocketAddr& adress)
 	    ppid = m_config.getIntValue("payload",ppid);
 	    if (ppid > 0)
 		soc->setPayload(ppid);
+	    soc->setBlocking(false);
 	    if (m_streamer)
 		m_reader = new StreamReader(this,soc);
 	    else
-		m_reader = new MessageReader(this,soc,addr);
+		Debug(this,DebugStub,"Add socket requested to create sctp message reader!");
 	    break;
 	}
 	case Unix :
@@ -926,17 +1110,33 @@ bool Transport::addSocket(Socket* socket,SocketAddr& adress)
 	    m_reader = new StreamReader(this,socket);
 	    break;
 	case Udp :
-	    m_reader = new MessageReader(this,socket,addr);
+	    Debug(this,DebugStub,"Add socket requested to create message reader for UDP socket type!");
 	    break;
 	default:
 	    Debug(this,DebugWarn,"Unknown socket type %d",m_type);
 	    return false;
     }
-    m_reader->start();
     setStatus(Up);
+    m_reader->start();
     return true;
 }
 
+bool Transport::hasThread()
+{
+    Lock lock(m_readerMutex);
+    if (!m_reader)
+	return false;
+    return m_reader->hasThread();
+}
+
+void Transport::stopThread()
+{
+    Lock lock(m_readerMutex);
+    if (m_reader)
+	m_reader->exitThread();
+}
+
+
 /**
  * class StreamReader 
  */
@@ -972,12 +1172,12 @@ void StreamReader::setSocket(Socket* s)
 
 bool StreamReader::sendMSG(const DataBlock& header, const DataBlock& msg, int streamId)
 {
+    Lock mylock(m_sending);
     if (!m_canSend) {
 	DDebug(m_transport,DebugNote,"Cannot send message at this time");
 	return false;
     }
     bool ret = false;
-    Lock mylock(m_sending);
     if (((m_sendBuffer.length() + msg.length()) + header.length()) < MAX_BUF_SIZE) {
 	m_sendBuffer += header;
 	m_sendBuffer += msg;
@@ -985,11 +1185,17 @@ bool StreamReader::sendMSG(const DataBlock& header, const DataBlock& msg, int st
     }
     else
 	Debug(m_transport,DebugWarn,"Buffer Overrun");
+    mylock.drop();
     return sendBuffer(streamId) && ret;
 }
 
 bool StreamReader::sendBuffer(int streamId)
 {
+    Lock mylock(m_sending);
+    if (!m_canSend) {
+	DDebug(m_transport,DebugNote,"Cannot send message at this time");
+	return false;
+    }
     if (!m_socket)
 	return needConnect();
     if (m_sendBuffer.null())
@@ -1000,8 +1206,10 @@ bool StreamReader::sendBuffer(int streamId)
 	return false;
     }
     if (error) {
-	if (m_socket->updateError() && !m_socket->canRetry())
-	    connectionDown();
+	if (m_socket->updateError() && !m_socket->canRetry()) {
+	    m_reconnect = true;
+	    m_canSend = false;
+	}
 	return false;
     }
     if (!sendOk)
@@ -1015,7 +1223,8 @@ bool StreamReader::sendBuffer(int streamId)
 	}
 	if (m_transport->status() == Transport::Up)
 	    if (!s->valid()) {
-		connectionDown();
+		m_reconnect = true;
+		m_canSend = false;
 		return false;
 	    }
 	int flags = 0;
@@ -1026,7 +1235,8 @@ bool StreamReader::sendBuffer(int streamId)
     if (len <= 0) {
 	if (!m_socket->canRetry()) {
 	    Debug(m_transport,DebugMild,"Send error detected. %s",strerror(errno));
-	    connectionDown();
+	    m_reconnect = true;
+	    m_canSend = false;
 	}
 	return false;
     }
@@ -1037,13 +1247,18 @@ bool StreamReader::sendBuffer(int streamId)
 bool StreamReader::connectSocket()
 {
     Time t;
-    if (t < m_tryAgain) {
+    if (t < m_tryAgain && !m_reconnect) {
 	Thread::yield(true);
 	return false;
     }
+    if (m_reconnect) {
+	m_reconnect = false;
+	m_interval = CONN_RETRY_MIN;
+    }
     m_tryAgain = t + m_interval;
     if (m_transport->connectSocket()) {
 	m_interval = CONN_RETRY_MIN;
+	m_tryAgain = 0;
 	return true;
     }
     m_tryAgain = Time::now() + m_interval;
@@ -1056,15 +1271,19 @@ bool StreamReader::connectSocket()
 
 bool StreamReader::readData()
 {
-    if (!(m_socket && m_sending.lock(SignallingEngine::maxLockWait())))
+    Lock myLock(m_sending,SignallingEngine::maxLockWait());
+    if (!(myLock.locked() && m_socket))
 	return false;
     if (m_reconnect) {
-	connectionDown();
-	m_sending.unlock();
+	connectionDown(false);
+	myLock.drop();
+	stopThread();
 	return false;
     }
     sendBuffer();
-    m_sending.unlock();
+    if (!m_socket)
+	return false;
+    myLock.drop();
     int stream = 0, len = 0;
     SocketAddr addr;
     unsigned char buf[MAX_BUF_SIZE];
@@ -1108,7 +1327,7 @@ bool StreamReader::readData()
 	m_totalPacketLen = m_transport->getMsgLen(auxBuf);
 	if (m_totalPacketLen >= 8 && m_totalPacketLen < MAX_BUF_SIZE) {
 	    m_totalPacketLen -= 8;
-	    XDebug(m_transport,DebugAll,"Expecting %d bytes of packet data",m_totalPacketLen);
+	    XDebug(m_transport,DebugAll,"Expecting %d bytes of packet data %d",m_totalPacketLen,stream);
 	    if (!m_totalPacketLen) {
 		m_transport->setStatus(Transport::Up);
 		m_transport->processMSG(m_transport->getVersion((unsigned char*)m_headerBuffer.data()),
@@ -1172,32 +1391,49 @@ bool StreamReader::readData()
     return false;
 }
 
-void StreamReader::connectionDown() {
+void StreamReader::connectionDown(bool stopTh) {
     Debug(m_transport,DebugMild,"Connection down [%p]",m_socket);
-    m_transport->setStatus(Transport::Down);
     while (!m_sending.lock(Thread::idleUsec()))
 	Thread::yield();
     m_canSend = false;
     m_sendBuffer.clear();
-    if (!m_socket) {
-	m_sending.unlock();
-	return;
+    if (m_socket) {
+	m_socket->terminate();
+	delete m_socket;
+	m_socket = 0;
     }
-    m_socket->terminate();
-    if (m_transport->listen())
-	stop();
-    delete m_socket;
-    m_socket = 0;
     m_sending.unlock();
+
+    if (stopTh) {
+	m_transport->setStatus(Transport::Down);
+	stopThread();
+    }
 }
 
+bool StreamReader::getSocketParams(const String& params, NamedList& result)
+{
+    Lock reconLock(m_sending,SignallingEngine::maxLockWait());
+    if (!(reconLock.locked() && m_socket))
+	return false;
+    m_socket->getParams(params,result);
+    return true;
+}
+
+void StreamReader::stopThread()
+{
+    m_transport->setStatus(Transport::Down);
+    if (!m_transport->listen())
+	return;
+    stop();
+}
 
 /**
  * class MessageReader
  */
 
 MessageReader::MessageReader(Transport* transport, Socket* sock, SocketAddr& addr)
-    : m_socket(sock), m_transport(transport), m_remote(addr)
+    : m_transport(transport), m_socket(sock), m_remote(addr),
+      m_reconnectInterval(s_maxDownAllowed),m_reconnectTryAgain(0)
 {
     DDebug(DebugAll,"Creating MessageReader [%p]",this);
 }
@@ -1231,7 +1467,6 @@ bool MessageReader::bindSocket()
 	Thread::yield(true);
 	return false;
     }
-    m_tryAgain = t + m_interval;
     Lock mylock(m_sending);
     if (m_transport->bindSocket()) {
 	m_interval = CONN_RETRY_MIN;
@@ -1245,6 +1480,22 @@ bool MessageReader::bindSocket()
     return false;
 }
 
+
+void MessageReader::reconnectSocket()
+{
+    u_int64_t t = Time::now();
+    if (t < m_reconnectTryAgain) {
+	Thread::yield(true);
+	return;
+    }
+    m_reconnect = true;
+    m_reconnectTryAgain = t + m_reconnectInterval;
+    // exponential backoff
+    m_reconnectInterval *= 2;
+    if (m_reconnectInterval > CONN_RETRY_MAX)
+	m_reconnectInterval = CONN_RETRY_MAX;
+}
+
 bool MessageReader::sendMSG(const DataBlock& header, const DataBlock& msg, int streamId)
 {
     if (!m_canSend) {
@@ -1257,7 +1508,8 @@ bool MessageReader::sendMSG(const DataBlock& header, const DataBlock& msg, int s
     while (m_socket && m_socket->select(0,&sendOk,&error,Thread::idleUsec())) {
 	if (error) {
 	    DDebug(m_transport,DebugAll,"Send error detected. %s",strerror(errno));
-	    m_transport->setStatus(Transport::Down);
+	    mylock.drop();
+	    updateTransportStatus(Transport::Down);
 	    break;
 	}
 	if (!sendOk)
@@ -1290,7 +1542,7 @@ bool MessageReader::sendMSG(const DataBlock& header, const DataBlock& msg, int s
 	    ret = true;
 	    break;
 	}
-	DDebug(m_transport,DebugMild,"Error sending message %d %d %s",len,totalLen,strerror(errno));
+	DDebug(m_transport,DebugMild,"Error sending message %d %d %s %s %d",len,totalLen,strerror(errno),m_remote.host().c_str(),m_remote.port());
 	break;
     }
     return ret;
@@ -1303,22 +1555,33 @@ bool MessageReader::readData()
     if (m_socket && m_reconnect) {
 	if (!reconLock.locked())
 	    return false;
-	if (m_transport->status() != Transport::Up)
-	    return false; // We are already in reconnecting state
-	m_transport->setStatus(Transport::Initiating);
+	if (m_downTime != 0) {
+	    int sec = (int)((Time::now() - m_downTime) / 1000000); // usec to sec
+	    Debug(m_transport,DebugNote,"Reconnecting sctp socket! is down for %d seconds.",sec);
+	}
 	m_socket->terminate();
 	delete m_socket;
 	m_socket = 0;
+	reconLock.drop();
+	updateTransportStatus(Transport::Initiating);
 	return false;
     }
     reconLock.drop();
     if (!m_socket && !bindSocket())
 	return false;
     bool readOk = false,error = false;
-    if (!(running() && m_socket && m_socket->select(&readOk,0,&error,Thread::idleUsec())))
+    if (!(running() && m_socket))
 	return false;
-    if (!readOk || error)
+    if (!m_socket->select(&readOk,0,&error,Thread::idleUsec())) {
+	if (m_transport->status() == Transport::Initiating)
+	    reconnectSocket();
 	return false;
+    }
+    if ((!readOk || error)) {
+	if (!readOk && m_transport->status() == Transport::Initiating)
+	    reconnectSocket();
+	return false;
+    }
 
     unsigned char buffer[MAX_BUF_SIZE];
     int stream = 0;
@@ -1344,14 +1607,14 @@ bool MessageReader::readData()
 	if (flags) {
 	     if (flags == 2) {
 		DDebug(m_transport,DebugAll,"Sctp connection is Up");
-		m_transport->setStatus(Transport::Up);
+		updateTransportStatus(Transport::Up);
 		return true;
 	    }
 	    DDebug(m_transport,DebugNote,"Message error [%p] %d",m_socket,flags);
 	    if (m_transport->status() != Transport::Up)
 		return false;
+	    updateTransportStatus(Transport::Initiating);
 	    Lock lock(m_sending);
-	    m_transport->setStatus(Transport::Initiating);
 	    Debug(m_transport,DebugInfo,"Terminating socket [%p] Reason: connection down!",m_socket);
 	    m_socket->terminate();
 	    delete m_socket;
@@ -1361,17 +1624,17 @@ bool MessageReader::readData()
     }
     else
 	r = m_socket->recvFrom((void*)buffer,MAX_BUF_SIZE,addr);
-
     if (r <= 0)
 	return false;
     m_interval = CONN_RETRY_MIN;
+    m_reconnectInterval = s_maxDownAllowed;
 
     u_int32_t len = m_transport->getMsgLen(buffer);
     if ((unsigned int)r != len) {
 	Debug(m_transport,DebugNote,"Protocol read error read: %d, expected %d",r,len);
 	return false;
     }
-    m_transport->setStatus(Transport::Up);
+    updateTransportStatus(Transport::Up);
     DataBlock packet(buffer,r);
     packet.cut(-8);
     m_transport->processMSG(m_transport->getVersion((unsigned char*)buffer),
@@ -1379,6 +1642,26 @@ bool MessageReader::readData()
     return true;
 }
 
+bool MessageReader::getSocketParams(const String& params, NamedList& result)
+{
+    Lock reconLock(m_sending,SignallingEngine::maxLockWait());
+    if (!(reconLock.locked() && m_socket))
+	return false;
+    m_socket->getParams(params,result);
+    return true;
+}
+
+void MessageReader::updateTransportStatus(int status)
+{
+    if (status == Transport::Up)
+	m_downTime = 0;
+    else if (m_downTime == 0) {
+	m_downTime = Time::now();
+	m_reconnectTryAgain = m_downTime + m_reconnectInterval;
+    }
+    m_transport->setStatus(status);
+}
+
 /**
  * class TransportModule
  */
@@ -1397,8 +1680,12 @@ TransportModule::~TransportModule()
 
 void TransportModule::initialize()
 {
+    Output("Initializing module SigTransport");
+    Configuration cfg(Engine::configFile("sigtransport"));
+    cfg.load();
+    s_maxDownAllowed = cfg.getIntValue(YSTRING("general"),YSTRING("max_down"),10);
+    s_maxDownAllowed *=1000000;
     if (!m_init) {
-	Output("Initializing module SigTransport");
 	m_init = true;
 	setup();
     }
