@@ -131,20 +131,6 @@ private:
     Tone2PoleFilter m_dtmfH[4];
 };
 
-class AttachHandler : public MessageHandler
-{
-public:
-    AttachHandler() : MessageHandler("chan.attach") { }
-    virtual bool received(Message& msg);
-};
-
-class RecordHandler : public MessageHandler
-{
-public:
-    RecordHandler() : MessageHandler("chan.record") { }
-    virtual bool received(Message& msg);
-};
-
 class ToneDetectorModule : public Module
 {
 public:
@@ -161,6 +147,24 @@ static int s_count = 0;
 
 static ToneDetectorModule plugin;
 
+class AttachHandler : public MessageHandler
+{
+public:
+    AttachHandler()
+	: MessageHandler("chan.attach",100,plugin.name())
+	{ }
+    virtual bool received(Message& msg);
+};
+
+class RecordHandler : public MessageHandler
+{
+public:
+    RecordHandler()
+	: MessageHandler("chan.record",100,plugin.name())
+	{ }
+    virtual bool received(Message& msg);
+};
+
 // generated CNG detector (1100Hz) - either of the 2 below:
 // mkfilter -Bp -Re 50 -a 0.137500
 //  -> 2-pole resonator bandpass, 1100Hz, Q-factor=50
