@@ -357,18 +357,6 @@ private:
     bool m_threadsCreated;      // True if reading and get events threads were created
 };
 
-/*
- * YIAXRegDataHandler
- */
-class YIAXRegDataHandler : public MessageHandler
-{
-public:
-    YIAXRegDataHandler()
-	: MessageHandler("user.login",150)
-	{ }
-    virtual bool received(Message &msg);
-};
-
 /*
  * YIAXDriver
  */
@@ -411,6 +399,23 @@ protected:
     int m_port;
 };
 
+static YIAXLineContainer s_lines;	// Lines
+static Thread::Priority s_priority = Thread::Normal;  // Threads priority
+static bool s_callTokenOut = true;      // Send an empty call token on outgoing calls
+static YIAXDriver iplugin;		// Init the driver
+
+/*
+ * YIAXRegDataHandler
+ */
+class YIAXRegDataHandler : public MessageHandler
+{
+public:
+    YIAXRegDataHandler()
+	: MessageHandler("user.login",150,iplugin.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
 class YIAXConnection;
 
 /*
@@ -452,7 +457,7 @@ class YIAXSource : public DataSource, public YIAXData
 public:
     YIAXSource(YIAXConnection* conn, u_int32_t format, const char* formatText, int type);
     ~YIAXSource();
-    void Forward(const DataBlock &data, unsigned long tStamp = 0, unsigned long flags = 0);
+    unsigned long Forward(const DataBlock &data, unsigned long tStamp, unsigned long flags);
 };
 
 /*
@@ -495,6 +500,8 @@ public:
     YIAXConsumer* getConsumer(int type);
     // Retrieve the data source from IAXFormat media type
     YIAXSource* getSource(int type);
+    // Retrieve the data source from IAXFormat
+    DataSource* getSourceMedia(int type);
 
 protected:
     // location: 0: from peer, else: from protocol
@@ -519,6 +526,7 @@ private:
     Mutex m_mutexTrans;                 // Safe m_transaction operations
     Mutex m_mutexRefIncreased;          // Safe ref/deref connection
     bool m_refIncreased;                // If true, the reference counter was increased
+    RefPointer<DataSource> m_sources[IAXFormat::TypeCount]; // Keep all types of sources
 };
 
 /*
@@ -560,11 +568,6 @@ private:
 /*
  * Local data
  */
-static YIAXLineContainer s_lines;	// Lines
-static Thread::Priority s_priority = Thread::Normal;  // Threads priority
-static bool s_callTokenOut = true;      // Send an empty call token on outgoing calls
-static YIAXDriver iplugin;		// Init the driver
-
 static unsigned int s_expires_min = 60;
 static unsigned int s_expires_def = 60;
 static unsigned int s_expires_max = 3600;
@@ -572,9 +575,9 @@ static unsigned int s_expires_max = 3600;
 static String s_statusCmd = "status";
 
 // Retrieve data endpoint name from IAXFormat type
-static inline const char* dataEpName(int type)
+static inline const String& dataEpName(int type)
 {
-    return IAXFormat::typeName(type);
+    return IAXFormat::typeNameStr(type);
 }
 
 // Retrieve format name from IAXFormat type
@@ -698,6 +701,8 @@ bool YIAXLineContainer::updateLine(Message& msg)
 	changed = changeLine(line,line->m_remoteAddr,addr) || changed;
 	changed = changeLine(line,line->m_username,msg.getValue("username")) || changed;
 	changed = changeLine(line,line->m_password,msg.getValue("password")) || changed;
+	changed = changeLine(line,line->m_callingNo,msg.getValue("caller")) || changed;
+	changed = changeLine(line,line->m_callingName,msg.getValue("callername")) || changed;
 	int interval = msg.getIntValue("interval");
 	if (interval < 60)
 	    interval = 60;
@@ -759,8 +764,10 @@ void YIAXLineContainer::regTerminate(IAXEvent* event)
 		}
 		// re-register at 75% of the expire time
 		line->m_nextReg = Time::secNow() + (line->expire() * 3 / 4);
-		line->m_callingNo = trans->callingNo();
-		line->m_callingName = trans->callingName();
+		if (trans->callingNo())
+		    line->m_callingNo = trans->callingNo();
+		if (trans->callingName())
+		    line->m_callingName = trans->callingName();
 	    }
 	    ok = line->m_register;
 	    break;
@@ -888,8 +895,10 @@ bool YIAXLineContainer::fillList(String& name, NamedList& dest, SocketAddr& addr
     line->lock();
     dest.setParam("username",line->username());
     dest.setParam("password",line->password());
-    dest.setParam("caller",line->callingNo());
-    dest.setParam("callername",line->callingName());
+    if (line->callingNo())
+	dest.setParam("caller",line->callingNo());
+    if (line->callingName())
+	dest.setParam("callername",line->callingName());
     String a = line->remoteAddr();
     int p = line->remotePort();
     registered = line->registered();
@@ -1003,12 +1012,15 @@ void YIAXEngine::processMedia(IAXTransaction* transaction, DataBlock& data,
     if (transaction) {
 	YIAXConnection* conn = static_cast<YIAXConnection*>(transaction->getUserData());
 	if (conn) {
-	    YIAXSource* src = conn->getSource(type);
+	    DataSource* src = conn->getSourceMedia(type);
 	    if (src) {
+		unsigned long ts = tStamp;
+		if (IAXFormat::Audio == type)
+		    ts *= 8; // assume 8kHz sampling
 		unsigned long flags = 0;
 		if (mark)
 		    flags = DataNode::DataMark;
-		src->Forward(data,tStamp,flags);
+		src->Forward(data,ts,flags);
 	    }
 	    else
 		DDebug(this,DebugAll,"processMedia. No media source");
@@ -1035,8 +1047,6 @@ IAXTransaction* YIAXEngine::reg(YIAXLine* line, bool regreq)
     IAXIEList ieList;
     ieList.appendString(IAXInfoElement::USERNAME,line->username());
     ieList.appendString(IAXInfoElement::PASSWORD,line->password());
-    ieList.appendString(IAXInfoElement::CALLING_NUMBER,line->callingNo());
-    ieList.appendString(IAXInfoElement::CALLING_NAME,line->callingName());
     ieList.appendNumeric(IAXInfoElement::REFRESH,line->expire(),2);
     // Make it !
     IAXTransaction* tr = startLocalTransaction(regreq ? IAXTransaction::RegReq : IAXTransaction::RegRel,addr,ieList);
@@ -1362,6 +1372,7 @@ void YIAXDriver::initialize()
     if (m_iaxEngine) {
 	m_iaxEngine->initialize(*gen);
 	m_iaxEngine->initFormats(cfg.getSection("formats"));
+	maxChans(gen->getIntValue("maxchans",maxChans()));
 	return;
     }
     setup();
@@ -1372,7 +1383,7 @@ void YIAXDriver::initialize()
     Engine::install(new YIAXRegDataHandler);
     // Init IAX engine
     u_int16_t transListCount = 64;
-    u_int16_t retransCount = 5;
+    u_int16_t retransCount = 4;
     u_int16_t retransInterval = 500;
     u_int16_t authTimeout = 30;
     u_int16_t transTimeout = 10;
@@ -1380,6 +1391,8 @@ void YIAXDriver::initialize()
     u_int32_t trunkSendInterval = 10;
     m_port = gen->getIntValue("port",4569);
     String iface = gen->getValue("addr","0.0.0.0");
+    // set max chans
+    maxChans(gen->getIntValue("maxchans",maxChans()));
     bool authReq = cfg.getBoolValue("registrar","auth_required",true);
     m_iaxEngine = new YIAXEngine(iface,m_port,transListCount,retransCount,retransInterval,authTimeout,
 	transTimeout,maxFullFrameDataLen,trunkSendInterval,authReq,gen);
@@ -1464,7 +1477,7 @@ bool YIAXDriver::msgExecute(Message& msg, String& dest)
     YIAXConnection* conn = new YIAXConnection(m_iaxEngine,tr,&msg,&params);
     conn->initChan();
     tr->setUserData(conn);
-    Channel* ch = static_cast<Channel*>(msg.userData());
+    CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (ch && conn->connect(ch,msg.getValue("reason"))) {
 	conn->callConnect(msg);
 	msg.setParam("peerid",conn->id());
@@ -1670,12 +1683,12 @@ YIAXSource::~YIAXSource()
 	IAXFormat::typeName(m_type),m_total,this);
 }
 
-void YIAXSource::Forward(const DataBlock& data, unsigned long tStamp, unsigned long flags)
+unsigned long YIAXSource::Forward(const DataBlock& data, unsigned long tStamp, unsigned long flags)
 {
     if (m_connection && m_connection->mutedIn())
-	return;
+	return invalidStamp();
     m_total += data.length();
-    DataSource::Forward(data,tStamp,flags);
+    return DataSource::Forward(data,tStamp,flags);
 }
 
 /**
@@ -2038,15 +2051,28 @@ bool YIAXConnection::route(bool authenticated)
 // Retrieve the data consumer from IAXFormat media type
 YIAXConsumer* YIAXConnection::getConsumer(int type)
 {
-    const char* name = dataEpName(type);
-    return name ? YOBJECT(YIAXConsumer,Channel::getConsumer(name)) : 0;
+    const String& name = dataEpName(type);
+    return YOBJECT(YIAXConsumer,Channel::getConsumer(name));
 }
 
 // Retrieve the data source from IAXFormat media type
 YIAXSource* YIAXConnection::getSource(int type)
 {
-    const char* name = dataEpName(type);
-    return name ? YOBJECT(YIAXSource,Channel::getSource(name)) : 0;
+    const String& name = dataEpName(type);
+    return YOBJECT(YIAXSource,Channel::getSource(name));
+}
+
+DataSource* YIAXConnection::getSourceMedia(int type)
+{
+    const String& name = dataEpName(type);
+    DataSource* src = Channel::getSource(name);
+    if (m_sources[type] != src) {
+	m_sources[type] = 0;
+	DataEndpoint::commonMutex().lock();
+	m_sources[type] = Channel::getSource(name);
+	DataEndpoint::commonMutex().unlock();
+    }
+    return m_sources[type];
 }
 
 // Create audio source with the proper format
@@ -2078,12 +2104,12 @@ void YIAXConnection::startMedia(bool in, int type)
 	exists = (cons != 0);
     }
     clearMedia(in,type);
-    const char* epName = dataEpName(type);
+    const String& epName = dataEpName(type);
     const char* formatText = format ? lookupFormat(format,type) : 0;
     const char* dir = in ? "incoming" : "outgoing";
     if (exists || format)
 	Debug(this,DebugAll,"Starting %s media '%s' format '%s' (%u) [%p]",
-	    dir,epName,formatText,format,this);
+	    dir,epName.c_str(),formatText,format,this);
     bool ok = true;
     if (in) {
 	YIAXSource* src = 0;
@@ -2103,14 +2129,14 @@ void YIAXConnection::startMedia(bool in, int type)
     }
     if (!ok)
 	Debug(this,DebugNote,"Failed to start %s media '%s' format '%s' (%u) [%p]",
-	    dir,epName,formatText,format,this);
+	    dir,epName.c_str(),formatText,format,this);
 }
 
 void YIAXConnection::clearMedia(bool in, int type)
 {
-    const char* epName = dataEpName(type);
+    const String& epName = dataEpName(type);
     DDebug(this,DebugAll,"Clearing %s %s media [%p]",
-	in ? "incoming" : "outgoing",epName,this);
+	in ? "incoming" : "outgoing",epName.c_str(),this);
     if (in)
 	setSource(0,epName);
     else
