@@ -96,11 +96,13 @@ static bool s_autoaddr  = true;
 static bool s_anyssrc   = false;
 static bool s_warnFirst = false;
 static bool s_warnLater = false;
+static bool s_monitor   = false;
 static bool s_rtcp  = true;
 static bool s_drill = false;
 
 static Thread::Priority s_priority = Thread::Normal;
 static int s_tos     = 0;
+static int s_udpbuf  = 0;
 static int s_sleep   = 5;
 static int s_interval= 0;
 static int s_timeout = 0;
@@ -204,7 +206,7 @@ class YRTPSession : public RTPSession
 {
 public:
     inline YRTPSession(YRTPWrapper* wrap)
-	: m_wrap(wrap), m_lastLost(0),
+	: m_wrap(wrap), m_lastLost(0), m_newPayload(-1),
 	  m_resync(false), m_anyssrc(false), m_getFax(true)
 	{ }
     virtual ~YRTPSession();
@@ -225,6 +227,7 @@ protected:
 private:
     YRTPWrapper* m_wrap;
     u_int32_t m_lastLost;
+    int m_newPayload;
     bool m_resync;
     bool m_anyssrc;
     bool m_getFax;
@@ -348,27 +351,6 @@ private:
     Cipher* m_cipher;
 };
 
-class AttachHandler : public MessageHandler
-{
-public:
-    AttachHandler() : MessageHandler("chan.attach") { }
-    virtual bool received(Message &msg);
-};
-
-class RtpHandler : public MessageHandler
-{
-public:
-    RtpHandler() : MessageHandler("chan.rtp") { }
-    virtual bool received(Message &msg);
-};
-
-class DTMFHandler : public MessageHandler
-{
-public:
-    DTMFHandler() : MessageHandler("chan.dtmf",150) { }
-    virtual bool received(Message &msg);
-};
-
 class YRTPPlugin : public Module
 {
 public:
@@ -378,9 +360,7 @@ public:
     virtual bool received(Message& msg, int id);
     virtual void statusParams(String& str);
     virtual void statusDetail(String& str);
-
-    void addStats(const NamedList& stats);
-    void genUpdate(Message& msg);
+    virtual void genUpdate(Message& msg);
 
 private:
     bool reflectSetup(Message& msg, const char* id, RTPTransport& rtp, const char* rHost, const char* leg);
@@ -390,10 +370,31 @@ private:
     void reflectAnswer(Message& msg, bool ignore);
     void reflectHangup(Message& msg);
     bool m_first;
-    ObjList m_updates;
 };
 
 static YRTPPlugin splugin;
+
+class AttachHandler : public MessageHandler
+{
+public:
+    AttachHandler() : MessageHandler("chan.attach",100,splugin.name()) { }
+    virtual bool received(Message &msg);
+};
+
+class RtpHandler : public MessageHandler
+{
+public:
+    RtpHandler() : MessageHandler("chan.rtp",100,splugin.name()) { }
+    virtual bool received(Message &msg);
+};
+
+class DTMFHandler : public MessageHandler
+{
+public:
+    DTMFHandler() : MessageHandler("chan.dtmf",150,splugin.name()) { }
+    virtual bool received(Message &msg);
+};
+
 static ObjList s_calls;
 static ObjList s_mirrors;
 static Mutex s_mutex(false,"YRTPChan");
@@ -433,6 +434,7 @@ YRTPWrapper::YRTPWrapper(const char* localip, CallEndpoint* conn, const char* me
     }
     else
 	setupRTP(localip,msg.getBoolValue(YSTRING("rtcp"),s_rtcp));
+    splugin.changed();
     s_mutex.unlock();
 }
 
@@ -444,6 +446,14 @@ YRTPWrapper::~YRTPWrapper()
     s_calls.remove(this,false);
     if (m_rtp) {
 	Debug(DebugAll,"Cleaning up RTP %p [%p]",m_rtp,this);
+	if (s_monitor) {
+	    Message* m = new Message("module.update");
+	    m->addParam("module",splugin.name());
+	    m_rtp->getStats(*m);
+	    m->setParam("noaudio",String(m_noAudio));
+	    m->setParam("lostaudio",String(m_lostAudio));
+	    Engine::enqueue(m);
+	}
 	TelEngine::destruct(m_rtp);
     }
     if (m_udptl) {
@@ -458,6 +468,7 @@ YRTPWrapper::~YRTPWrapper()
 	Debug(&splugin,DebugGoOn,"There is still a RTP consumer %p [%p]",m_consumer,this);
 	TelEngine::destruct(m_consumer);
     }
+    splugin.changed();
     s_mutex.unlock();
 }
 
@@ -634,6 +645,7 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, Message& msg)
     int evpayload = msg.getIntValue(YSTRING("evpayload"),101);
     const char* format = msg.getValue(YSTRING("format"));
     int tos = msg.getIntValue(YSTRING("tos"),dict_tos,s_tos);
+    int buflen = msg.getIntValue(YSTRING("buffer"),s_udpbuf);
     int msec = msg.getIntValue(YSTRING("msleep"),s_sleep);
 
     if (!format)
@@ -715,6 +727,8 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, Message& msg)
     m_rtp->dataPayload(payload);
     m_rtp->eventPayload(evpayload);
     m_rtp->setTOS(tos);
+    if (buflen > 0)
+	m_rtp->setBuffer(buflen);
     m_rtp->padding(msg.getIntValue(YSTRING("padding"),s_padding));
     if (msg.getBoolValue(YSTRING("drillhole"),s_drill)) {
 	bool ok = m_rtp->drillHole();
@@ -777,7 +791,7 @@ bool YRTPWrapper::setupSRTP(Message& msg, bool buildMaster)
 	if (srtp)
 	    srtp = new RTPSecure(*srtp);
 	else
-	    srtp = new RTPSecure(msg.getValue(YSTRING("crypto_suite")));
+	    srtp = new RTPSecure(msg[YSTRING("crypto_suite")]);
     }
     else
 	buildMaster = false;
@@ -924,15 +938,8 @@ void YRTPWrapper::terminate(Message& msg)
 {
     Debug(&splugin,DebugInfo,"YRTPWrapper::terminate() [%p]",this);
     String stats;
-    if (m_rtp) {
-	NamedList nl("");
-	m_rtp->getStats(nl);
-	nl.setParam("noaudio",String(m_noAudio));
-	nl.setParam("lostaudio",String(m_lostAudio));
-    	splugin.addStats(nl);
-    	splugin.changed();
+    if (m_rtp)
 	m_rtp->getStats(stats);
-    }
     if (m_udptl)
 	m_udptl->getStats(stats);
     if (stats)
@@ -1021,6 +1028,10 @@ void YRTPSession::rtpNewPayload(int payload, unsigned int timestamp)
 	Debug(&splugin,DebugInfo,"Activating RTP silence payload %d in wrapper %p",payload,m_wrap);
 	silencePayload(payload);
     }
+    else if (payload != m_newPayload) {
+	m_newPayload = payload;
+	Debug(&splugin,DebugMild,"Unexpected payload %d in wrapper %p",payload,m_wrap);
+    }
 }
 
 void YRTPSession::rtpNewSSRC(u_int32_t newSsrc, bool marker)
@@ -1303,6 +1314,9 @@ YRTPReflector::~YRTPReflector()
     TelEngine::destruct(m_rtpB);
     TelEngine::destruct(m_monA);
     TelEngine::destruct(m_monB);
+    s_mutex.lock();
+    splugin.changed();
+    s_mutex.unlock();
 }
 
 
@@ -1332,7 +1346,7 @@ bool AttachHandler::received(Message &msg)
 	return false;
 
     const char* media = msg.getValue(YSTRING("media"),"audio");
-    String rip(msg.getValue(YSTRING("remoteip")));
+    const String& rip = msg[YSTRING("remoteip")];
     CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (!ch) {
 	if (!src.null())
@@ -1344,7 +1358,7 @@ bool AttachHandler::received(Message &msg)
 
     RefPointer<YRTPWrapper> w = YRTPWrapper::find(ch,media);
     if (!w)
-	w = YRTPWrapper::find(msg.getValue(YSTRING("rtpid")));
+	w = YRTPWrapper::find(msg[YSTRING("rtpid")]);
     if (!w) {
 	String lip(msg.getValue(YSTRING("localip")));
 	if (lip.null())
@@ -1383,7 +1397,7 @@ bool AttachHandler::received(Message &msg)
 bool RtpHandler::received(Message &msg)
 {
     bool udptl = false;
-    String trans = msg.getValue(YSTRING("transport"));
+    const String& trans = msg[YSTRING("transport")];
     if (trans && !trans.startsWith("RTP/")) {
 	if (trans &= "udptl")
 	    udptl = true;
@@ -1392,7 +1406,7 @@ bool RtpHandler::received(Message &msg)
     }
     Debug(&splugin,DebugAll,"%s message received",(trans ? trans.c_str() : "No-transport"));
     bool terminate = msg.getBoolValue(YSTRING("terminate"),false);
-    String dir(msg.getValue(YSTRING("direction")));
+    const String& dir = msg[YSTRING("direction")];
     RTPSession::Direction direction = terminate ? RTPSession::FullStop : RTPSession::SendRecv;
     bool d_recv = false;
     bool d_send = false;
@@ -1417,16 +1431,21 @@ bool RtpHandler::received(Message &msg)
     if (w)
 	Debug(&splugin,DebugAll,"Wrapper %p found by CallEndpoint %p",(YRTPWrapper*)w,ch);
     else {
-	const char* rid = msg.getValue(YSTRING("rtpid"));
+	const String& rid = msg[YSTRING("rtpid")];
 	w = YRTPWrapper::find(rid);
 	if (w)
-	    Debug(&splugin,DebugAll,"Wrapper %p found by ID '%s'",(YRTPWrapper*)w,rid);
+	    Debug(&splugin,DebugAll,"Wrapper %p found by ID '%s'",(YRTPWrapper*)w,rid.c_str());
     }
     if (w)
 	w->deref();
     if (terminate) {
 	if (w) {
+	    if (w->host())
+		msg.setParam("localip",w->host());
+	    if (w->port())
+		msg.setParam("localport",String(w->port()));
 	    w->terminate(msg);
+	    msg.setParam("status","terminated");
 	    return true;
 	}
 	return false;
@@ -1436,7 +1455,8 @@ bool RtpHandler::received(Message &msg)
 	return false;
     }
 
-    String rip(msg.getValue(YSTRING("remoteip")));
+    const String& rip = msg[YSTRING("remoteip")];
+    const char* status = "updated";
 
     if (!w) {
 	// it would be pointless to create an unreferenced wrapper
@@ -1450,6 +1470,7 @@ bool RtpHandler::received(Message &msg)
 	    return false;
 	}
 
+	status = "created";
 	w = new YRTPWrapper(lip,ch,media,direction,msg,udptl);
 	w->setMaster(msg.getValue(YSTRING("id")));
 
@@ -1494,6 +1515,7 @@ bool RtpHandler::received(Message &msg)
     msg.setParam("localip",w->host());
     msg.setParam("localport",String(w->port()));
     msg.setParam("rtpid",w->id());
+    msg.setParam("status",status);
 
     if (msg.getBoolValue(YSTRING("getsession"),!msg.userData()))
 	msg.userData(w);
@@ -1503,10 +1525,10 @@ bool RtpHandler::received(Message &msg)
 
 bool DTMFHandler::received(Message &msg)
 {
-    String targetid(msg.getValue(YSTRING("targetid")));
+    const String& targetid = msg[YSTRING("targetid")];
     if (targetid.null())
 	return false;
-    String text(msg.getValue(YSTRING("text")));
+    const String& text = msg[YSTRING("text")];
     if (text.null())
 	return false;
     RefPointer<YRTPWrapper> wrap = YRTPWrapper::find(targetid);
@@ -1537,23 +1559,14 @@ YRTPPlugin::~YRTPPlugin()
     s_mirrors.clear();
 }
 
-void YRTPPlugin::addStats(const NamedList& stats)
-{
-    DDebug(this,DebugAll,"Add updates");
-    s_mutex.lock();
-    m_updates.append(new NamedList(stats));
-    s_mutex.unlock();
-}
-
 void YRTPPlugin::genUpdate(Message& msg)
 {
-    DDebug(this,DebugAll,"::genUpdate() [%p] updates=%d",this,m_updates.count());
-    Lock l(s_mutex);
-    if (m_updates.count() == 0)
-	return;
-    NamedList* updateParams = static_cast<NamedList*>(m_updates.remove(false));
-    msg.copyParams(*updateParams);
-    TelEngine::destruct(updateParams);
+    s_mutex.lock();
+    msg.setParam("chans",String(s_calls.count()));
+    s_mutex.unlock();
+    s_refMutex.lock();
+    msg.setParam("mirrors",String(s_mirrors.count()));
+    s_refMutex.unlock();
 }
 
 void YRTPPlugin::statusParams(String& str)
@@ -1702,6 +1715,9 @@ void YRTPPlugin::reflectExecute(Message& msg)
     s_refMutex.lock();
     s_mirrors.append(r);
     s_refMutex.unlock();
+    s_mutex.lock();
+    changed();
+    s_mutex.unlock();
 }
 
 void YRTPPlugin::reflectAnswer(Message& msg, bool ignore)
@@ -1836,6 +1852,7 @@ void YRTPPlugin::initialize()
     s_minJitter = cfg.getIntValue("general","minjitter",50);
     s_maxJitter = cfg.getIntValue("general","maxjitter",Engine::clientMode() ? 120 : 0);
     s_tos = cfg.getIntValue("general","tos",dict_tos);
+    s_udpbuf = cfg.getIntValue("general","udpbuf",0);
     s_localip = cfg.getValue("general","localip");
     s_autoaddr = cfg.getBoolValue("general","autoaddr",true);
     s_anyssrc = cfg.getBoolValue("general","anyssrc",true);
@@ -1843,6 +1860,7 @@ void YRTPPlugin::initialize()
     s_rtcp = cfg.getBoolValue("general","rtcp",true);
     s_interval = cfg.getIntValue("general","rtcp_interval",4500);
     s_drill = cfg.getBoolValue("general","drillhole",Engine::clientMode());
+    s_monitor = cfg.getBoolValue("general","monitoring",false);
     s_sleep = cfg.getIntValue("general","defsleep",5);
     RTPGroup::setMinSleep(cfg.getIntValue("general","minsleep"));
     s_priority = Thread::priority(cfg.getValue("general","thread"));
