@@ -213,8 +213,78 @@ private:
     ObjList m_sections;
 };
 
+/**
+ * Class that implements atomic / locked access and operations to its shared variables
+ * @short Atomic access and operations to shared variables
+ */
+class YATE_API SharedVars : public Mutex
+{
+public:
+    /**
+     * Constructor
+     */
+    inline SharedVars()
+	: Mutex(false,"SharedVars"), m_vars("")
+	{ }
+
+    /**
+     * Get the string value of a variable
+     * @param name Name of the variable
+     * @param rval String to return the value into
+     */
+    void get(const String& name, String& rval);
+
+    /**
+     * Set the string value of a variable
+     * @param name Name of the variable to set
+     * @param val New value to assign to a variable
+     */
+    void set(const String& name, const char* val);
+
+    /**
+     * Create and set a variable only if the variable is not already set
+     * @param name Name of the variable to set
+     * @param val New value to assign to a variable
+     * @return True if a new variable was created
+     */
+    bool create(const String& name, const char* val = 0);
+
+    /**
+     * Clear a variable
+     * @param name Name of the variable to clear
+     */
+    void clear(const String& name);
+
+    /**
+     * Check if a variable exists
+     * @param name Name of the variable
+     * @return True if the variable exists
+     */
+    bool exists(const String& name);
+
+    /**
+     * Atomically increment a variable as unsigned integer
+     * @param name Name of the variable
+     * @param wrap Value to wrap around at, zero disables
+     * @return Value of the variable before increment, zero if it was not defined or not numeric
+     */
+    unsigned int inc(const String& name, unsigned int wrap = 0);
+
+    /**
+     * Atomically decrement a variable as unsigned integer
+     * @param name Name of the variable
+     * @param wrap Value to wrap around at, zero disables (stucks at zero)
+     * @return Value of the variable after decrement, zero if it was not defined or not numeric
+     */
+    unsigned int dec(const String& name, unsigned int wrap = 0);
+
+private:
+    NamedList m_vars;
+};
+
 class MessageDispatcher;
 class MessageRelay;
+class Engine;
 
 /**
  * This class holds the messages that are moved around in the engine.
@@ -406,8 +476,11 @@ public:
      * Creates a new message handler.
      * @param name Name of the handled message - may be NULL
      * @param priority Priority of the handler, 0 = top
+     * @param trackName Name to be used in handler tracking
+     * @param addPriority True to append :priority to trackName
      */
-    explicit MessageHandler(const char* name, unsigned priority = 100);
+    explicit MessageHandler(const char* name, unsigned priority = 100,
+	const char* trackName = 0, bool addPriority = true);
 
     /**
      * Handler destructor.
@@ -433,6 +506,21 @@ public:
     inline unsigned priority() const
 	{ return m_priority; }
 
+    /**
+     * Retrieve the tracking name of this handler
+     * @return Name that is to be used in tracking operation
+     */
+    inline const String& trackName() const
+	{ return m_trackName; }
+
+    /**
+     * Set a new tracking name for this handler.
+     * Works only if the handler was not yet inserted into a dispatcher
+     * @param name Name that is to be used in tracking operation
+     */
+    inline void trackName(const char* name)
+	{ if (!m_dispatcher) m_trackName = name; }
+
     /**
      * Retrieve the filter (if installed) associated to this handler
      */
@@ -469,6 +557,7 @@ protected:
 private:
     virtual bool receivedInternal(Message& msg);
     void safeNow();
+    String m_trackName;
     unsigned m_priority;
     int m_unsafe;
     MessageDispatcher* m_dispatcher;
@@ -505,9 +594,14 @@ public:
      * @param receiver Receiver of th relayed messages
      * @param id Numeric identifier to pass to receiver
      * @param priority Priority of the handler, 0 = top
+     * @param trackName Name to be used in handler tracking
+     * @param addPriority True to append :priority to trackName
      */
-    MessageRelay(const char* name, MessageReceiver* receiver, int id, int priority = 100)
-	: MessageHandler(name,priority), m_receiver(receiver), m_id(id) { }
+    inline MessageRelay(const char* name, MessageReceiver* receiver, int id,
+	int priority = 100, const char* trackName = 0, bool addPriority = true)
+	: MessageHandler(name,priority,trackName,addPriority),
+	  m_receiver(receiver), m_id(id)
+	{ }
 
     /**
      * This method is not called from MessageHandler through polymorphism
@@ -571,18 +665,26 @@ class YATE_API MessagePostHook : public GenObject, public MessageNotifier
  */
 class YATE_API MessageDispatcher : public GenObject, public Mutex
 {
+    friend class Engine;
     YNOCOPY(MessageDispatcher); // no automatic copies please
 public:
     /**
      * Creates a new message dispatcher.
+     * @param trackParam Name of the parameter used in tracking handlers
      */
-    MessageDispatcher();
+    MessageDispatcher(const char* trackParam = 0);
 
     /**
      * Destroys the dispatcher and the installed handlers.
      */
     ~MessageDispatcher();
 
+    /**
+     * Retrieve the tracker parameter name
+     */
+    inline const String& trackParam() const
+	{ return m_trackParam; }
+
     /**
      * Installs a handler in the dispatcher.
      * The handlers are installed in ascending order of their priorities.
@@ -663,10 +765,19 @@ public:
      */
     void setHook(MessagePostHook* hook, bool remove = false);
 
+protected:
+    /**
+     * Set the tracked parameter name
+     * @param paramName Name of the parameter used in tracking handlers
+     */
+    inline void trackParam(const char* paramName)
+	{ m_trackParam = paramName; }
+
 private:
     ObjList m_handlers;
     ObjList m_messages;
     ObjList m_hooks;
+    String m_trackParam;
     unsigned int m_changes;
     u_int64_t m_warnTime;
 };
@@ -1170,6 +1281,12 @@ public:
      */
     static void clearEvents(const String& type);
 
+    /**
+     * Access the engine's shared variables
+     * @return Reference to the static variables shared between modules
+     */
+    static SharedVars& sharedVars();
+
 protected:
     /**
      * Destroys the engine and everything. You must not call it directly,
