@@ -1210,48 +1210,48 @@ public:
      * @param type Type of data endpoint: "audio", "video", "text"
      * @return A pointer to the DataEndpoint object or NULL if not found
      */
-    DataEndpoint* getEndpoint(const char* type = "audio") const;
+    DataEndpoint* getEndpoint(const String& type = CallEndpoint::audioType()) const;
 
     /**
      * Get a data endpoint of this object, create if required
      * @param type Type of data endpoint: "audio", "video", "text"
      * @return A pointer to the DataEndpoint object or NULL if an error occured
      */
-    DataEndpoint* setEndpoint(const char* type = "audio");
+    DataEndpoint* setEndpoint(const String& type = CallEndpoint::audioType());
 
     /**
      * Clear one or all data endpoints of this object
      * @param type Type of data endpoint: "audio", "video", "text", NULL to clear all
      */
-    void clearEndpoint(const char* type = 0);
+    void clearEndpoint(const String& type = String::empty());
 
     /**
      * Set a data source of this object
      * @param source A pointer to the new source or NULL
      * @param type Type of data node: "audio", "video", "text"
      */
-    void setSource(DataSource* source = 0, const char* type = "audio");
+    void setSource(DataSource* source = 0, const String& type = CallEndpoint::audioType());
 
     /**
      * Get a data source of this object
      * @param type Type of data node: "audio", "video", "text"
      * @return A pointer to the DataSource object or NULL
      */
-    DataSource* getSource(const char* type = "audio") const;
+    DataSource* getSource(const String& type = CallEndpoint::audioType()) const;
 
     /**
      * Set the data consumer of this object
      * @param consumer A pointer to the new consumer or NULL
      * @param type Type of data node: "audio", "video", "text"
      */
-    void setConsumer(DataConsumer* consumer = 0, const char* type = "audio");
+    void setConsumer(DataConsumer* consumer = 0, const String& type = CallEndpoint::audioType());
 
     /**
      * Get the data consumer of this object
      * @param type Type of data node: "audio", "video", "text"
      * @return A pointer to the DataConsumer object or NULL
      */
-    DataConsumer* getConsumer(const char* type = "audio") const;
+    DataConsumer* getConsumer(const String& type = CallEndpoint::audioType()) const;
 
     /**
      * Clear a data node from any slot of a DataEndpoint of this object
@@ -1259,7 +1259,13 @@ public:
      * @param type Type of data node: "audio", "video", "text"
      * @return True if the node was removed from at least one slot
      */
-    bool clearData(DataNode* node, const char* type = "audio");
+    bool clearData(DataNode* node, const String& type = CallEndpoint::audioType());
+    
+    /**
+     * Return the defaul audio type "audio"
+     * @return Return a string naming the "audio" type
+     */
+    static const String& audioType();
 
 protected:
     /**
@@ -1625,6 +1631,7 @@ private:
     u_int64_t m_timeout;
     u_int64_t m_maxcall;
     u_int64_t m_dtmfTime;
+    unsigned int m_toutAns;
     unsigned int m_dtmfSeq;
     String m_dtmfText;
     String m_dtmfDetected;
@@ -1904,15 +1911,17 @@ public:
     /**
      * Set the time this channel will time out on outgoing calls
      * @param msg Reference of message possibly holding "maxcall" parameter
+     * @param defTout Default timeout to apply, negative to not alter
      */
-    inline void setMaxcall(const Message& msg)
-	{ setMaxcall(&msg); }
+    inline void setMaxcall(const Message& msg, int defTout = -1)
+	{ setMaxcall(&msg,defTout); }
 
     /**
      * Set the time this channel will time out on outgoing calls
      * @param msg Pointer to message possibly holding "maxcall" parameter
+     * @param defTout Default timeout to apply, negative to not alter
      */
-    void setMaxcall(const Message* msg);
+    void setMaxcall(const Message* msg, int defTout = -1);
 
     /**
      * Get the connected channel identifier.
@@ -2127,6 +2136,7 @@ private:
     int m_timeout;
     int m_maxroute;
     int m_maxchans;
+    int m_chanCount;
     bool m_dtmfDups;
 
 public:
@@ -2231,6 +2241,20 @@ public:
     inline int total() const
 	{ return m_total; }
 
+    /**
+     * Get the number of running channels
+     * @return Number of channels running at this time
+     */
+    inline int chanCount() const
+	{ return m_chanCount; }
+
+    /**
+     * Get the maximum number of running channels for this driver
+     * @return Maximum number of calls to run simultaneously, zero to accept all
+     */
+    inline int maxChans() const
+	{ return m_maxchans; }
+
 protected:
     /**
      * Constructor
